@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -249,7 +249,8 @@ public class ColumnCalculator {
         Iterator<Obj> it = p.iterator();
         while( it.hasNext() ) {
            Obj o = it.next();
-           if( !(o instanceof Source) ) continue;
+//           if( !(o instanceof Source) ) continue;
+           if( !o.asSource() ) continue;
            Source s = (Source)o;
            if( s!=null ) vec.addElement(s);
         }
