@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -411,6 +411,10 @@ public class DirectoryTree extends JTree {
          mapIcon.put("sdss.jhu",new MyImageIcon(img));
          mapIcon.put("sdss.jhu/color",new MyImageIcon(setIconTag(img)));
         
+         img = aladin.getImagette("astron.png");
+         mapIcon.put("astron.nl",new MyImageIcon(img));
+         mapIcon.put("hips.astron.nl",new MyImageIcon(img));
+
          mapIcon.put("f",new MyImageIcon(img));
          
          mapIcon.put("defaut",new MyImageIcon());
@@ -473,6 +477,8 @@ public class DirectoryTree extends JTree {
          textBackground = UIManager.getColor("Tree.textBackground");
          
          nonLeafRenderer.setBackgroundNonSelectionColor( getBackground() );
+         
+         setFont( getFont().deriveFont( getFont().getSize2D()* Aladin.getUIScale() ));
       }
 
       public Component getTreeCellRendererComponent(JTree tree, Object obj, boolean selected, boolean expanded,
