@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -237,7 +237,7 @@ public class FrameHeaderFits extends JFrame {
       headerFits = new HeaderFits(s,this);
       if( plan!=null ) {
          Calib c = new Calib(headerFits);
-         plan.projd = new Projection(Projection.WCS, c);
+         plan.projd = new Projection(Projection.WCS, c, plan);
          plan.setHasSpecificCalib();
          plan.aladin.view.newView(1);
          plan.aladin.view.repaintAll();
