@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -151,7 +151,8 @@ public final class FrameMocFiltering extends FrameRGBBlink {
          Iterator<Obj> it = pCat.iterator();
          while( it.hasNext() ) {
             Obj o = it.next();
-            if( !(o instanceof Source) ) continue;
+//            if( !(o instanceof Source) ) continue;
+            if( !o.asSource() ) continue;
             
             long npixSpace=-1;
             boolean in=false;
@@ -162,7 +163,7 @@ public final class FrameMocFiltering extends FrameRGBBlink {
                c.del=o.getDec();
                if( Double.isNaN(c.al) ||  Double.isNaN(c.del) ) continue;
                c=Localisation.frameToFrame(c,Localisation.ICRS, pMoc.frameOrigin);
-               npixSpace = hpx.ang2pix(Moc.MAXORDER, c.al, c.del);
+               npixSpace = hpx.ang2pix(SMoc.MAXORD_S, c.al, c.del);
 
                in = spaceMoc.contains( npixSpace);
                if( in!=lookIn ) continue;
