@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -21,25 +21,51 @@
 
 package cds.aladin;
 
-import cds.aladin.prop.PropPanel;
-import cds.astro.*;
-import cds.fits.HeaderFits;
-import cds.tools.Util;
-
-import java.awt.*;
+import java.awt.AWTEvent;
+import java.awt.BorderLayout;
+import java.awt.Color;
+import java.awt.Component;
+import java.awt.Dimension;
+import java.awt.FlowLayout;
+import java.awt.Font;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
+import java.awt.GridLayout;
+import java.awt.Insets;
+import java.awt.Label;
+import java.awt.Point;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
 import java.awt.event.WindowEvent;
-import java.text.ParseException;
-import java.util.*;
+import java.util.Enumeration;
+import java.util.Stack;
+import java.util.StringTokenizer;
+import java.util.Vector;
 
-import javax.swing.*;
+import javax.swing.BorderFactory;
+import javax.swing.ButtonGroup;
+import javax.swing.JButton;
+import javax.swing.JComboBox;
+import javax.swing.JFrame;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JRadioButton;
+import javax.swing.JScrollPane;
+import javax.swing.JTabbedPane;
+import javax.swing.JTextArea;
+import javax.swing.JTextField;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 
+import cds.aladin.prop.PropPanel;
+import cds.astro.AstroMath;
+import cds.astro.Astrocoo;
+import cds.astro.Astroframe;
+import cds.fits.HeaderFits;
+import cds.tools.Util;
+
 /**
  * Gestion de la fenetre associee a la creation d'une calib manuelle
  *
@@ -320,7 +346,7 @@ public final class FrameNewCalib extends JFrame
 
       // Dans le cas ou l'on veut editer le WCS
       if( proj==null ) {
-         getWCS(new Projection(label,Projection.SIMPLE,raj,dej,rm,rm1,cx,cy,r,r1,rot,sym,Calib.TAN,system));
+         getWCS(new Projection(label,Projection.SIMPLE,raj,dej,rm,rm1,cx,cy,r,r1,rot,sym,Calib.TAN,system,plan));
       } else getWCS(proj);
 
       pack();
@@ -744,7 +770,8 @@ e.printStackTrace();
          // Methode par parametres
          if( modeCalib==SIMPLE ) {
             error="coordinate";
-            Astrocoo c = new Astrocoo(new ICRS(),cooT.getText());
+            Astrocoo c = new Astrocoo( Astroframe.create("ICRS"),cooT.getText());
+//            Astrocoo c = new Astrocoo(new ICRS(),cooT.getText());
             double raj=c.getLon();
             double dej=c.getLat();
             StringTokenizer st = new StringTokenizer(xyT.getText());
@@ -781,7 +808,7 @@ e.printStackTrace();
                p=oldp;
                p.modify(label,Projection.SIMPLE,raj,dej,rm,rm1,cx,cy,r,r1,rot,sym,type,system);
             } else {
-               p = new Projection(label,Projection.SIMPLE,raj,dej,rm,rm1,cx,cy,r,r1,rot,sym,type,system);
+               p = new Projection(label,Projection.SIMPLE,raj,dej,rm,rm1,cx,cy,r,r1,rot,sym,type,system,plan);
             }
 
          // Methode par quadruplets
@@ -793,7 +820,7 @@ e.printStackTrace();
                if( plan.projd!=null ) p = plan.projd.copy();
                else {
                   p=new Projection(label,Projection.SIMPLE,
-                        coo[0].al,coo[0].del,cx,cy,coo[0].x,coo[0].y,cx*2,cy*2,0,false,Calib.TAN,system);
+                        coo[0].al,coo[0].del,cx,cy,coo[0].x,coo[0].y,cx*2,cy*2,0,false,Calib.TAN,system,plan);
                }
             }
             p.modify(label,coo);
@@ -804,7 +831,7 @@ e.printStackTrace();
             HeaderFits headerFits = new HeaderFits(s);
             Calib c = new Calib(headerFits);
             if( flagModif ) p=oldp;
-            else p = new Projection(label,Projection.WCS,c.alphai,c.deltai,/*raj,dej,*/rm,rm1,cx,cy,r,r1,rot,sym,Calib.TAN,system);
+            else p = new Projection(label,Projection.WCS,c.alphai,c.deltai,/*raj,dej,*/rm,rm1,cx,cy,r,r1,rot,sym,Calib.TAN,system,plan);
             p.modify(label,c);
          }
 
