@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -24,7 +24,6 @@ package cds.aladin;
 import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Component;
-import java.awt.Composite;
 import java.awt.FontMetrics;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
@@ -114,6 +113,7 @@ public class HealpixKey implements Comparable<HealpixKey> {
    protected boolean fromNet=true;// true si le losange provient du rseau et non du cache
    protected int timeStream;      // stat de lecture du stream,
    protected int timeNet;         // stat de lecture du stream via le net
+   protected boolean gzipped=false; // true si le flux tait gzipp
    protected long sizeStream;     // Nombre d'octets du stream
    private int timeJPEG, timePixel;  // stat de la cration de l'image, de l'extraction des pixels
    private int typeColor;         // Mode couleur JPEG ou PNG
@@ -240,6 +240,7 @@ public class HealpixKey implements Comparable<HealpixKey> {
       planBG = father.planBG;
       order=father.order+1;
       npix=father.npix*4+child;
+      timerLoad=father.timerLoad;
       hpix = new Hpix(order,npix,planBG.frameOrigin);
       anc=father.anc;
       if( anc==null ) anc=father;
@@ -255,7 +256,7 @@ public class HealpixKey implements Comparable<HealpixKey> {
       allSky=father.allSky;
       setStatus(UNKNOWN);
    }
-
+   
    /** Cration dynamique des 4 losanges fils  partir du losange courant */
    protected HealpixKey [] getChild() {
       if( getStatus()!=READY ) return null;
@@ -325,14 +326,17 @@ public class HealpixKey implements Comparable<HealpixKey> {
       //      long t = (int)(getAskRepaintTime()/1000L);
       return code+"["+Util.align(priority+"",5)+"] "+
       Util.align(getStringNumber()+(fils!=null?">":" "),8)+
-      Util.align(getLongFullMem(),8)+
+      Util.align((sizeStream!=0?Util.getUnitDisk(sizeStream)+"/":"")+getLongFullMem(),12)+
       (truePixels ? " truePix ":"         ")+
       Util.align(getStatusString(),16)+
       ( timer==-1 ? -1 : getCurrentLiveTime()/1000 ) +
       //             "/"+t + "s => "+VIE[-getLive()]+
       "s => "+VIE[-getLive()]+
-      (getStatus()==READY?(fromNet?" Net":" Cache")+":"+timeStream+"+"+
-            timeJPEG+"+"+timePixel+"ms" : "");
+      (getStatus()==READY?(fromNet?" Net":" Cache")+":"+(timeNet+timeStream+ timeJPEG+timePixel)+"ms" : "");
+      
+//      (getStatus()==READY?(fromNet?" Net":" Cache")+":"+(timeStream+"+"+
+//            timeJPEG+"+"+timePixel+"ms" : "");
+
    }
 
    //   private String toStringCorners() {
@@ -730,6 +734,20 @@ public class HealpixKey implements Comparable<HealpixKey> {
    }
 
 
+   /** Indique si on veut charger les tuiles distantes en gzip live ou non */
+   
+//   protected boolean askGzip() {
+//      boolean rep=askGzip1();
+//      System.out.println("askgzip for "+order+"/"+npix+" => "+rep);
+//      return rep;
+//   }
+   protected boolean askGzip() {
+      if( Aladin.GZIP==1 ) return true;
+      if( Aladin.GZIP==0 ) return false;
+      int modegzip = planBG.getGzippedMode();
+      if( modegzip==1 || modegzip==-1 ) return true;
+      return false;
+   }
 
    /** Chargement total du fichier en mmoire soit en un coup si fichier local, soit morceau
     * par morceau via le rseau
@@ -740,8 +758,8 @@ public class HealpixKey implements Comparable<HealpixKey> {
    protected byte [] loadStream(String filename,int skip) throws Exception {
       byte [] buf;
       boolean local=true;
-      long t1 = Util.getTime();
       planBG.aladin.trace(5,"HealpixKey.loadStream("+filename+")...");
+      long t1 = Util.getTime();
       MyInputStream dis=null;
       boolean fastLoad = this instanceof HealpixAllsky;
 
@@ -749,7 +767,8 @@ public class HealpixKey implements Comparable<HealpixKey> {
       if( filename.startsWith("http://") || filename.startsWith("https://") ) {
          local=false;
          try {
-            dis = Util.openStream(filename,false,true,10000);
+            dis = Util.openStream(filename,false,askGzip(),10000);
+            gzipped = (dis.getType()& MyInputStream.GZ)!=0;
             if( skip>0 ) dis.skip(skip);
             buf = readFully(dis, fastLoad );
             
@@ -770,6 +789,7 @@ public class HealpixKey implements Comparable<HealpixKey> {
             detectTypeColor(c);
 
             if( (c[0] & 0xFF)==31 && (c[1] & 0xFF)==139 ) {
+               gzipped = true;
                //                Aladin.trace(4,"HealpixKey.loadStream: "+filename+" gzipped => reading by MyInputStream rather than RandomAccessFile");
                FileInputStream fgz = new FileInputStream(new File(filename));
                dis = new MyInputStream( new GZIPInputStream(fgz) );
@@ -791,7 +811,11 @@ public class HealpixKey implements Comparable<HealpixKey> {
       // Stats de lecture
       int t = (int)(Util.getTime()-t1);
       if( local ) timeStream = t;
-      else timeNet = t;
+      else {
+         timeNet = t;
+         planBG.incrGzipMode( gzipped ? 1 : 2, t);
+      }
+      sizeStream = buf.length;
 
       return buf;
    }
@@ -876,6 +900,32 @@ public class HealpixKey implements Comparable<HealpixKey> {
       }
       throw new Exception();
    }
+   
+   /** Retourne le pixel (RGB) d'indice healpixIdxPixel (l'order de rfrence est celui du pixel final) */
+   protected int getPixelRGB(long healpixIdxPixel) {
+      long startIdx =  npix * width * width;
+      int order = (int)CDSHealpix.log2(width);
+      if( planBG.hpx2xy == null || planBG.hpx2xy.length!=width*width ) {
+         try { planBG.createHealpixOrder(order); } catch( Exception e ) { return 0; }
+      }
+      int idx = planBG.hpx2xy((int)(healpixIdxPixel-startIdx));
+      int x = idx%width;
+      int y = width - idx/width -1;
+      return rgb[ y*width+x ];
+   }
+
+   /** Retourne le pixel (byte) d'indice healpixIdxPixel (l'order de rfrence est celui du pixel final) */
+   protected byte getPixelByte(long healpixIdxPixel) {
+      long startIdx =  npix * width * width;
+      int order = (int)CDSHealpix.log2(width);
+      if( planBG.hpx2xy == null || planBG.hpx2xy.length!=width*width ) {
+         try { planBG.createHealpixOrder(order); } catch( Exception e ) { return 0; }
+      }
+      int idx = planBG.hpx2xy((int)(healpixIdxPixel-startIdx));
+      int x = idx%width;
+      int y = width - idx/width -1;
+      return pixels[ y*width+x ];
+   }
 
    /** Retourne le pixel (true bits) d'indice healpixIdxPixel (l'order de rfrence est celui du pixel final)
     * @param mode NOW, ONLYIFAVAIL, ONLYIFLOCALAVAIL - prt  attendre le chargement des donnes ou pas
@@ -1152,9 +1202,7 @@ public class HealpixKey implements Comparable<HealpixKey> {
                tmp.pixels = in;
                if( planBG.flagRecutRadius>0 && w!=width) {
                   try { tmp.pixels = getSample(planBG.flagRecutCoo, w, in, width); }
-                  catch( Exception e ) {
-                     if( planBG.aladin.levelTrace>=3 ) e.printStackTrace();
-                  }
+                  catch( Exception e ) { }
                }
                double [] range = tmp.findAutocutRange(0,0,true);
 
@@ -1673,11 +1721,9 @@ public class HealpixKey implements Comparable<HealpixKey> {
       return mem;
    }
 
-   /** Retourne la taille du losange et de tous ses fytes dans une unit comprhensible */
+   /** Retourne la taille du losange et de tous ses bytes dans une unit comprhensible */
    protected String getLongFullMem() {
-      int n = getFullMem()/1024;
-      if( n>1024 ) return n/1024+"MB";
-      return n+"KB";
+      return Util.getUnitDisk( getFullMem() );
    }
 
    /** Retourne true si le losange dcrit par ses quatres coins se trouve
@@ -1710,6 +1756,8 @@ public class HealpixKey implements Comparable<HealpixKey> {
    }
    
    protected boolean isTooLarge(PointD b[], int N) throws Exception {
+//      if( planBG.maxOrder>=order && order>3 ) return false;
+      
       if( planBG.aladin.isAnimated() ) N *=6;
       N *= N;
       double d1,d2;
@@ -1724,6 +1772,13 @@ public class HealpixKey implements Comparable<HealpixKey> {
 
    static protected int nDraw = 0;
    static protected int nOut = 0;
+   
+   protected boolean mayCrossTheSky(ViewSimple v) {
+      Projection proj = v.getProj();
+      if( proj.t==Calib.TAN || proj.t==Calib.SIN || proj.t==Calib.STG ) return false;
+      return v.getTaille(proj)> 40;
+   }
+
 
    /** Trac du losange Healpix.
     * Si le bord d'un losange fait plus de 100 pixels, on trace les 4 fils rcursivement (prof maximum 8);
@@ -1735,13 +1790,6 @@ public class HealpixKey implements Comparable<HealpixKey> {
     * @param maxParente parente maximale autorise pour le trac des fils (longueur de la descendance, -1 si aucune)
     * @return le nombre d'images (java) tracs
     */
-
-   protected boolean mayCrossTheSky(ViewSimple v) {
-      Projection proj = planBG.projd;
-      if( proj.t==Calib.TAN || proj.t==Calib.SIN || proj.t==Calib.STG ) return false;
-      return v.getTaille(proj)> 40;
-   }
-   
    //   protected int draw(Graphics g, ViewSimple v) { return draw(g,v,-1,planBG.redraw); }
    protected int draw(Graphics g, ViewSimple v) { return draw(g,v,-1); }
    protected int draw(Graphics g, ViewSimple v,int maxParente) {
@@ -1769,12 +1817,14 @@ public class HealpixKey implements Comparable<HealpixKey> {
       boolean drawFast = planBG.mustDrawFast();
 //      boolean animated = planBG.aladin.isAnimated();
       
+      Projection proj=v.getProj();
+      
       // On a les 4 coins
       if( b[0]!=null && b[1]!=null && b[2]!=null && b[3]!=null ) {
 
          // En cas de bord du ciel : mthode des triangles (plus rapide que par rcursion),
          // mais ne fonctionne bien que pour les ellipses (AITOFF et MOLDWEIDE)
-         boolean methodeTriangle = drawFast && (planBG.projd.t==Calib.AIT  || planBG.projd.t==Calib.MOL);
+         boolean methodeTriangle = drawFast && (proj.t==Calib.AIT  || proj.t==Calib.MOL);
          if( methodeTriangle && (aDroite(b[0],b[1],b[2])*aDroite(b[3],b[1],b[2])>=0
                || aDroite(b[1],b[0],b[3])*aDroite(b[2],b[0],b[3])>=0) ) {
             double d12=dist(b,1,2);
@@ -1798,32 +1848,39 @@ public class HealpixKey implements Comparable<HealpixKey> {
          } else {
             
             try {
-
                boolean mayCrossTheSky = mayCrossTheSky(v);
                boolean methodeRecursive =  
-                     ( planBG.projd.t==Calib.ZEA || (planBG.projd.t==Calib.ARC ) ||
-                     planBG.projd.t==Calib.MOL || planBG.projd.t==Calib.AIT ) && mayCrossTheSky 
-                     || planBG.projd.t==Calib.CAR;
+                     ( proj.t==Calib.ZEA || (proj.t==Calib.ARC ) ||
+                     proj.t==Calib.MOL || proj.t==Calib.AIT ) && mayCrossTheSky 
+                     || proj.t==Calib.CAR || proj.t==Calib.MER;
                
                // Methode rcursive pour s'approcher du bord du ciel
-               if( methodeRecursive && isTooLarge(b, planBG.projd.t==Calib.ARC ? 100 : 150) ) {
+               if( methodeRecursive && isTooLarge(b, proj.t==Calib.ARC ? 100 : 150) ) {
+
                   resetTimer();
-                  int m = drawFils(g,v,drawFast?1:planBG.projd.t==Calib.ZEA?8:4);
+                  int rec = drawFast?1:proj.t==Calib.ZEA?8:4;
+                  int m = drawFils(g,v,rec);
                   if( m>0 ) return m;   // si aucun fils n'est trac, on tentera le pre
 
-               // Mthode rcursive pour dimiduer les dformations
+                  // Mthode rcursive pour dimiduer les dformations
                } else {
 
                   if( !drawFast && mustBeDivided(b)  ) {
                      resetTimer();
-                     int m = drawFils(g,v,8);
+                     int rec = 8;
+
+                     // Si on ne dispose pas de losange d'ordre >=3, il faudra subdiviser davantage
+                     if( planBG.maxOrder<3 ) rec = rec * (int)CDSHealpix.pow2(3-planBG.maxOrder);
+                     
+                     int m = drawFils(g,v,rec);
                      if( m>0 ) return m;   // si aucun fils n'est trac, on tentera le pre
                   }
                }
 
                // Les losanges trop grands sont simplement ignors
-               if( mayCrossTheSky && isTooLarge(b,150) ) return 0;
-               if( planBG.projd.t==Calib.STG && isTooLarge(b,2*v.rv.width/3) ) return 0;
+               if( mayCrossTheSky && isTooLarge(b,Aladin.TESTV12 && proj.t==Calib.ZEA ? 125 : 150) ) return 0;
+               if( proj.t==Calib.STG && isTooLarge(b,2*v.rv.width/3) ) return 0;
+               if( isTooLarge(b,v.rv.width*2) ) return 0;
 
             } catch( Exception e ) { e.printStackTrace(); return 0; }
 
@@ -1880,12 +1937,10 @@ public class HealpixKey implements Comparable<HealpixKey> {
       try { img=createImage(); }
       catch( Exception e ) {
          e.printStackTrace();
-         return 0; }
+         return 0; 
+      }
 
       Graphics2D g2d = (Graphics2D)g;
-      float opacity = getFadingOpacity();
-      Composite saveComposite = g2d.getComposite();
-      g2d.setComposite( Util.getImageComposite( opacity ) );
       Shape clip = g2d.getClip();
 
       try {
@@ -1897,30 +1952,28 @@ public class HealpixKey implements Comparable<HealpixKey> {
          planBG.clearBuf();
       }
       finally {
-         g2d.setComposite(saveComposite);
          g2d.setClip(clip);
       }
       if( parente>0 ) { pixels=null; rgb=null; }
-      
-      if( opacity<1f ) planBG.updateFading(true);
-
       return n;
    }
 
-   private long loadDate = System.currentTimeMillis();
-
-   static private final long TIMEFADER = 1500;
-   static private final float MINFADER = 0.6f;
+   static private final long TIMEFADER = 600;
+   static private final float MINFADER = 0.1f;
 
    // Pour le moment pas d'animation de fondu-enchain
    // EN FAIT IL FAUDRAIT MELANGER LES PIXELS AVEC LES VALEURS DU LOSANGE PERE
    // PLUTOT QUE DE JOUER SUR L'OPACITY
-   private float getFadingOpacity() {
-      return 1f;
-      //      if( allSky ) return 1f;
-      //      long t = System.currentTimeMillis() - loadDate;
-      //      float op = t>=TIMEFADER ? 1f : MINFADER +( (float)t/TIMEFADER)*(1f-MINFADER);
-      //      return op;
+   // ET D'AUTRE PART, IL FAUDRAIT TRAVAILLER SUR UN BUFFER ADDITIONNEL POUR EVITER
+   // DE VOIR APPARAITRE LES BORDS DES SUBDIVISIONS DES LOSANGES QUI SE RECOUVRENT PARTIELLEMENT
+   // ET POUR FINIR, CA FAIT CHAUFFER LA CARTE GRAPHIQUE - PF JUILLET 2022
+   protected float getFadingOpacity() {
+//      return 1f;      
+            if( allSky || timerLoad==0) return 1f;
+            long t0 = System.currentTimeMillis();
+            long t = t0 - timerLoad;
+            float op = t>=TIMEFADER ? 1f : MINFADER +( (float)t/TIMEFADER)*(1f-MINFADER);
+            return op;
    }
 
    /** Retourne >0 si le point a est " droite" de la droite passant par g et d */
