@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -52,11 +52,15 @@ abstract public class Icon extends JComponent implements
 
    protected Aladin aladin;
    protected int W,H;
+   protected int DX;           // Marge de gauche pour l'icone
 
   /** Creation */
    protected Icon(Aladin aladin,int width,int height) {
-      W=width;
-      H=height;
+      double scale = (Aladin.getUIScale()-1)/1.25 +1;
+      W=(int)( width*scale );
+      H=(int)( height*scale );
+      DX =(int)Math.round( (W-width)/2. );
+      
       this.aladin=aladin;
       addMouseMotionListener(this);
       addMouseListener(this);
@@ -161,7 +165,6 @@ abstract public class Icon extends JComponent implements
       drawLogo(gr);
    }
    
-
    public void mouseDragged(MouseEvent e) { }
    public void mouseClicked(MouseEvent e) { }
    public void mousePressed(MouseEvent e) { }
