@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -36,21 +36,16 @@ public class IconInside extends Icon {
    static String INSIDE;
    
    private boolean activated=false;
-
-  /** Creation */
-   protected IconInside(Aladin aladin) {
-      super(aladin,32,24);
-      INSIDE=  aladin.chaine.getString("INSIDE");
-   }
    
-//   // Barres horizontales du dessin  (un scateur)
-//   static final private int TX[][] = {
-//         {0,2,7},  {0,11,12},
-//         {1,0,8},  {1,10,17},
-//         {2,1,1},  {2,9,10},   {2,13,16},
-//         {3,8,9},  {3,11,15},
-//         {4,0,8},  {4,12,14},
-//         {5,1,7},   };
+   /** Creation */
+   protected IconInside(Aladin aladin) { this(aladin,32,24); }
+      
+   protected IconInside(Aladin aladin, int w, int h) {
+      super(aladin,w,h);
+      INSIDE=  aladin.chaine.getString("INSIDE");
+     
+   }
+
    
    // Barres horizontales du dessin (cadre et une flche pointant  l'intrieur)
    static final private int TX[][] = {
@@ -84,7 +79,7 @@ public class IconInside extends Icon {
   /** Affichage du logo */
    protected void drawLogo(Graphics g) {
       super.drawLogo(g);
-      int x = 10;
+      int x = 10+DX;
       int y = 1;
       boolean isAvailable = isAvailable();
       
