@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -373,7 +373,7 @@ public final class Legende extends AbstractTableModel  {
    /** retourne true s'il s'agit d'une lgende comportant un point de SED */
    protected boolean isSED() {
       for( Field f : field ) {
-         if( f.sed!=0 ) return true;
+         if( f.sed==Field.FLUX ) return true;
       }
       return false;
    }
@@ -454,7 +454,17 @@ public final class Legende extends AbstractTableModel  {
       return -1;
    }
    
-   /** Retourne la desc
+   protected int getAlternatePrecision(int i) {
+      if( i>=field.length ) return -1;
+      return field[i].alternatePrecision;
+   }
+   
+   protected void setAlternatePrecision(int i,int prec) {
+      if( i>=field.length ) return;
+      field[i].alternatePrecision = prec;
+   }
+   
+  /** Retourne la desc
   /** Retourne le nombre de caracteres associes au champ.
     * @param i numero du champ
     * @return le nombre de caracteres, 10 si non specifie, -1 si erreur
@@ -540,7 +550,7 @@ public final class Legende extends AbstractTableModel  {
    /*******************************  Structure de la JTable *******************************/
 
    static final private String [] HEAD = {
-      "","Visible","Coo","Name","Description","Unit","Datatype","UCD","Utype","Width","Arraysize","Precision"
+      "","Visible","Ref","Name","Description","Unit","Datatype","UCD","Utype","Width","Arraysize","Precision"
    };
 
    static final private int [] WHEAD = { 30, 50, 50,100, 190, 50, 70, 110, -1, 40,40,40 };
@@ -605,6 +615,14 @@ public final class Legende extends AbstractTableModel  {
    /** Retourne l'indice du premier champ numrique */
    protected int getIndexNumericField() { return getIndexNumericField(-1); }
    
+   /** Retourne l'indice d'un champ de flux sed si existant, sinon -1 */
+   protected int getIndexFluxField() {
+      for( int i=0; i<field.length; i++ ) {
+         if( field[i].sed==Field.FLUX ) return i;
+      }
+      return -1;
+   }
+   
    /** Retourne l'indice du premier champ numrique aprs l'indique indiqu
     * @param after
     * @return
@@ -925,5 +943,5 @@ public final class Legende extends AbstractTableModel  {
    }
 
    // Juste pour de debuging
-   public String toString() { return field[0].name+" "+field[0].ucd+" ..."; }
+   public String toString() { return "Field[0]:\n"+field[0]+" ..."; }
 }
