@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -40,12 +40,6 @@ import javax.swing.JPopupMenu;
 import cds.astro.Astrocoo;
 import cds.astro.Astroframe;
 import cds.astro.Coo;
-import cds.astro.Ecliptic;
-import cds.astro.FK4;
-import cds.astro.FK5;
-import cds.astro.Galactic;
-import cds.astro.ICRS;
-import cds.astro.Supergal;
 import cds.tools.Astrodate;
 import cds.tools.Util;
 
@@ -373,7 +367,7 @@ public class Localisation extends MyBox  {
    protected JComboBox createSimpleChoice() {
 //      return new JComboBox(REPERE);
       JComboBox c = new JComboBox();
-      int n = Aladin.BETA ? REPERE.length : REPERE.length-2;
+      int n = aladin.configuration.isPlanet() ? REPERE.length : REPERE.length-2;
       for( int i=0; i<n; i++ ) c.addItem(REPERE[i]);
       return c;
       
@@ -387,8 +381,8 @@ public class Localisation extends MyBox  {
       });
       c.setPrototypeDisplayValue(new Integer(100000));
       c.setMaximumRowCount(REPERE.length);
-      c.setFont(F);
-      int n = Aladin.BETA ? REPERE.length : REPERE.length-2;
+      c.setFont(Aladin.PLAIN);
+      int n = aladin.configuration.isPlanet() ? REPERE.length : REPERE.length-2;
       for( int i=0; i<n; i++ ) c.addItem(REPERE[i]);
       //      else for( int i=0; i<REPERE.length-1; i++ ) c.addItem(REPERE[i]);
       c.setSelectedIndex(ICRS);
@@ -470,28 +464,12 @@ public class Localisation extends MyBox  {
       }
       
       for( i=0; i<n; i++ ) {
-         if( a[i]>='a' && a[i]<='z' ||  a[i]>='A' && a[i]<='Z' ) return true;
+         if( a[i]>='a' && a[i]<='z' ||  a[i]>='A' && a[i]<='Z' || a[i]=='/' ) return true;
       }
       return false;
    }
    
    
-//   static final public int ICRS   = 0;
-//   static final public int ICRSD  = 1;
-//   static final public int ECLIPTIC = 2;
-//   static final public int GAL    = 3;
-//   static final public int SGAL   = 4;
-//   static final public int J2000  = 5;
-//   static final public int J2000D = 6;
-//   static final public int B1950  = 7;
-//   static final public int B1950D = 8;
-//   static final public int B1900  = 9;
-//   static final public int B1875  = 10;
-//   static final public int XY     = 11;
-//   static final public int XYNAT  = 12;
-//   static final public int XYLINEAR  = 13;
-
-   
    /** Retourne le frame "gnrique" courant (ICRS | GAL | SGAL | ECLIPTIC | -1) */
    public int getFrameGeneric() {
       return frame==GAL ? GAL :
@@ -602,14 +580,14 @@ public class Localisation extends MyBox  {
    protected Coord getLastCoordInCurrentFrame() { return ICRSToFrame(coo); }
    protected void setLastCoord(double ra,double dec) { coo=new Coord(ra,dec); }
 
-   static final Astroframe AF_FK4 = new FK4();
-   static final Astroframe AF_FK5 = new FK5();
-   static final Astroframe AF_GAL = new Galactic();
-   static final Astroframe AF_SGAL = new Supergal();
-   static final Astroframe AF_ICRS = new ICRS();
-   static final Astroframe AF_ECLI = new Ecliptic();
-   static final Astroframe AF_FK4_1900 = new FK4(1900);
-   static final Astroframe AF_FK4_1875 = new FK4(1875);
+   static final Astroframe AF_FK4 = Astroframe.create("FK4");
+   static final Astroframe AF_FK5 = Astroframe.create("FK5");
+   static final Astroframe AF_GAL = Astroframe.create("Galactic");
+   static final Astroframe AF_SGAL = Astroframe.create("Supergalactic");
+   static final Astroframe AF_ICRS = Astroframe.create("ICRS");
+   static final Astroframe AF_ECLI = Astroframe.create("Ecliptic");
+   static final Astroframe AF_FK4_1900 = Astroframe.create("FK4(B1900)");
+   static final Astroframe AF_FK4_1875 = Astroframe.create("FK4(B1875)");
 
    // Retourne la valeur du frame prevue dans Astroframe
    // en fonction de la valeur courante du menu deroulant
