@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -75,15 +75,15 @@ KeyListener,ActionListener,
 MouseWheelListener, Widget
 {
    private int NBLIGNE = 5;        // Nombre de lignes visibles par defaut
-   static Font FONT = Aladin.COURIER;
-   static final int HF = Aladin.SIZE;   // Hauteur des lettres
-   static final int HL = HF+3;          // Hauteur d'une ligne
-   static final int MH = HL;            // Hauteur de la marge du haut
-   static final int MB = 1;             // Hauteur de la marge du bas
+   private Font FONT;
+   private int HF;          // Hauteur des lettres
+   private int HL;          // Hauteur d'une ligne
+   private int MH;          // Hauteur de la marge du haut
+   private int MB;          // Hauteur de la marge du bas
 
    // Triangles-reperes de la ligne courante (position sur 1ere ligne)
-   static final int [] tX =  {         5,         5,     14 };    // Exterieur
-   static final int [] tY =  { HL/2+1 -5, HL/2+1 +5, HL/2+1 };
+   private int [] tX;    // Exterieur
+   private int [] tY;
 
    Aladin aladin;        // Reference
    MyScrollbar scrollV;  // Reference
@@ -91,7 +91,7 @@ MouseWheelListener, Widget
 
    int W=500,H;              // Taille courante du MCanvas
    Image img;            // Double buffer
-   int wblanc;           // Taille d'un ``blanc'' dans le texte
+   double wblanc=-1;        // Taille d'un ``blanc'' dans le texte
    int firstsee;         // Premiere ligne affichee
    int lastsee;          // Derniere ligne affichee
    int currentsee=-1;    // Ligne courante (sous la souris)
@@ -133,14 +133,21 @@ MouseWheelListener, Widget
       this.aladin = aladin;
       this.scrollV = scrollV;
       this.scrollH = scrollH;
+      
+      FONT = Aladin.COURIER;
+      HF = FONT.getSize(); // Hauteur des lettres
+      HL = HF+3;          // Hauteur d'une ligne
+      MH = HL;            // Hauteur de la marge du haut
+      MB = 1;             // Hauteur de la marge du bas
+
+      tX = new int[]{         5,         5,     14 };    // Exterieur
+      tY = new int[]{ HL/2+1 -5, HL/2+1 +5, HL/2+1 };
+
 
       // Determination des tailles des lettres
-      wblanc = getToolkit().getFontMetrics(FONT).stringWidth("M");
-
-//      if( Aladin.OUTREACH ) NBLIGNE=3;
+//      wblanc = getToolkit().getFontMetrics(FONT).stringWidth("M");
 
       TIPHEAD = aladin.chaine.getString("TIPHEAD");
-      //      TIPREP  = aladin.chaine.getString("TIPREP");
       TIPGLU  = aladin.chaine.getString("TIPGLU");
       TIPARCH = aladin.chaine.getString("TIPARCH");
       TIPFOV  = aladin.chaine.getString("TIPFOV");
@@ -150,7 +157,7 @@ MouseWheelListener, Widget
       createPinPopupMenu();
 
       setOpaque(true);
-      setDoubleBuffered(false);
+      setDoubleBuffered(false); 
       addMouseListener(this);
       addMouseMotionListener(this);
       addKeyListener(this);
@@ -325,6 +332,7 @@ MouseWheelListener, Widget
     * l'application SAMP cibles via le popupmenu qui va apparatre */
    protected void toSamp(String url,int x,int y) {
       urlSamp=url;
+      aladin.view.infoCassis(null);
       showSAMPPopMenu(x,y);
    }
 
@@ -581,7 +589,6 @@ MouseWheelListener, Widget
     */
    protected int drawWords(Graphics g, Words w, boolean flagClear) {
       return drawWords(g,w,flagClear, w.y<=HF ?  Aladin.COLOR_MEASUREMENT_HEADER_BACKGROUND : Aladin.COLOR_MEASUREMENT_BACKGROUND );
-//      return drawWords(g,w,flagClear, aladin.getBackground() ); //w.y<=HF ? Aladin.BKGD: (w.num%2==0 ? Color.white : BG ));
    }
    private int drawWords(Graphics g, Words w, boolean flagClear, Color background) {
       int y = w.y+HF;        // Ligne de base
@@ -591,18 +598,12 @@ MouseWheelListener, Widget
       int widthw;	     // Taille du mot
       String text =  w.text;
       
-      // Prise en compte de la precision
+      // Prise en compte de la precision pour aligner le point dcimal
       if( w.precision>=0 ) {
-         //         int i = text.lastIndexOf('.');
          int j = text.indexOf(' ');
          if( j<0 ) {
-            //            int pos = w.precision>0 ? w.precision+1 : w.precision;
-            //            if( i+pos<text.length() ) text = text.substring(0,i+pos);
-
             
             try {
-//               double v = Double.parseDouble(w.text);
-//               text = (new Formatter(Locale.ENGLISH)).format("%."+w.precision+"f", v).toString();
                text = Util.myRound(w.text,w.precision, false);
                int i = text.lastIndexOf('.');
                if( i>0 ) {
@@ -615,6 +616,8 @@ MouseWheelListener, Widget
                      for( j=k;j<n-1;j++) trail.append(' ');
                      text = text.substring(0,k+1)+trail.toString();
                   }
+               } else {
+                  for( j=0; j<w.precision+1; j++ ) text+=' ';
                }
             } catch( Exception e) {}
          }
@@ -624,15 +627,16 @@ MouseWheelListener, Widget
       boolean flagCut=false;  // true si le mot est tronqu
       FontMetrics fm = g.getFontMetrics();
       widthw = fm.stringWidth(text) + (w.sort!=Field.UNSORT ? 12 : 0);
+//      widthw = (int)( text.length()*wblanc + (w.sort!=Field.UNSORT ? 12 : 0) );
       if( w.onMouse ) {
-         if( widthw<w.width*wblanc ) width = w.width*wblanc;
+         if( widthw<(int)(w.width*wblanc) ) width = (int)(w.width*wblanc);
          else {
             width=widthw;
             if( w.archive || w.footprint ) width+=3;
          }
       } else {
-         width = w.width*wblanc;
-         flagCut = widthw>w.width*wblanc;
+         width = (int)(w.width*wblanc);
+         flagCut = widthw>(int)(w.width*wblanc) ;
       }
 
       // Remplissage prealable si necessaire
@@ -653,11 +657,11 @@ MouseWheelListener, Widget
          }
 
          g.setColor( bg );
-         g.fillRect(x-4,y-HF,width+7,HL-1);
+         g.fillRect(x-6,y-HF,width+9,HL-1);
          if( w.onMouse && !(w.archive || w.footprint)  ) {
             int M=2;
             g.setColor( Aladin.COLOR_MEASUREMENT_BACKGROUND_MOUSE_CELL );
-            g.fillRect(x-4+M,y-HF+M-1,width+7-2*M,HL-1-2*M+2);
+            g.fillRect(x-6+M,y-HF+M-1,width+9-2*M,HL-1-2*M+2);
          }
       }
 
@@ -700,7 +704,7 @@ MouseWheelListener, Widget
 
       // Mot trop long ?
       if( flagCut ) {
-         int c = (widthw - (width - (w.archive?6:0)) )/wblanc +2;
+         int c = (int)((widthw - (width - (w.archive?6:0)) )/wblanc +2);
          int offset=text.length()-c;
          if( offset>0 ) text = text.substring(0,offset);
       }
@@ -718,7 +722,7 @@ MouseWheelListener, Widget
       // On ajoute des points  la fin du mot s'il est coup
       if( flagCut ) {
          Font f = g.getFont();
-         int xp = g.getFontMetrics().stringWidth(text)+xtext;
+         int xp = g.getFontMetrics().stringWidth(text.trim())+xtext;
          g.setFont(Aladin.PLAIN);
          g.drawString("...",xp,y);
          g.setFont(f);
@@ -741,7 +745,7 @@ MouseWheelListener, Widget
 
       drawBordG(g,x,y-HF,width);
 
-      return w.width*wblanc;
+      return (int)Math.ceil(w.width*wblanc)+1;
    }
 
 
@@ -802,14 +806,14 @@ MouseWheelListener, Widget
          }
 
          w.setPosition(x,y-HF,width,HF);  // Memo de la boite recouvrant le mot, le repere
-         x += width+wblanc;               // Calcul de la prochaine abscisse
+         x += (int)(width+wblanc);               // Calcul de la prochaine abscisse
          flagFirst=false;
       }
 
       if( W!=-1 ) {
 
          // Affichage du repere de debut de ligne si necessaire
-         if( rep!=null && y>HF ) Util.drawCheckbox(g,X+3,y-11,o.plan.c,null,null,o.isTagged());
+         if( rep!=null && y>HF ) Util.drawCheckbox(g,X+3,y-HF/2-3,o.plan.c,null,null,o.isTagged());
 
          // On finit le fond de la ligne si ncessaire
          if( x<X+W ) {
@@ -892,7 +896,7 @@ MouseWheelListener, Widget
          }
 
          w.setPosition(x,y-HF,width,HF);  // Memo de la boite recouvrant le mot, le repere
-         x += width+wblanc;               // Calcul de la prochaine abscisse
+         x += (int)(width+wblanc);               // Calcul de la prochaine abscisse
       }
 
       if( W!=-1 ) {
@@ -1213,7 +1217,7 @@ MouseWheelListener, Widget
       // On redimensionne la colonne
       if( onBordField!=-1 ) {
          Field f = oleg.field[onBordField];
-         int offset = (onBordX-x)/wblanc;
+         int offset = (int)((onBordX-x)/wblanc);
          if( offset==0 ) return;
          int colSize = f.columnSize;
          colSize-=offset;
@@ -1293,7 +1297,7 @@ MouseWheelListener, Widget
       }
 
       // Affichage du repere de debut de ligne si necessaire
-      if( rep!=null && y>HF) Util.drawCheckbox(g,3,y+1,o.plan.c,null,null,o.isTagged());
+      if( rep!=null && y>HF) Util.drawCheckbox(g,3,y+HF/2-3,o.plan.c,null,null,o.isTagged());
    }
 
    /** Initialisation de la ligne  montrer  + affichage */
@@ -1789,7 +1793,7 @@ MouseWheelListener, Widget
       // Juste en cas d'ouverture du panel
       if( W!=oW ) {
          scrollH.setVisibleAmount(W);
-         scrollH.setBlockIncrement(W-10*wblanc);
+         scrollH.setBlockIncrement( (int)(W-10*wblanc) );
          oW=W;
       }
 
@@ -1798,7 +1802,7 @@ MouseWheelListener, Widget
          aladin.mesure.add(scrollH,"South");
          showScrollH=true;
          scrollH.setVisibleAmount(W);
-         scrollH.setBlockIncrement(W-10*wblanc);
+         scrollH.setBlockIncrement( (int)(W-10*wblanc));
          nbligne--;
       }
       if( !ok ) return;
@@ -1838,6 +1842,7 @@ MouseWheelListener, Widget
    
    private Image iconOut=null;
    private Rectangle rectOut = null;
+   
 
    // Regeneration du plan image pour la fenetre des mesures
    public void paintComponent(Graphics gr) {
@@ -1863,6 +1868,15 @@ MouseWheelListener, Widget
       
       super.paintComponent(gr);
       
+      // Initialisation de la taille d'une lettre (la fonte est ncessaire monoscaped)
+      if( wblanc==-1 ) {
+         StringBuilder s = new StringBuilder();
+         for( int i=0; i<100; i++ ) s.append('M');
+         wblanc = gr.getFontMetrics(FONT).stringWidth(s.toString())/(double)s.length();
+      }
+      
+      boolean flagDoubleBuffering = Aladin.useDoubleBuffering(gr);
+      
       int j;
 
       // Positionnement du curseur apres le demarrage d'Aladin
@@ -1875,16 +1889,14 @@ MouseWheelListener, Widget
       showLigne=null;
       ow=null;
       
-      // Double buffer (beaucoup plus rapide que le double buffering natif de SWING)
-      if( img==null || img.getWidth(this)!=getWidth()
-            || img.getHeight(this)!=getHeight() ) {
+      if( img==null || img.getWidth(this)!=getWidth() || img.getHeight(this)!=getHeight() 
+          ){
          W=getWidth();
          H=getHeight();
-         //         img = getGraphicsConfiguration().createCompatibleImage(W,H);
          img = aladin.createImage(W,H);
       }
-      Graphics g=img.getGraphics();
-
+      Graphics g = flagDoubleBuffering ? img.getGraphics() : gr;
+      
       // AntiAliasing
       aladin.setAliasing(g);
 
@@ -1950,7 +1962,7 @@ MouseWheelListener, Widget
       // Les bordures du cadre
       Util.drawEdge(g,W,H);
       
-      gr.drawImage(img,0,0,this);
+      if( flagDoubleBuffering ) gr.drawImage(img,0,0,this);
       
       if( aladin.view.zoomview.flagSED ) aladin.view.zoomview.repaint();
       
