@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -45,6 +45,8 @@ import javax.swing.JScrollPane;
 import javax.swing.JTextArea;
 
 import cds.astro.Unit;
+import cds.moc.Healpix;
+import cds.moc.SMoc;
 import cds.tools.Astrodate;
 import cds.tools.ConfigurationReader;
 import cds.tools.ScientificUnitsUtil;
@@ -128,7 +130,6 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       this.plan = plan;
       this.c=c;                        // Couleur du plan
       nb_o = 0;
-      //      nRa=nDec=-1;
    }
 
    protected void free() {
@@ -193,7 +194,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       }
 
       // Test sur le recouvrement des champs
-      if( /* Projection.isOk(proj) && */plan.type!=Plan.TOOL && plan.type!=Plan.APERTURE
+      if( plan.type!=Plan.TOOL && plan.type!=Plan.APERTURE
             && !(plan.isSimpleCatalog() && (plan.hasXYorig || v.isPlot()) )
             && !proj.agree(plan.projd,v) ) return;
 
@@ -207,7 +208,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       for( int i=0; i<nb_o; i++ ) {
          try {
             o[i].projection(v);   // On applique la projection a chaque source
-         } catch( Exception e ) {}
+         } catch( Exception e ) { if( aladin.levelTrace>=3 ) e.printStackTrace(); }
       }
 
       // Pour que les (x,y) soient recalcules dans la vue courante
@@ -331,7 +332,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             rajc,dejc,rm*2,
             250.0,250.0,500.0,
             0.0,false,
-            typeProj,Calib.FK5));
+            typeProj,Calib.FK5,plan));
 
       // Positionnement du centre que si ce n'est pas la valeur par dfaut 0,0
       if( rajc!=0 && dejc!=0 ) plan.co=new Coord(rajc,dejc);
@@ -392,7 +393,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    public void endTable() { 
       long t = Util.getTime() - timeStartTable;
       int nbObj = nb_o-lastNb_o;
-      tableParserInfo("   -Table loaded & parsed in "+Util.getTemps(t)
+      tableParserInfo("   -Table loaded & parsed in "+Util.getTemps(t*1000L)
             + " for "+nbObj+" object"+(nbObj>1?"s":"")
             + (nbObj<1000?"":" ("+Util.myRound(""+1000.*nbObj/t)+" objects per sec)"));
    }
@@ -520,6 +521,9 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    private TableParser res;          // Parser utilis pour crer les objets
    private StringBuilder line = new StringBuilder(500);
    private Map<Integer, Field> standardisedColumns = new HashMap<>();
+   
+   /** Demande d'interruption d'un parsing en cours */
+   public void interrupt() throws Exception { res.interrupt(); }
 
 
 //   // Lgende gnrique qui vient remplacer toutes les lgendes propres
@@ -627,7 +631,9 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             }
 
             // En cas de plan HiPS catalog, la lgende est globale et externe
-            if( plan instanceof PlanBGCat ) leg=((PlanBGCat)plan).getFirstLegende();
+            if( plan instanceof PlanBGCat ) {
+               leg=((PlanBGCat)plan).getFirstLegende();
+            }
             if( leg==null ) {
                leg = new Legende(v);
                leg.name=table;
@@ -698,7 +704,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
 //               }
 //            } catch( Exception e ) { }
             
-            if( indexSTC==-1 && Util.indexOfIgnoreCase( value[i], "Polygon ")==0 ) {
+            if( indexSTC==-1 && ( Util.indexOfIgnoreCase( value[i], "Polygon ")==0 
+                               || Util.indexOfIgnoreCase( value[i], "Circle ")==0 )) {
                indexSTC=j;
             }
             
@@ -985,7 +992,9 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       while( it.hasNext() ) {
          Obj o = it.next();
          if( o.isSelected() ) return true;
-         if( o instanceof Source && ((Source)o).isTagged() ) return true;
+//         if( o instanceof Source && ((Source)o).isTagged() ) return true;
+         if( o.asSource() && ((Source)o).isTagged() ) return true;
+
       }
       return false;
    }
@@ -1016,6 +1025,12 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
     */
    public void tableParserWarning(String msg) {
       if( msg.startsWith("!!!") ) {
+         int n = msg.indexOf("BODY=");
+         if( n>0 ) {
+            String body = msg.substring(n+5);
+            plan.setBody(body);
+            return;
+         }
          if( msg.indexOf("OVERFLOW")>=0 ) plan.error=msg;
          if( msg.indexOf("ERROR")>=0 ) plan.error=msg;
       }
@@ -1170,7 +1185,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       if( ok ) {
          if( !flagEndResource ) endResource();
          long duree=System.currentTimeMillis()-d;
-         String s = "Catalog queried, loaded and parsed in "+Util.getTemps(duree);
+         String s = "Catalog queried, loaded and parsed in "+Util.getTemps(duree*1000L);
          tableParserInfo("\n"+s);
          Aladin.trace(3,s);
          if( !flagXY && rm==0.0 ) plan.error = aladin.error = "no RA or DE columns";
@@ -1179,43 +1194,87 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       return ok && nb_o>=0?nb_o:-1;
    }
 
-   /** (Re)gnration de la projection par dfaut associe  la liste des objets en prenant
-    * comme centre de projection le barycentre */
    protected void createDefaultProj() {
-      for( int i=0; i<nb_o; i++ ) {
-         Position s = (Position)o[i];
-         if( i==0 || s.raj < minRa )  minRa  = s.raj;
-         if( i==0 || s.raj > maxRa )  maxRa  = s.raj;
-         if( i==0 || s.dej < minDec ) minDec = s.dej;
-         if( i==0 || s.dej > maxDec ) maxDec = s.dej;
+      if( nb_o==0 ) { 
+         rajc=dejc=0; 
+         rm=7*60.;
+         
+      } else if( nb_o==1 ) {
+         Position s = (Position)o[1];
+         rajc = s.raj;
+         dejc = s.dej;
+         rm=7*60.;
+         
+      } else {
+         try {
+            // Si toute tombe dans la mme cellule MOC de 1, on fera une projection limite  1
+            // sinon on fera large (on ne s'embte plus car gnralement il y a un HiPS en dessous)
+            Healpix hpx = new Healpix();
+            SMoc moc = new SMoc(6);    // cellule de 1
+            moc.bufferOn();
+            
+            int gap = 1;
+            if( nb_o>1000 ) gap = nb_o/1000;
+            
+            Position target=null;
+            for( int i=0; i<nb_o; i+=gap ) {
+               Position s = (Position)o[i];
+               moc.add(hpx, s.raj, s.dej);
+               if( i==0 ) target=s;
+            }
+            moc.bufferOff();
+            rajc = target.raj;
+            dejc = target.dej;
+            rm = (moc.getNbValues()==1 ? 2: 90)*60.;
+            Aladin.trace(3,"computeTarget by Moc =>"+rajc+","+dejc+" rm=["+rm+"] moc.getNbValues="+moc.getNbValues( ));
+
+         } catch( Exception e ) {
+            e.printStackTrace();
+         }
       }
-      computeTarget();
       postJob(rajc,dejc,rm,false);
    }
-
-   /** Determine le target des donnees en fonction.
-    * en fonction de minRa, maxRa, minDec, maxDec et
-    * met a jour rajc, dejc et rm en fonction
-    */
-   private void computeTarget() {
-//      if( maxDec-minDec>90 ) {
-      if( maxDec-minDec>90 || maxRa-minRa>180.) {
-         dejc=rajc=0;
-         rm=180*60.;
-      } else {
-         if( maxRa-minRa>180. ) {
-            double alpha = 360-maxRa;
-            rajc = (minRa+alpha)/2 - alpha;
-         } else rajc = (minRa+maxRa)/2;
-         dejc = (minDec+maxDec)/2;
-//         double r = Math.max(Math.abs(minRa-rajc)*Math.cos( (dejc*Math.PI)/180.0 ), Math.abs(minDec-dejc));
-//         rm = (nb_o==1 || r==0.)?7:r*60.0*1.4142;
-         double r = Coord.getDist( new Coord(minRa, minDec), new Coord(maxRa, maxDec) )/2;
-         rm = (nb_o==1 || r==0.)?7:r*60.0;
-
-      }
-      Aladin.trace(3,"computeTarget ra=["+minRa+".."+maxRa+"]=>"+rajc+" de=["+minDec+".."+maxDec+"]=>"+dejc+" rm=["+rm+"]");
-   }
+   
+   private void computeTarget() { createDefaultProj(); }
+   
+   
+   /** (Re)gnration de la projection par dfaut associe  la liste des objets en prenant
+    * comme centre de projection le barycentre */
+//   protected void createDefaultProj() {
+//      for( int i=0; i<nb_o; i++ ) {
+//         Position s = (Position)o[i];
+//         if( i==0 || s.raj < minRa )  minRa  = s.raj;
+//         if( i==0 || s.raj > maxRa )  maxRa  = s.raj;
+//         if( i==0 || s.dej < minDec ) minDec = s.dej;
+//         if( i==0 || s.dej > maxDec ) maxDec = s.dej;
+//      }
+//      computeTarget();
+//      postJob(rajc,dejc,rm,false);
+//   }
+//   
+//   /** Determine le target des donnees en fonction.
+//    * en fonction de minRa, maxRa, minDec, maxDec et
+//    * met a jour rajc, dejc et rm en fonction
+//    */
+//   private void computeTarget() {
+////      if( maxDec-minDec>90 ) {
+//      if( maxDec-minDec>90 || maxRa-minRa>180.) {
+//         dejc=rajc=0;
+//         rm=180*60.;
+//      } else {
+//         if( maxRa-minRa>180. ) {
+//            double alpha = 360-maxRa;
+//            rajc = (minRa+alpha)/2 - alpha;
+//         } else rajc = (minRa+maxRa)/2;
+//         dejc = (minDec+maxDec)/2;
+////         double r = Math.max(Math.abs(minRa-rajc)*Math.cos( (dejc*Math.PI)/180.0 ), Math.abs(minDec-dejc));
+////         rm = (nb_o==1 || r==0.)?7:r*60.0*1.4142;
+//         double r = Coord.getDist( new Coord(minRa, minDec), new Coord(maxRa, maxDec) )/2;
+//         rm = (nb_o==1 || r==0.)?7:r*60.0;
+//
+//      }
+//      Aladin.trace(3,"computeTarget ra=["+minRa+".."+maxRa+"]=>"+rajc+" de=["+minDec+".."+maxDec+"]=>"+dejc+" rm=["+rm+"]");
+//   }
 
    // Analyse de la chaine indiquant le target
    // REMARQUE : Cette procedure a ete ecrite en deux temps trois mouvements, il faudra
@@ -1358,7 +1417,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       FootprintBean footprint;
       PlanField pf;
       for( int i=0; i<nb_o; i++ ) {
-         if( ! (o[i] instanceof Source) ) continue;
+//         if( ! (o[i] instanceof Source) ) continue;
+         if( !o[i].asSource() ) continue;
          s = (Source)o[i];
          pf=null;
 
@@ -1441,10 +1501,10 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       return i;
    }
    
-   /**Insertion de la source aprs la dernire source de mme lgende, sinon  la fin */
+   /** Insertion de la source aprs la dernire source de mme lgende, sinon  la fin */
    protected void insertSource(Source src) {
       for( int i=nb_o-1; i>=0; i-- ) {
-         if( !(o[i] instanceof Source) ) continue;
+         if( !o[i].asSource() ) continue;
          
          // On a trouv ?
          if( ((Source)o[i]).getLeg()==src.getLeg() ) {
@@ -1471,7 +1531,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    protected void fixInfo(Legende leg) {
       int j=0;
       for( int i=0; i<nb_o; i++ ) {
-         if( !(o[i] instanceof Source) ) continue;
+//         if( !(o[i] instanceof Source) ) continue;
+         if( !o[i].asSource() ) continue;
          Source s = (Source)o[i];
          if( s.getLeg()==leg ) { s.fixInfo(); j++; }
       }
@@ -1486,6 +1547,17 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
 
    protected boolean removable = false;     // Possibilit de changer le statut d'un plan catalogue
 
+   /** Suppression d'un objet, simple ou multicomponent
+    * @param obj L'objet a supprimer
+    * @param force true si on peut supprimer mme les sources des catalogues
+    * @return <I>true</I> si trouve, sinon <I>false</I>
+    */
+   protected boolean delObjet(Obj obj) { return delObjet(obj,removable); }
+   protected boolean delObjet(Obj obj,boolean force) {
+      if( obj instanceof Ligne && !(obj instanceof Cote) ) return delLigne(obj);
+      return delObjetOne(obj,force);
+   }
+
    /** Suppression d'un objet.
     * Suppression d'un objet par ecrasement de sa reference
     * avec le dernier element du tableau
@@ -1493,12 +1565,12 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
     * @param force true si on peut supprimer mme les sources des catalogues
     * @return <I>true</I> si trouve, sinon <I>false</I>
     */
-   protected boolean delObjet(Obj obj) { return delObjet(obj,removable); }
-   protected boolean delObjet(Obj obj,boolean force) {
+   protected boolean delObjetOne(Obj obj,boolean force) {
       int i;
 
       // Les sources ne peuvent etre supprimer individuellement
-      if( !force && obj instanceof Source ) return false;
+//      if( !force && obj instanceof Source ) return false;
+      if( !force && obj.asSource() ) return false;
 
       // Peut tre faut-il qu'il mette  jour certains paramtres (ex. Ligne)
       obj.remove();
@@ -1506,13 +1578,25 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       // Parcours de tous les objets du plan
       for( i=0; i<nb_o && obj!=o[i]; i++ );
       if( i<nb_o ) {
-         //         if( i!=nb_o-1 ) o[i] = o[nb_o-1];
          for( ; i<nb_o-1; i++ ) o[i]=o[i+1];  // Pour conserver l'ordonnancement
          nb_o--;
          return true;
       }
       return false;
    }
+   
+   /** Suppression d'une polyligne
+    * @param obj L'objet a supprimer (un des sommets de la polyligne)
+    * @return <I>true</I> si trouve, sinon <I>false</I>
+    */
+   protected boolean delLigne(Obj obj) {
+      Iterator it = obj.iterator();
+      while( it.hasNext() ) {
+         obj = (Obj)it.next();
+         if( !delObjetOne(obj,true) ) return false;
+      }
+      return false;
+   }
 
    /** Affiche l'id associe a un objet.
     * @param i Indice de l'objet
@@ -1671,7 +1755,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    synchronized protected void writeLinkFlex(OutputStream out, ViewSimple v,boolean draw) throws Exception {
       if( !computeAndTestDraw(v,draw) ) return;
       for( int i=0; i<nb_o; i++ ) {
-         if( ! (o[i] instanceof Source) ) continue;
+//         if( ! (o[i] instanceof Source) ) continue;
+         if( !o[i].asSource() ) continue;
          ((Source)o[i]).writeLinkFlex(out,v);
       }
    }
