@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -209,7 +209,6 @@ public class PlanBGCat extends PlanBG {
 //      System.out.println("hipsOld="+hipsOld+" minOrder = "+minOrder);
       
       SMoc lastMoc = new SMoc();   // Moc des tuiles "last" (inutile d'aller plus loin)
-      try { lastMoc.setCheckConsistencyFlag(false); } catch( Exception e1 ) { }
       
       resetDrawnInView(v);
       
@@ -240,7 +239,8 @@ public class PlanBGCat extends PlanBG {
          for( int i=0; i<pix.length; i++ ) {
             
             // Losange  carter ?
-            if( lastMoc.isDescendant(norder, pix[i]) ) continue;
+//            if( lastMoc.isDescendant(norder, pix[i]) ) continue;
+            if( lastMoc.isIncluding(norder, pix[i]) ) continue;
             if( isOutMoc(norder, pix[i]) )  continue;
             if( (new HealpixKey(this,norder,pix[i],HealpixKey.NOLOAD)).isOutView(v) ) continue;
 
@@ -276,7 +276,7 @@ public class PlanBGCat extends PlanBG {
             // Il n'y aura plus rien dans cette branche du HiPS
             if( norder!=order ) {
                if( healpix.isLast() ) {
-                  try { lastMoc.add(healpix.order,healpix.npix); } catch( Exception e ) { }
+                  try { lastMoc.add(healpix.order,healpix.npix); } catch( Exception e) {}
                }
                
             // Y a-t-il encore des sources non visibles dans le champ (PAS SUR DE BIEN COMPRENDRE PF Mars 2019)
@@ -333,23 +333,61 @@ public class PlanBGCat extends PlanBG {
    protected double getCompletude() { return completude; }
 
    private int oNbObj = -1;   // Compteur d'objets dj charges
+   private int oHashObj=-1;   // Cl de hash de l'ensemble des objets dj chargs
    
    /** Demande de raffichage des vues */
    protected void askForRepaint() {
-      // Mise  jour des filtres (uniquement si le nombre d'objets  chang)
-      // UN PEU CASSE GUEULE COMME TEST, mais pour le moment a va faire l'affaire
+      
       int nbObj = getCounts();
-      if( oNbObj!=getCounts() ) {
+      int hashObj = 0;
+      
+      boolean flagUpdate=false;    // passera  true s'il faut relancer les filtres
+      if( oNbObj!=nbObj ) flagUpdate=true;
+      else {
+         hashObj = hashSrc();
+         if( hashObj!=oHashObj ) flagUpdate=true;
+      }
+      if( PlanFilter.DEBUG ) System.err.println("PlanBGCat.askForRepaint() nbObj="+nbObj+" hashSrc="+hashSrc()+" => update required");
+      
+      if( flagUpdate ) {
          oNbObj = nbObj;
-//         System.out.println("update filter");
-//         for( PlanFilter pf : PlanFilter.allFilters ) System.out.println("."+pf);
+         if( hashObj==0 ) hashObj = hashSrc();   // pas encore calcul ?
+         oHashObj = hashObj;
+         if( PlanFilter.DEBUG ) {
+            System.err.println("PlanBGCat.askForRepaint(): update filter");
+            for( PlanFilter pf : PlanFilter.allFilters ) System.err.println("  ."+pf);
+         }
          updateDedicatedFilter();
       }
       aladin.view.repaintAll();
    }
    
+   // Calcule une cl de hash pour l'ensemble des objets actuellement chargs
+   private int hashSrc() {
+      int hash=0;
+      Iterator<Obj> it = iterator();
+      if( it==null ) return hash;
+      while( it.hasNext() ) {
+         Obj obj = it.next();
+         hash += 31*obj.hashCode();
+      }
+      return hash;
+   }
+   
    protected void planReady(boolean ready) {
       super.planReady(ready);
+      
+      // Pour viter le deadlock, on temporise l'activation du filtre par dfaut (s'il y a lieu)
+//      if( filterIndex==-1 ) setFilter(filterIndex);
+//      else {
+//         (new Thread(){
+//            public void run() {
+//               Util.pause(300);
+//               setFilter(filterIndex);
+//            }
+//         }).start();
+//      }
+      
       setFilter(filterIndex);
       askForRepaint();
    }
@@ -401,18 +439,19 @@ public class PlanBGCat extends PlanBG {
    }
 
    protected Legende getFirstLegende() { return genericPcat==null ? null : genericPcat.leg; }
-   protected boolean hasGenericPcat() { return genericPcat!=null; }
+   protected boolean hasGenericPcat() {  return genericPcat!=null; }
    protected Pcat getGenericPcat() { return genericPcat; }
    protected boolean hasCatalogInfo() { return hasGenericPcat() && genericPcat.hasCatalogInfo(); }
    
    
-   
-   
    /** Mmorisation d'un pcat "talon" qui va me servir pour connatre la lgende gnrique
     * ou spcifique  la premire tuile charge
     */
    protected void setGenericPcat(Pcat pcat) {
-      genericPcat = pcat;
+      if( pcat==null ) return;
+      
+      if( genericPcat==null )  genericPcat=pcat;
+      
       if( pcat.hasObj() ) {
          try {
             Source src = (Source)pcat.iterator().next();
