@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -21,6 +21,7 @@
 
 package cds.aladin;
 
+import java.awt.Color;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -29,6 +30,8 @@ import java.net.URL;
 import java.util.Iterator;
 import java.util.Vector;
 
+import javax.swing.SwingUtilities;
+
 import cds.tools.Util;
 
 /**
@@ -72,7 +75,7 @@ public class PlanCatalog extends Plan {
       flagLocal=true;
       flagWaitTarget=true;  // Voir Command.waitingPlanInProgress
 
-      Suite(aladin,label,"","",null,null, null, null);
+      Suite(aladin,label,"","",null,null, null, null, null);
    }
 
   /** Creation d'un plan de type CATALOG (via un InputStream)
@@ -84,7 +87,7 @@ public class PlanCatalog extends Plan {
       this.dis = in;
       if( label==null) label="VOApp";
       flagWaitTarget=true;
-      Suite(aladin,label,"","",origin,server, null, null);
+      Suite(aladin,label,"","",origin,server, null, null, null);
    }
 
    protected PlanCatalog(Aladin aladin, MyInputStream in,String label) {
@@ -97,16 +100,16 @@ public class PlanCatalog extends Plan {
       this.httpConn = httpConn;
       if( label==null) label="HttpConn";
       flagWaitTarget=true;
-      Suite(aladin,label,"","",null,null, null, null);
+      Suite(aladin,label,"","",null,null, null, null, null);
      }
 	
-	protected PlanCatalog(Aladin aladin, MyInputStream in, String label,String origin, Server server, URL url, String query, int requestId) {
+	protected PlanCatalog(Aladin aladin, MyInputStream in, String label,String origin, Server server, URL url, String query, int requestId, Color color) {
 		this.dis = in;
 		if (label == null)
 			label = "VOApp";
 		flagWaitTarget = true;
 		this.requestId = requestId;
-		Suite(aladin, label, "", "", origin, server, url, query);
+		Suite(aladin, label, "", "", origin, server, url, query,color);
 	}
 
   /** Creation d'un plan de type CATALOG (sans info)
@@ -114,8 +117,8 @@ public class PlanCatalog extends Plan {
   protected PlanCatalog(Aladin aladin) {
     this.aladin = aladin;
     type = CATALOG;
-    c          = Couleur.getNextDefault(aladin.calque);
-    pcat       = new Pcat(this,c,aladin.calque,aladin.status,aladin);
+    c = Couleur.getNextDefault(aladin.calque);
+    pcat = new Pcat(this,c,aladin.calque,aladin.status,aladin);
     flagOk=true;
   }
 
@@ -136,7 +139,7 @@ public class PlanCatalog extends Plan {
       flagLocal  = false;
 
 //      Suite(aladin,label,objet,param,from,server, null, null);  <= Chaitra bug
-      Suite(aladin,label,objet,param,from,server, u, null);
+      Suite(aladin,label,objet,param,from,server, u, null,null);
 
    }
    
@@ -149,11 +152,12 @@ public class PlanCatalog extends Plan {
    * @param server le serveur d'origine
  * @param query 
    */
-   protected void Suite(Aladin aladin, String label,String objet,String param,String from,Server server, URL url, String query ) {
+   protected void Suite(Aladin aladin, String label,String objet,String param,
+         String from,Server server, URL url, String query, Color color ) {
       setLogMode(true);
       this.aladin= aladin;
       type       = CATALOG;
-      c          = Couleur.getNextDefault(aladin.calque);
+      c = color!=null ? color : Couleur.getNextDefault(aladin.calque);
       setLabel(label);
       id=this.label;
       this.objet = objet;
@@ -195,8 +199,9 @@ public class PlanCatalog extends Plan {
       Iterator<Obj> it = iterator();
       while( it.hasNext() ) {
          Obj o = it.next();
-         if( !(o instanceof Source) ) continue;
-         return getTableName( (Source)o );
+//         if( !(o instanceof Source) ) continue;
+         if( !o.asSource() ) continue;
+        return getTableName( (Source)o );
       }
       return null;
    }
@@ -215,13 +220,20 @@ public class PlanCatalog extends Plan {
   /** Libere le plan.
    * cad met toutes ses variables a <I>null</I> ou a <I>false</I>
    */
-   protected boolean Free() {
-      if( getCounts()<=0 || isSED() ) return true;
+   protected boolean Free() { return Free(false); }
+   protected boolean Free(boolean flagAskInterrupt) {
+      int nbObj = getCounts();
+      if( nbObj<=0 || isSED() ) return true;
       
+      if( flagAskInterrupt && nbObj>0 && !isReady() ) {
+         if( aladin.confirmation(aladin.chaine.getString("INTERRUPTCAT")) ) {
+            loadInterrupt();
+            return false;
+         }
+      }
+
       aladin.view.deSelect(this);
-//      if (Aladin.PROTO) {
-    	  TapManager.getInstance(aladin).updateDeleteUploadPlans(this);
-//      }
+      TapManager.getInstance(aladin).updateDeleteUploadPlans(this);
       super.Free();
       aladin.view.free(this);
       headerFits=null;
@@ -231,6 +243,111 @@ public class PlanCatalog extends Plan {
       return true;
    }
    
+   // Demande d'interruption de chargement (on attendra tout de mme la fin de l'enregistrement courant)
+   private void loadInterrupt() {
+      try {
+         pcat.interrupt();
+      } catch( Exception e ) {
+         e.printStackTrace();
+      }
+   }
+   
+   final String TAGCIRCLE="CIRCLE('ICRS',";
+   final String TAGTARGET="$TARGET";
+   
+   /** Retourne true si le plan peut tre rinterroger sur la position courante */
+   protected boolean isRedoable() {
+      if( !flagOk ) return false;
+      String s =  getBookmarkCode();
+      if( s!=null && s.indexOf(TAGTARGET)>0 ) return true;
+      s = getAdqlQuery();
+      if( s!=null && s.indexOf(TAGCIRCLE)>0 ) return true;
+      return false;
+   }
+   
+   /** Remet  jour le contenue du plan (catalogue classique) en fonction 
+    * du champ de la vue courante 
+    */
+   protected boolean redoConeSearch() {
+      String code =  getBookmarkCode();
+      String query = getAdqlQuery();
+      if( code==null && query==null ) return false;
+      
+      String cmd;
+      
+      // Rcupration du centre et de la taille du champ courant
+      Coord target = aladin.view.getCurrentView().getCooCentre();
+      double radius = aladin.view.getCurrentView().getTaille();
+
+      // Substitution de la position et du rayon dans la contrainte 
+      // ADQL de position
+      if( query!=null ) {
+         int offset = query.indexOf(TAGCIRCLE);
+         if( offset<0 ) return false;
+         int end = query.indexOf(')',offset);
+         if( end<0 ) return false;
+         query = query.substring(0,offset+TAGCIRCLE.length())
+               + target.al + ", "+ target.del+", "+radius
+               + query.substring(end);
+         
+         cmd = "get TAP("+getIdPrefix(id)+","+Tok.quote(query,true)+")";
+
+      // Substitution de la position et du rayon dans le bookmark Cone Search
+      } else {
+         int offset = code.indexOf(TAGTARGET);
+         if( offset<0 ) return false;
+         code = code.substring(0,offset);
+         cmd = code+" "+target.getRA()+" "+target.getDE()+" "+radius+"deg";
+      }
+      
+      // Pour faire clignoter le plan
+      flagOk=false;
+      pourcent=0;
+      
+      // Exccution de la commande en remplacement du plan courant
+      aladin.execCommand(label+"="+cmd);
+      
+      // Affectation de la mme couleur
+      Plan p = aladin.calque.getPlan(label,1);
+      p.c = c;
+      
+      return true;
+   }
+   
+   /** Remet  jour le contenue du plan (catalogue classique) en fonction 
+    * d'une nouvelle requte ADQL
+    * @return le nouveau plan, ou null sinon
+    */
+   protected boolean redoAdql(String query, final boolean flagWithProp) {
+      if( getAdqlQuery()==null ) return false;
+      String cmd = "get TAP("+getIdPrefix(id)+","+Tok.quote(query,true)+")";
+      
+      // Exccution de la commande en remplacement du plan courant
+      aladin.execAsyncCommand(label+"="+cmd);
+
+      SwingUtilities.invokeLater( new Runnable() {
+         public void run() {
+            Util.pause(500);
+            // Rcupration du plan (pas trs joli tout a !)
+            Plan p = aladin.calque.getPlan(label,1);
+            // Affectation de la mme couleur
+            p.c = c;
+            if( flagWithProp ) {
+//               System.out.println("createProperties => "+p);
+               Properties.createProperties(p);
+            }
+         }
+      });
+      return true;
+   }
+   
+   // Enlve le suffixe ~nn d'un identificateur CDS/P/Simbad~123
+   private String getIdPrefix(String id) {
+      int offset = id.indexOf('~');
+      if( offset<0 ) return id;
+      return id.substring(0,offset);
+   }
+   
    protected boolean isCatalog() { return true; }
    
    protected boolean isTime() { return isCatalogTime(); }
@@ -342,13 +459,32 @@ public class PlanCatalog extends Plan {
       if( getNbTable()>1 ) aladin.calque.splitCatalog(this);
       
       setActivated(true);
-      aladin.view.setRepere(this);
+      
+      // Dplacement de la vue principale si aucune source dedans
+//      aladin.view.setRepere(this);
+      setFirstLocation();
 
      callAllListeners(new PlaneLoadEvent(this, PlaneLoadEvent.SUCCESS, null));
      
      return true;
    }
    
+   // Positionne la vue
+   private void setFirstLocation() {
+     ViewSimple v = aladin.view.getCurrentView();
+     
+     Iterator<Obj> it = pcat.iterator();
+     while( it.hasNext() ) {
+        Obj o = it.next();
+        if( v.isInView(o.raj, o.dej) ) {
+           return;  // Au-moins un objet dj visible dans la vue
+        }
+     }
+     
+     // Rien de visible ? on zoome sur le premier objet
+     if( v.getProj().agree(projd, v))  aladin.view.setRepere(this);
+   }
+   
    /** Dsactive tous les filtres ddies */
    static protected void desactivateAllDedicatedFilters(Aladin aladin) {
       Plan [] allPlan = aladin.calque.getPlans();
@@ -367,7 +503,8 @@ public class PlanCatalog extends Plan {
       Iterator<Obj> it = iterator();
       while( it.hasNext() ) {
          Obj o = it.next();
-         if( !(o instanceof Source) ) continue;
+//         if( !(o instanceof Source) ) continue;
+         if( !o.asSource() ) continue;
          ((Source)o).increaseSourceSize(sens);
       }
    }
@@ -378,7 +515,8 @@ public class PlanCatalog extends Plan {
       Iterator<Obj> it = iterator();
       while( it.hasNext() ) {
          Obj o = it.next();
-         if( !(o instanceof Source) ) continue;
+//         if( !(o instanceof Source) ) continue;
+         if( !o.asSource() ) continue;
          Source s = (Source)o;
          if( s.getLeg()==null ) continue;
          if( !leg.contains(s.getLeg()) ) leg.addElement(s.getLeg());
@@ -391,7 +529,8 @@ public class PlanCatalog extends Plan {
       Iterator<Obj> it = iterator();
       while( it.hasNext() ) {
          Obj o = it.next();
-         if( !(o instanceof Source) ) continue;
+//         if( !(o instanceof Source) ) continue;
+         if( !o.asSource() ) continue;
          Source s = (Source)o;
          if( s.getLeg()!=null ) return s.getLeg();
       }
