@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -34,8 +34,8 @@ import javax.swing.JPanel;
 
 import cds.astro.AstroMath;
 import cds.astro.Astrocoo;
+import cds.astro.Astroframe;
 import cds.astro.Coo;
-import cds.astro.ICRS;
 import cds.astro.Proj3;
 import cds.tools.Util;
 import cds.tools.VOApp;
@@ -263,6 +263,8 @@ public final class PlanField extends Plan {
       pcat       = new Pcat(this,c,aladin.calque,aladin.status,aladin);
       aladin.calque.selectPlan(this);
       objet	 = null;
+      // IL FAUDRAIT AJOUTER LE BODY ICI. ON VA ASSUMER sky POUR LE MOMENT
+      body = BODYSKY;
 
       // set instrument code
       setInstr(instrument);
@@ -364,6 +366,8 @@ public final class PlanField extends Plan {
       pcat = new Pcat(this, c, aladin.calque, aladin.status, aladin);
       aladin.calque.selectPlan(this);
       objet = null;
+      // IL FAUDRAIT AJOUTER LE BODY ICI. ON VA ASSUMER sky POUR LE MOMENT
+      body = BODYSKY;
       instr = -1;
       this.fpBean = fpBean;
       setObjects(fpBean);
@@ -379,6 +383,8 @@ public final class PlanField extends Plan {
       pcat = new Pcat(this, c, aladin.calque, aladin.status, aladin);
       aladin.calque.selectPlan(this);
       objet = null;
+      // IL FAUDRAIT AJOUTER LE BODY ICI. ON VA ASSUMER sky POUR LE MOMENT
+      body = BODYSKY;
 
       instr = -1;
       this.fpBean = fpBean;
@@ -409,6 +415,8 @@ public final class PlanField extends Plan {
          setActivated(true);
          make(co.al,co.del,roll);
       }
+      
+      setRollable( fpBean.isRollable());
    }
 
    protected void setLabel(String s) {
@@ -502,7 +510,7 @@ public final class PlanField extends Plan {
       projd = new Projection(null,Projection.SIMPLE,
             ra,de,/*instr==WFPC2?5:*/instr==MEGACAM || instr==MEGAPRIME ?100:45,
                   250,250,500,
-                  0,false,Calib.TAN,Calib.FK5);
+                  0,false,Calib.TAN,Calib.FK5,this);
       switch(instr) {
          case MEGACAM:
             setRollable(false);
@@ -589,7 +597,8 @@ public final class PlanField extends Plan {
             p.y = Util.tand( (DDE[i]+((k==2||k==3)?1.0:0.0))*DE+gapde -ODE );
 
             if( k>0 ) p.debligne=(Ligne)pcat.o[j-1];
-            if( k<4 ) p.finligne=(Ligne)pcat.o[j+1];
+            if( k<4 ) p.finligne=(Ligne)pcat.o[j+1]; 
+            else p.bout=3;
          }
 
          // CCD labels ?
@@ -711,6 +720,8 @@ public final class PlanField extends Plan {
             a.set(x,y);
             p.raj = a.getLon();
             p.dej = a.getLat();
+            
+//            System.out.println("x,y="+x+","+y+" -> raj,dej="+p.raj+","+p.dej);
          }
       }
 
@@ -718,6 +729,20 @@ public final class PlanField extends Plan {
       Properties.majProp(this);
 
    }
+   
+//   static public void main(String [] argv) {
+//      try {
+//         double x = -9.739519033341476E-5;
+//         double y = -0.011581459001125264;
+//         Proj3 a = new Proj3(Proj3.TAN,0.,0.);
+//         a.set( x,y );
+//         double raj = a.getLon();
+//         double dej = a.getLat();
+//         System.out.println("x,y="+x+","+y+" -> raj,dej="+raj+","+dej);
+//      } catch( Exception e ) {
+//         e.printStackTrace();
+//      }
+//   }
 
    /** Reset the FoV for recomputing all drawing objects (after a translation
     * or a rotation to avoid distortions and repaint it
@@ -1017,7 +1042,8 @@ public final class PlanField extends Plan {
       return aladin.localisation.J2000ToString(c.raj,c.dej,Astrocoo.MAS+3,false);
    }
 
-   private Astrocoo afs = new Astrocoo(new ICRS());    // Frame ICRS (la reference de base)
+//   private Astrocoo afs = new Astrocoo(new ICRS());    // Frame ICRS (la reference de base)
+   private Astrocoo afs = new Astrocoo( Astroframe.create("ICRS") );    // Frame ICRS (la reference de base)
 
    /** Return the FoV center in J2000 sexagesimal coordinates */
    protected String getTarget() {
@@ -1061,7 +1087,7 @@ public final class PlanField extends Plan {
    protected boolean isRollable() { return flagRoll; }
 
    /** Positionne l'attibut Rollable pour l'Aperture */
-   protected void setRollable(boolean b) { this.flagRoll = b; }
+   protected void setRollable(boolean b) { flagRoll = b; }
 
    /** Return true if the FoV is rollable and its roll center is movable */
    protected boolean isCenterRollable() { return flagCenterRoll; }
@@ -1119,7 +1145,7 @@ public final class PlanField extends Plan {
     * Export pointing centers, as a new catalogue plane (dedicated to ALMA footprints)
     */
    protected void exportAlmaPointings() {
-      List<Forme> pointings = new ArrayList<Forme>();
+      List<Forme> pointings = new ArrayList<>();
       for (Obj obj: pcat.o) {
          if (obj instanceof Cercle && ! (obj instanceof Pickle)) {
             pointings.add((Forme)obj);
