@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -22,6 +22,7 @@
 package cds.aladin;
 
 import java.awt.Graphics;
+import java.awt.image.BufferedImage;
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.EOFException;
@@ -42,6 +43,7 @@ import cds.allsky.Constante;
 import cds.allsky.Context;
 import cds.fits.Fits;
 import cds.fits.HeaderFits;
+import cds.moc.Healpix;
 import cds.moc.Moc;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
@@ -430,7 +432,7 @@ public class PlanHealpix extends PlanBG {
       }
 
       Projection p =  new Projection("allsky",Projection.WCS,co.al,co.del,60*4,60*4,250,250,500,500,0,
-            longAsc, defaultProjType,Calib.FK5);
+            longAsc, defaultProjType,Calib.FK5,this);
 
       p.frame = getCurrentFrameDrawing();
 //      if( Aladin.OUTREACH ) p.frame = Localisation.GAL;
@@ -1229,6 +1231,8 @@ public class PlanHealpix extends PlanBG {
 
    
    private int lastOrder=0;
+   static final double DENSPROB = (Math.PI/180.)*(Math.PI/180.);
+   
 //   boolean X=true;
 //   private int histo[] = new int[30];
 
@@ -1258,12 +1262,24 @@ public class PlanHealpix extends PlanBG {
       // On repart toujours du dernier ordre trait car il y a une forte probabilit
       // que deux pixels conscutifs soient de mme ordre
       int maxOrder = (int) log2( newNSideFile );
+      
       for( int i=0; i<=maxOrder; i++,lastOrder++ ) {
          if( lastOrder>maxOrder ) lastOrder=0;
          int gap = 2*(maxOrder-lastOrder);
          long nuniq = Moc.hpix2uniq(lastOrder, npix>>>gap);
+         
          Double a = (Double)partialValues.get( nuniq );
-         if( a!=null ) return a; 
+         double s=DENSPROB;
+         long ncell = 12<<(maxOrder*2);
+         double s1=Healpix.SKYAREA/ncell;
+         
+//         if( nuniq==33845035) {
+//            System.out.println("nuniq="+nuniq+" npix="+npix+" order="+lastOrder+" a="+a+" area="+s+" prob2="+((a*s)*s1));
+//         }
+         if( a!=null ) {
+            a = a*s*s1;
+            return a; 
+         }
       }
       return Double.NaN;
    }
@@ -2081,9 +2097,9 @@ public class PlanHealpix extends PlanBG {
    }
 
    /** Dessin du ciel complet en rapide */
-   protected boolean drawAllSky(Graphics g,ViewSimple v) {
+   protected boolean drawAllSky(Graphics g,ViewSimple v,BufferedImage imgb) {
       local=true;
-      return super.drawAllSky(g,v);
+      return super.drawAllSky(g,v,imgb);
    }
 
    //    static final int FIRST   = 0;
@@ -2173,7 +2189,10 @@ public class PlanHealpix extends PlanBG {
          }
       }
       if( empty ) out=null;
-      else out.writeFITS(file);
+      else {
+         out.addDataSum();
+         out.writeFITS(file);
+      }
 
       return out;
    }
