@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -27,10 +27,12 @@ import java.awt.Graphics2D;
 import java.util.ArrayList;
 import java.util.Iterator;
 
-import cds.moc.Array;
+import cds.moc.Healpix;
 import cds.moc.Moc;
+import cds.moc.Moc1D;
 import cds.moc.MocCell;
 import cds.moc.SMoc;
+import cds.moc.STMoc;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
 import cds.tools.pixtools.Hpix;
@@ -51,9 +53,9 @@ public class PlanMoc extends PlanBGCat {
    static final public int DRAW_FILLIN   = 0x4;      // Remplissage avec aplat de demi-opacit
    static final public int DRAW_PERIMETER     = 0x8; // Trac du primtres
 
-   protected Moc moc = null;                 // Le MOC
+   protected Moc moc = null;        // Le Moc
    protected int wireFrame=DRAW_BORDER | DRAW_FILLIN; // Mode de tracage par dfaut
-   
+           
    static int MAXGAPORDER=3;
    protected int gapOrder=0;
 
@@ -61,35 +63,26 @@ public class PlanMoc extends PlanBGCat {
    protected ArrayList<Hpix> arrayHpix = null;    // Liste des cellules correspondant aux cellules tracs (order courant)
    private ArrayList<Hpix> arrayPeri = null;      // Liste des cellules correspondant au primtre trac (ordre courant)
    
-   public PlanMoc(Aladin a) {
-      super(a);
-      type = ALLSKYMOC;
-   }
+   public PlanMoc(Aladin a) { super(a); type = ALLSKYMOC; }
+   
+   protected int getTimeStackIndex() { return 0; }
 
    /** Cration d'un Plan MOC  partir d'un MOC pr-xistant */
-   protected PlanMoc(Aladin aladin, SMoc moc, String label, Coord c, double radius) {
-      this(aladin,null,moc,label,c,radius);
+   protected PlanMoc(Aladin aladin, SMoc moc, String label, Coord c, double radius,String url) {
+      this(aladin,null,moc,label,c,radius,url);
    }
 
    /** Cration d'un Plan MOC  partir d'un flux */
-   protected PlanMoc(Aladin aladin, MyInputStream in, String label, Coord c, double radius) {
-      this(aladin,in,null,label,c,radius);
+   protected PlanMoc(Aladin aladin, MyInputStream in, String label, Coord c, double radius,String url) {
+      this(aladin,in,null,label,c,radius,url);
    }
    
-   protected PlanMoc(Aladin aladin, MyInputStream in, SMoc moc, String label, Coord c, double radius) {
+   protected PlanMoc(Aladin aladin, MyInputStream in, SMoc moc, String label, Coord c, double radius,String url) {
       super(aladin);
       this.dis   = in;
       this.moc   = moc;
+      this.url = url;
       useCache = false;
-      frameOrigin = Localisation.ICRS;
-      if( moc!=null ) {
-         String f = moc.getSys();
-         frameOrigin = f.equals("E")?Localisation.ECLIPTIC :
-            f.equals("G")?Localisation.GAL:Localisation.ICRS;
-         
-         // Si le MOC est petit, affichage immdiat  la rsolution max
-         if( moc.getSize()<10000 ) setMaxGapOrder();
-      }
       type = ALLSKYMOC;
       this.c = Couleur.getNextDefault(aladin.calque);
       setOpacityLevel(1.0f);
@@ -100,37 +93,51 @@ public class PlanMoc extends PlanBGCat {
       aladin.trace(3,"MOC creation: "+Plan.Tp[type]+(c!=null ? " around "+c:""));
       suite();
    }
-
+   
    /** Recopie du Plan  l'identique dans p1 */
    protected void copy(Plan p1) {
       super.copy(p1);
       PlanMoc pm = (PlanMoc)p1;
       pm.frameOrigin=frameOrigin;
-      pm.moc = moc!=null ? moc.clone() : null;
+      try {
+         pm.moc = moc!=null ? moc.clone() : null;
+      } catch( CloneNotSupportedException e ) {
+         e.printStackTrace();
+      }
       pm.wireFrame=wireFrame;
       pm.gapOrder=gapOrder;
       pm.arrayHpix = arrayPeri = null;
 //      pm.arrayMoc = new Moc[CDSHealpix.MAXORDER+1];
    }
    
+   protected String getHipsFrame() {
+      if( moc==null ) return super.getHipsFrame();
+      String s = moc.getSpaceSys();
+      return s+(s.equals("C")?" => ICRS":s.equals("G")?" => Galatic":s.equals("E")?" => Ecliptic":"");
+   }
+   
    /** Ajoute des infos sur le plan */
    protected void addMessageInfo( StringBuilder buf, MyProperties prop ) {
-      double cov = ((SMoc)moc).getCoverage();
-      double degrad = Math.toDegrees(1.0);
-      double skyArea = 4.*Math.PI*degrad*degrad;
-      ADD( buf, "\n* Space: ",Coord.getUnit(skyArea*cov, false, true)+"^2, "+Util.round(cov*100, 3)+"% of sky");
-      ADD( buf, "\n* Best ang.res: ",Coord.getUnit(((SMoc)moc).getAngularRes()));
+      SMoc m = (SMoc)moc;
+      boolean isEmpty = m.isEmpty();
+      double cov = m.getCoverage();
+      String body = getBody();
+      if( body==null ) body="?";
+      ADD( buf, "\n* Space: ",isEmpty?"--empty--": Coord.getUnit(Healpix.SKYAREA*cov, false, true)+"^2, "+Util.round(cov*100, 3)+"% of "+body);
+      ADD( buf, "\n* Resolution: ",Coord.getUnit(m.getAngularRes()));
       
-      int order = moc.getMocOrder();
-      int drawOrder = getDrawOrder();
-      ADD( buf,"\n","* SMOC order: "+ (order==drawOrder ? order+"" : "draw:"+drawOrder+"/"+order));
+      int order = m.getMocOrder();
+      int drawOrder = isEmpty ? -1 : getDrawOrder();
+      ADD( buf,"\n","* Order: "+ (drawOrder==-1 ? order : order==drawOrder ? order+"" : "draw:"+drawOrder+"/"+order) );
+      if( !isEmpty ) ADD( buf,"\n \nRAM: ",Util.getUnitDisk( moc.getMem() ) );
    }
 
    /** Changement de rfrentiel si ncessaire */
    public SMoc toReferenceFrame(String coordSys) throws Exception {
-      SMoc moc1 = SMoc.convertTo((SMoc)moc,coordSys);
-      if( moc!=moc1 ) {
-         aladin.trace(2,"Moc reference frame conversion: "+moc.getSys()+" => "+moc1.getSys());
+      SMoc m = (SMoc)moc;
+      SMoc moc1 = Util.convertTo(m,coordSys);
+      if( m!=moc1 ) {
+         aladin.trace(2,"Moc reference frame conversion: "+m.getSpaceSys()+" => "+moc1.getSpaceSys());
       }
       return moc1;
    }
@@ -169,13 +176,8 @@ public class PlanMoc extends PlanBGCat {
    
    /** Retourne le Moc.maxOrder rel, mme pour les vieux MOCs dont le Norder est gnralement
     * faux */
-   static protected int getRealMaxOrder(SMoc m) {
-      int nOrder = m.getMaxUsedOrder();
-      if( nOrder<=0 ) return nOrder;
-      Array a;
-      while( ( (a=m.getArray(nOrder))==null || a.getSize()==0) && nOrder>0 ) nOrder--;
-      return nOrder;
-   }
+//   static protected int getRealMaxOrder(SMoc m) { return Math.max(m.getMocOrder(),m.getDeepestOrder()); }
+   static protected int getRealMaxOrder(SMoc m) { return m.getMocOrder(); }
    
    // POUR LE MOMENT ON N'UTILISE PAS ENCORE CETTE FONCTION
    // Cration du Pcat qui contient les lignes du primtre du MOC
@@ -231,10 +233,11 @@ public class PlanMoc extends PlanBGCat {
       int maxOrder = getRealMaxOrder(moc);
       ArrayList<double[]> a = new ArrayList<>();
       if( maxOrder==-1 || moc.isFull() ) return a;
-      SMoc done = new SMoc( moc.getSys(), moc.getMinOrder(),maxOrder );
+//      SMoc done = new SMoc( moc.getSys(), moc.getMinOrder(),maxOrder );
+      SMoc done = moc.dup();
 //      long nside = CDSHealpix.pow2( maxOrder );
 
-      Iterator<Long> it = moc.pixelIterator();
+      Iterator<Long> it = moc.valIterator();
       while( it.hasNext() ) {
          long pix = it.next();
          parcoursBord(maxOrder,moc,done,a,maxOrder,pix,0,0);
@@ -330,12 +333,13 @@ public class PlanMoc extends PlanBGCat {
       long [] voisins = new long[4];
       long [] neib = CDSHealpix.neighbours(order,npix);
       for( int i=0,j=0; i<voisins.length; i++, j+=2 ) {
-         voisins[i] = moc.isIn(maxOrder, neib[j]) ? neib[j] : -1;
+//         voisins[i] = moc.isIn(maxOrder, neib[j]) ? neib[j] : -1;
+         voisins[i] = moc.isIncluding(maxOrder, neib[j]) ? neib[j] : -1;
       }
       return voisins;
    }
       
-   protected int getMocOrder() { return moc.getMocOrder(); }
+   protected int getMocOrder() { return ((SMoc)moc).getMocOrder(); }
 
    /** Retourne le Moc */
    protected Moc getMoc() { return moc; }
@@ -386,10 +390,10 @@ public class PlanMoc extends PlanBGCat {
    }
    
    /** Retourne une chaine dcrivant le pourcentage de couverture */
-   public String getPropCoverage() { return Util.myRound(((SMoc)moc).getCoverage()); }
+   public String getPropCoverage() { return Util.myRound(moc.getCoverage()); }
 
    /** Retourne une chaine dcrivant le MOC order */
-   public String getPropMocOrder() { return getRealMaxOrder((SMoc)moc)+""; }
+   public String getPropMocOrder() { return Math.max( ((SMoc)moc).getMocOrder(),getRealMaxOrder( (SMoc)moc))+""; }
 
    protected boolean isCatalog() { return false; }
    protected boolean isTime() { return false; }
@@ -399,32 +403,83 @@ public class PlanMoc extends PlanBGCat {
    
    // Pas de MOc  charger pour un plan Moc
    protected void planReadyMoc() {}
+   
 
+   // Lecture du MOC, avec son entte FITS si possible
+   protected void readMoc(Moc moc, MyInputStream dis ) throws Exception {
+      
+      if(  (dis.getType() & MyInputStream.FITS)!=0 ) {
+         // Lecture de l'entete Fits en se prparant pour reprendre le flux
+         dis.mark(10000);
+         headerFits = new FrameHeaderFits(this,dis);
+         headerFits = new FrameHeaderFits(this,dis);   // On veut la deuximme HDU
+
+         // puis reprise de la lecture complete
+         dis.reset();
+         moc.readFITS(dis);
+         
+      } 
+      else if(  (dis.getType() & MyInputStream.JSON)!=0 ) moc.readASCII(dis);
+      else moc.readASCII(dis);
+      
+   }
+   
+   
+   /** Post traitement aprs lecture ou passage du MOC */
+   protected void analyseMoc(Moc moc) {
+      if( moc==null ) return;
+      String f = moc.getSpaceSys();
+      frameOrigin = f.equals("E")?Localisation.ECLIPTIC : f.equals("G")?Localisation.GAL:Localisation.ICRS;
+      
+      if( moc instanceof SMoc ) {
+         
+         // Je force le MOC minOrder  3 pour que l'affichage soit toujours propre
+         if( ((SMoc)moc).getMinOrder()<3 ) {
+            try {
+               if( ((SMoc)moc).getMocOrder()<3 ) ((SMoc)moc).setMocOrder(3);   
+               ((SMoc)moc).setMinOrder(3);
+            } catch( Exception e ) {
+               if( aladin.levelTrace>=3 ) e.printStackTrace();
+               error="MOC error";
+            }
+         }
+
+         // Si le MOC est petit, affichage immdiat  la rsolution max
+         if( moc.getNbCells()<10000 ) setMaxGapOrder();
+      }
+
+      // Body : cleste ou planto ?
+      if( moc.isSpace() ) {
+         if( f.equals("C") || f.equals("G") || f.equals("E") ) setBody(Plan.BODYSKY);
+         else setBody(f);
+      }
+   }
+
+   
    protected boolean waitForPlan() {
       if( dis!=null ) {
          super.waitForPlan();
          try {
             if( moc==null && dis!=null ) {
                moc = new SMoc();
-               if(  (dis.getType() & MyInputStream.FITS)!=0 ) moc.readFits(dis);
-               else moc.readASCII(dis);
+               readMoc(moc,dis);
             }
-            String c = moc.getSys();
-            frameOrigin = ( c==null || c.charAt(0)=='G' ) ? Localisation.GAL : Localisation.ICRS;
-            if( moc.getSize()==0 ) error="Empty MOC";
          }
          catch( Exception e ) {
             if( aladin.levelTrace>=3 ) e.printStackTrace();
+            error="MOC error";
             return false;
          }
       }
       
+      analyseMoc(moc);
+      
       // Dtermination ou correction du target
       if( !(/* flagNoTarget || */ moc==null || moc.isEmpty()) ) {
          
          // est-ce que le target tombe bien dans le moc ? sinon on va forcer le recalcul sur la premire cellule du MOC
          if( co!=null ) {
-            int order = moc.getMocOrder();
+            int order = ((SMoc)moc).getMocOrder();
             Coord c = Localisation.frameToFrame(co, Localisation.ICRS, frameOrigin);
             double[] radec = CDSHealpix.radecToPolar(new double[] { c.al, c.del });
             try {
@@ -437,8 +492,7 @@ public class PlanMoc extends PlanBGCat {
          if( co==null || co.al==0 && co.del==0  ) {
             try {
                MocCell cell = moc.iterator().next();
-//               long nside = CDSHealpix.pow2(cell.getOrder());
-               double res[] = CDSHealpix.pix2ang_nest(cell.getOrder(), cell.getNpix());
+               double res[] = CDSHealpix.pix2ang_nest(cell.order, cell.start);
                double[] radec = CDSHealpix.polarToRadec(new double[] { res[0], res[1] });
                co = Localisation.frameToFrame( new Coord(radec[0],radec[1]) , frameOrigin, Localisation.ICRS);
                aladin.trace(3,"MOC target (re)computed from the first MOC HEALPix cell => "+co);
@@ -448,18 +502,6 @@ public class PlanMoc extends PlanBGCat {
          }
       }
 
-      // Je force le MOC minOrder  3 pour que l'affichage soit toujours propre
-      if( moc!=null && ((SMoc)moc).getMinOrder()<3 ) {
-         try {
-            if( moc.getMocOrder()<3 ) moc.setMocOrder(3);   
-            ((SMoc)moc).setMinOrder(3);
-         } catch( Exception e ) {
-            if( aladin.levelTrace>=3 ) e.printStackTrace();
-            error="MOC error";
-            return false;
-         }
-      }
-
       return true;
    }
 
@@ -468,10 +510,10 @@ public class PlanMoc extends PlanBGCat {
    protected void resetProj(int n) { }
    protected boolean isDrawn() { return true; }
 
-   // Fournit le MOC qui couvre le champ de vue courant
-   protected SMoc getViewMoc(ViewSimple v,int order) throws Exception {
-      return (SMoc) v.getMoc(order);
-   }
+//   // Fournit le MOC qui couvre le champ de vue courant
+//   protected Moc getViewMoc(ViewSimple v,int order) throws Exception {
+//      return v.getMoc(order);
+//   }
 //   protected SMoc getViewMoc(ViewSimple v,int order) throws Exception {
 //      Coord center = getCooCentre(v);
 //      long [] pix = getPixList(v,center,order);
@@ -538,10 +580,11 @@ public class PlanMoc extends PlanBGCat {
    
    /** Retourne l'order  utiliser pour l'affichage courant en fonction de la position
     * du slider "densit" */
-   protected int getLowOrder(int order, int gapOrder ) {
-      int mo = moc.getSpaceOrder();
+   protected int getLowOrder(int order, int gapOrder) {
+      int mo = -1;
+      mo = moc.getSpaceOrder();
       if( mo==-1 ) {
-         try { mo = getRealMaxOrder( (SMoc) moc); } catch( Exception e ) {  e.printStackTrace(); }
+         try { mo = getRealMaxOrder( (SMoc)moc); } catch( Exception e ) {  e.printStackTrace(); }
       }
       if( mo<3 ) mo=3;
       order += 5;
@@ -549,34 +592,43 @@ public class PlanMoc extends PlanBGCat {
       order += gapOrder;
       if( order<5 ) order=5;
       if( order>mo ) order=mo;
+      
       return order;
    }
    
    // retourne/construit la liste du MOC  l'ordre courant (mode progressif)
    protected Moc getSpaceMocLow1(ViewSimple v,int order,int gapOrder) {
+      lastOrderDrawn = moc.getSpaceOrder();
       if( aladin.NOGUI ) return moc;
-
-      int mo = moc.getMaxUsedOrder();
+      
+      int mo = moc.getDeepestOrder();
       if( mo<3 ) mo=3;
       order = getLowOrder(order,gapOrder);
       
+      lastLowMoc=null;
+      
       // Pour les petits champs, on travaille sans cache, en dcoupant le MOC  la bonne
       // rsolution et  la bonne taille.
       if( v.getTaille()<20 ) {
          try {
             // On amliore un peu la rsolution
-            if( order<moc.getSpaceOrder() ) order++;
+//            if( order<moc.getSpaceOrder() ) order++;
             
             // Peut tre y a-t-il dans le cache des MocLows la Moc complet  la bonne rsolution ?
             // Sinon on prend le MOC d'origine
             Moc mocP = arrayMoc!=null && arrayMoc[order]!=null 
-                  && arrayMoc[order].getSize()>0 ? arrayMoc[order] : moc;
+                  && !arrayMoc[order].isEmpty() ? arrayMoc[order] : moc;
+                  
+            Moc1D mv = v.getMoc();
+            
+            // POUR LE MOMENT ON NE PREND PAS EN COMPTE LE SYSTEME DE REFERENCE
+            mocP.setSpaceSys( mv.getSpaceSys());
             
             // On dcoupe la zone concerne
-            Moc moclow = mocP.intersection( v.getMoc() );
+            Moc moclow = mocP.intersection( mv );
             
-            // On ajuste l'ordre spatial max
-            moclow.setSpaceOrder(order);
+            // Ajustement du MocOrder si on a travaill directement sur le MOC original
+            if( order<moclow.getSpaceOrder() ) moclow.setSpaceOrder(order);
             return moclow;
             
          } catch( Exception e ) {
@@ -591,25 +643,25 @@ public class PlanMoc extends PlanBGCat {
          BuildLow t = new BuildLow(moc,order,mo);
          
          // Si petit, je ne threade pas
-         if( moc.getSize()<100000 || (lastBuildingTime>=0 && lastBuildingTime<30) ) t.run();
+         if( moc.getNbCells()<100000 || (lastBuildingTime>=0 && lastBuildingTime<30) ) t.run();
          else t.start();
-
       }
       // peut tre y a-t-il dj un MOC de plus basse rsolution dj prt
-      if( arrayMoc[order].getSize()==0 ) {
+      if( arrayMoc[order].isEmpty() ) {
          isLoading=true;
          int i=order;
-         for( ; i>=5 && (arrayMoc[i]==null || arrayMoc[i].getSize()==0); i--);
+         for( ; i>=5 && (arrayMoc[i]==null || arrayMoc[i].isEmpty()); i--);
          if( i>=5 ) order=i;
       } else isLoading=false;
 
       lastOrderDrawn = order;
+      lastLowMoc = arrayMoc[order];
       return arrayMoc[order];
    }
    
    protected void initArrayMoc(int order) {
-      if( arrayMoc==null ) arrayMoc = new Moc[Moc.MAXORDER+1];
-      arrayMoc[order] = new SMoc();   // pour viter de lancer plusieurs threads sur le meme calcul
+      if( arrayMoc==null ) arrayMoc = new Moc[SMoc.MAXORD_S+1];
+      arrayMoc[order] = new SMoc(order);   // pour viter de lancer plusieurs threads sur le meme calcul
    }
    
    private long lastBuildingTime=-1;
@@ -628,29 +680,38 @@ public class PlanMoc extends PlanBGCat {
       
       public void run() {
          long t0 = System.currentTimeMillis();
-         Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") running...");
          Moc mocLow=null;
-         if( myOrder==myMo ) mocLow = moc;
+         if( myOrder==myMo && !(moc instanceof STMoc) ) mocLow = moc;
          else {
             // Si dj calcul, on va dgrader le MOC  partir d'un autre  une meilleure rsolution la plus proche
             for( int o=myOrder; o<myMo; o++ ) {
-               if( arrayMoc[o]!=null && arrayMoc[o].getSize()!=0) {
-                  mocLow = arrayMoc[o].clone();
+               if( arrayMoc[o]!=null && !arrayMoc[o].isEmpty()) {
+                  try { mocLow = arrayMoc[o].clone(); } catch( CloneNotSupportedException e ) { e.printStackTrace(); }
                   break;
                }
             }
-            if( mocLow==null ) mocLow = moc.clone();
+            if( mocLow==null ) {
+               try { mocLow = moc.clone(); } catch( CloneNotSupportedException e ) { e.printStackTrace(); }
+            }
          }
 //         Moc mocLow = myOrder==myMo ? moc : moc.clone();
+         
+         // Rduction de la rsolution
          try {
             mocLow.setSpaceOrder(myOrder);
-         } catch( Exception e ) { e.printStackTrace(); }
+            if( moc instanceof STMoc ) {
+               while( mocLow.getTimeOrder()>9 && mocLow.getNbRanges()>10000 ) {
+                  mocLow.setTimeOrder( mocLow.getTimeOrder()-1);
+//                  System.out.println("****mocLow spaceOrder="+mocLow.getSpaceOrder()+" timeOrder="+mocLow.getTimeOrder()+" nbRange="+mocLow.getNbRanges()+" nbCells="+mocLow.getNbCells());
+               }
+            }
+        } catch( Exception e ) { e.printStackTrace(); }
+         
          arrayMoc[myOrder]=mocLow;
          lastBuildingTime = System.currentTimeMillis() - t0;
-         Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") done in "+lastBuildingTime+"ms");
+         Aladin.trace(4,"PlanMoc.getSpaceMocLow1("+myOrder+") done in "+lastBuildingTime+"ms");
          askForRepaint();
       }
-
    }
 
    protected boolean isLoading=false;
@@ -660,9 +721,14 @@ public class PlanMoc extends PlanBGCat {
 
    /** Retourne true si tout a t dessine, sinon false */
    protected boolean hasMoreDetails() {
-      return moc!=null && lastOrderDrawn < moc.getMaxUsedOrder();
+      if( !isReady() ) return false;
+      if( flagProcessing ) return false;
+      return moc!=null && lastOrderDrawn < moc.getDeepestOrder();
    }
    
+   /** Reset de la dernire valeur connue du lastOrderDrawn */
+   private void resetLastOrderDrawn() { } //lastOrderDrawn=-1; }
+   
    /** Retourne l'ordre du dernier MOC dessin effectivement */
    protected int getDrawOrder() { return lastOrderDrawn; }
 
@@ -699,31 +765,35 @@ public class PlanMoc extends PlanBGCat {
    
    // Trac du MOC visible dans la vue
    protected void draw(Graphics g,ViewSimple v) {
+      if( v.isPlot() ) return;
       drawInSpaceView(g,v);
    }
    
    private SMoc lastDrawMoc=null;
    protected SMoc getLastDrawMoc() { return lastDrawMoc; }
+   
+   private Moc lastLowMoc=null;
+   protected Moc getLastLowMoc() { return lastLowMoc; }
 
-      
    protected void drawInSpaceView(Graphics g, ViewSimple v) {
+      if( moc==null ) return;
+      if( v.isPlot() ) return;  // Il ne s'agit pas d'une View space
            
       long t1 = Util.getTime();
       g.setColor(c);
-      int max = Math.min(maxOrder(v),maxOrder)+1;
       
       try {
+         int max = Math.min(maxOrder(v),maxOrder)+1;
          
-         long t=0;
          int myOrder = max+ (v.isAllSky()?0:1);
          
-//         t = System.currentTimeMillis();
+         resetLastOrderDrawn();
          
          int drawingOrder = 0;
          Moc lowMoc = null;
-         boolean flagPeri = isDrawingPerimeter();
+         boolean flagPeri   = isDrawingPerimeter();
          boolean flagBorder = isDrawingBorder();
-         boolean flagFill = isDrawingFillIn();
+         boolean flagFill   = isDrawingFillIn();
 
          int gapOrder = this.gapOrder;
          
@@ -736,27 +806,41 @@ public class PlanMoc extends PlanBGCat {
             lowMoc = getSpaceMocLow(v,myOrder,gapOrder);
             drawingOrder = getRealMaxOrder((SMoc)lowMoc);
             if( drawingOrder==-1 ) return;
-            //            System.out.println("Rcupration Hpix order "+drawingOrder);
-//            long drawingNside = CDSHealpix.pow2(drawingOrder);
+//            System.out.println("Rcupration Hpix drawingOrder="+drawingOrder+" myOrder="+myOrder+" gapOrder="+gapOrder
+//                  +" lowMoc.getMocOrder="+((SMoc)lowMoc).getMocOrder());
+            //            long drawingNside = CDSHealpix.pow2(drawingOrder);
+
+            Moc x = v.getMoc();
+            if( !(x instanceof SMoc) ) {
+               if( aladin.levelTrace>=3 ) {
+                  System.out.println("Bizarre v.getMoc() ne me retourne pas un SMOC => "+x.toDebug());
+                  try {
+                     throw new Exception ();
+                  } catch( Exception e ) { e.printStackTrace(); }
+               }
+               return;
+            }
+            lastDrawMoc =(SMoc) x;
             
-            lastDrawMoc = getViewMoc(v,max);
-            String coordsys = moc.getSys();
+            String coordsys = moc instanceof SMoc ? ((SMoc)moc).getSpaceSys() : "C";
             boolean notEquatorial = moc!=null && coordsys!=null && !coordsys.equals("C");
             SMoc viewMoc = v.isAllSky() || notEquatorial ? null : lastDrawMoc ;
             ArrayList<Hpix> a1 = new ArrayList<>(10000);
             ArrayList<Hpix> a2 = !flagPeri ? null : new ArrayList<Hpix>(10000);
+            
+            
             Iterator<MocCell> it = lowMoc.iterator();
             while( it.hasNext() ) {
                MocCell c = it.next();
-               if( viewMoc!=null && !viewMoc.isIntersecting(c.order, c.npix)) continue;
-               Hpix p = new Hpix(c.order, c.npix, frameOrigin);
-               if( p.isOutView(v) ) continue;
+               if( viewMoc!=null && !viewMoc.isIntersecting(c.order, c.start)) continue;
+               Hpix p = new Hpix(c.order, c.start, frameOrigin);
+               if(  viewMoc==null && p.isOutView(v) ) continue;
                a1.add(p);
-
+               
                if( flagPeri )  {
-                  long [] vo = getVoisinsSameOrder(p.order, (SMoc)lowMoc, p.order, p.npix);
+                  long [] vo = getVoisinsSameOrder(p.order, (SMoc)lowMoc, p.order, p.start);
                   if( vo[0]!=-1 && vo[1]!=-1 && vo[2]!=-1 && vo[3]!=-1 ) continue;
-                  long base = p.npix << 2*(drawingOrder-p.order);
+                  long base = p.start << 2*(drawingOrder-p.order);
                   Bord bord = new Bord((int)CDSHealpix.pow2(drawingOrder-p.order));
                   while( bord.hasNext() ) {
                      long b = bord.next();
@@ -773,6 +857,7 @@ public class PlanMoc extends PlanBGCat {
                }
 
             }
+            
             arrayHpix=a1;
             arrayPeri=a2;
             oiz=v.getIZ();
@@ -780,6 +865,7 @@ public class PlanMoc extends PlanBGCat {
             oGapOrder=gapOrder;
          }
          
+         
          // Trac en aplat avec demi-niveau d'opacit
          if( flagFill && arrayHpix!=null && g instanceof Graphics2D ) {
             Graphics2D g2d = (Graphics2D)g;
@@ -806,14 +892,8 @@ public class PlanMoc extends PlanBGCat {
             for( Hpix p : arrayPeri ) p.draw(g,v);
          }
 
-//         t1 = System.currentTimeMillis();
-//         System.out.println("draw " in "+(t1-t)+"ms"+(n>0 ? " => "+(double)n/(t1-t)+"/ms":"") );
-
-         t = Util.getTime();
-         statTimeDisplay = t-t1;
-
       } catch( Exception e ) {
-         if( Aladin.levelTrace>=3 ) e.printStackTrace();
+         if( Aladin.levelTrace>=4 ) e.printStackTrace();
       }
    }
    
