@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -39,7 +39,7 @@ public class PlanMocColl extends PlanMoc {
    
    protected PlanMocColl(Aladin aladin,String label,String directory,int order,
          boolean strict,boolean recursive,double blank,int [] hdu) {
-      super(aladin,null,null,label,null,0);
+      super(aladin,null,null,label,null,0,null);
       this.directory = directory;
       this.order = order;
       this.strict=strict;
