@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -34,9 +34,8 @@ import java.awt.RenderingHints;
 import java.awt.Stroke;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Enumeration;
+import java.util.ArrayList;
 import java.util.Iterator;
-import java.util.Vector;
 
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
@@ -57,17 +56,22 @@ import cds.tools.Util;
  */
 public class Plot {
    
+   static public final String PROJBODY = "__PLOT__";  // Body bidon associ  la projection du Plot (voir Projection.isUncompatibleBody()
+   
    // les rfrences
    protected Aladin aladin;
    protected ViewSimple viewSimple;
 
    protected Projection plotProj = null;              // la projection du graphique
-   protected Vector<PlotItem> plotTable = null;       // la liste des plots  raliser
+   protected ArrayList<PlotItem> plotTable = null;       // la liste des plots  raliser
    protected boolean flagTime=false;                  // true si l'absisse est du temps
    
    public Plot(ViewSimple viewSimple) {
       this.aladin=viewSimple.aladin;
       this.viewSimple = viewSimple;
+      
+      // Pour pouvoir passer une rfrence  un plan pour les projections utilises dans les plot
+      if( planFree==null ) planFree= new PlanFree(aladin,PROJBODY);
    }
    
    /** Retourne la projection associe au plot */
@@ -94,7 +98,7 @@ public class Plot {
       } else {
          if( hasTable() ) {
             for( int i=0; i<2 && i<plotTable.size(); i++ ) { 
-               PlotItem p = plotTable.elementAt(i);
+               PlotItem p = plotTable.get(i);
                if( s.length()>0 ) s.append(" + ");
                try {
                   Legende leg = p.plan.getFirstLegende();
@@ -143,14 +147,14 @@ public class Plot {
 
    // initialisation du plot
    protected void initPlot() {
-      if( plotTable==null ) plotTable = new Vector<>(10);
+      if( plotTable==null ) plotTable = new ArrayList<>(10);
    }
    
    // recopie du plot pour la vue passe en paramtre
    protected Plot copyIn(ViewSimple v) {
       Plot plot = new Plot(v);
       plot.plotProj = plotProj;
-      plot.plotTable = plotTable==null ? null : (Vector) plotTable.clone();
+      plot.plotTable = plotTable==null ? null : (ArrayList) plotTable.clone();
       plot.flagTime = flagTime;
       return plot;
    }
@@ -183,7 +187,7 @@ public class Plot {
                double [] val = null;
                while( it.hasNext() ) {
                   Obj o = it.next();
-                  if( !(o instanceof Source) ) continue;
+                  if( !o.asSource() ) continue;
                   Source s = (Source)o;
                   val = getValues(val,s);
                   if( Double.isNaN(val[0]) || Double.isNaN(val[1]) ) continue;
@@ -235,16 +239,16 @@ public class Plot {
             
             // Pas de sries temporelles ? on donne tout de mme une largeur pour que la projection
             // fonction pour les TMOC ventuels.
-            if( flagNoSerie && !first ) {  max1Y = h; min1Y=0; flagTime=true; }
+            if( flagNoSerie && !first ) {  max1Y = h; min1Y=0; setTimePlot( true ); }
          }
          
          memoMaxTimeRange(min1X, max1X, min1Y, max1Y);
          
          if( plotProj!=null ) {
             plotProj = new Projection(0,0,0,0, (max1X-min1X), (max1Y-min1Y), w, h, 
-                  plotProj.isFlipXPlot(), plotProj.isFlipYPlot(), plotProj.isLogXPlot(), plotProj.isLogYPlot());
+                  plotProj.isFlipXPlot(), plotProj.isFlipYPlot(), plotProj.isLogXPlot(), plotProj.isLogYPlot(),planFree);
          } else {
-            plotProj = new Projection(0,0,0,0, (max1X-min1X), (max1Y-min1Y), w, h, false,false,false,false);
+            plotProj = new Projection(0,0,0,0, (max1X-min1X), (max1Y-min1Y), w, h, false,false,false,false,planFree);
          }
          viewSimple.newView(1);
          viewSimple.setZoomRaDec(1, (min1X+max1X)/2, (min1Y+max1Y)/2);
@@ -256,6 +260,8 @@ public class Plot {
       }
    }
    
+   static private PlanFree planFree = null;
+   
    private double [] maxTimeRange = null;
    private void memoMaxTimeRange(double jdmin, double jdmax, double ymin, double ymax ) { 
       maxTimeRange = new double [] {jdmin,jdmax,ymin,ymax};
@@ -273,7 +279,8 @@ public class Plot {
       double max1X = Double.isNaN( t[1] ) ? maxTimeRange[1] : t[1];
       
       plotProj = new Projection(0,0,0,0, (max1X-min1X), plotProj.rm1, w, h, 
-            plotProj.isFlipXPlot(), plotProj.isFlipYPlot(), plotProj.isLogXPlot(), plotProj.isLogYPlot());
+            plotProj.isFlipXPlot(), plotProj.isFlipYPlot(), plotProj.isLogXPlot(), plotProj.isLogYPlot(),planFree);
+
       viewSimple.setZoomRaDec(1, (min1X+max1X)/2, (maxTimeRange[2]+maxTimeRange[3])/2);
       viewSimple.newView(1);
    }
@@ -357,7 +364,7 @@ public class Plot {
       PlotItem p = findPlotTable(plan);
       boolean modify=true;
       boolean isTime=false;
-      if( p==null ) { p = new PlotItem(); plotTable.addElement(p); modify=false; }
+      if( p==null ) { p = new PlotItem(); plotTable.add(p); modify=false; }
       if( indexX==-1 || indexY==-1 ) {
          Legende leg = plan.getFirstLegende();
          indexX=leg.getTime();
@@ -365,21 +372,26 @@ public class Plot {
          if( !isTime ) {
             indexX = leg.getIndexNumericField();
             indexY = leg.getIndexNumericField(indexX);
+            
+         // Si c'est un graphe temporel, on va d'abord chercher une colonne de flux pour l'ordonne
          } else {
-            indexY = leg.getIndexNumericField();
-            if( indexY==indexX ) indexY = leg.getIndexNumericField(indexY);
+            indexY = leg.getIndexFluxField();
+            if( indexY==-1 ) {
+               indexY = leg.getIndexNumericField();
+               if( indexY==indexX ) indexY = leg.getIndexNumericField(indexY);
+            }
          }
+      } else {
+         Legende leg = plan.getFirstLegende();
+         isTime = leg.field[ indexX ].isTime();
       }
-      p.flagIsTime= isTime;
-      p.index[0]  = indexX;
-      p.index[1]  = indexY;
-      p.plan      = plan;
+      p.set( plan, indexX, indexY, isTime);
       
-      aladin.trace(4,"ViewSimple.addPlotTable: "+(modify?"modify":"add")+" plan="+plan.label+" indexX="+indexX+" indexY="+indexY);
+      aladin.trace(4,"ViewSimple.addPlotTable: "+(modify?"modify":"add")+" plan="+plan.label+" indexX="+indexX+" indexY="+indexY+" isTime="+isTime);
       
       final boolean flagMainPlot=isMainPlot(p);
       final boolean flagIsTime=isTime;
-      if( flagMainPlot ) this.flagTime=isTime;
+      if( flagMainPlot ) setTimePlot( isTime );
       adjustPlot(p);
       if( openProp ) {
          SwingUtilities.invokeLater( new Runnable() {
@@ -389,7 +401,7 @@ public class Plot {
                if( flagMainPlot && flagIsTime ) timePlotButton.setEnabled(false);
             }
          });
-      }
+      } 
       return new int[] { indexX, indexY };
    }
    
@@ -418,19 +430,31 @@ public class Plot {
    private JCheckBox flipX=null, flipLog=null;
    
    // Choisit automatiquement la colonne de temps en abscisse
-   private void toTimePlot(PlotItem p) {
+   protected void toTimePlot(PlotItem p) {
       Legende leg = p.plan.getFirstLegende();
       int nindex=leg.getTime();
       if( nindex<0 ) return;
       p.setIndex(0,nindex);
       comboX.setSelectedItem( leg.getNameAndDescription(nindex) );
-      flagTime = p.isTime();
+      setTimePlot( p.isTime() );
       adjustPlot();
       viewSimple.repaint();
    }
    
+   private void setTimePlot( boolean flag ) {
+      flagTime = flag;
+      if( !flag ) {
+      try {
+         throw new Exception();
+      } catch( Exception e ) {
+         e.printStackTrace();
+      }
+      }
+      
+   }
+   
    // Mets  jour l'tat des widgets de contrle
-   private void adjustWidgets() {
+   private void adjustWidgets( ) {
       if( timePlotButton!=null ) timePlotButton.setEnabled( !flagTime );
       if( flipX!=null ) flipX.setEnabled( !flagTime );
       if( flipLog!=null ) flipLog.setEnabled( !flagTime );
@@ -498,7 +522,7 @@ public class Plot {
 //            Aladin.trace(4,"getPlotControlPanelForOneIndex: plan="+p.plan.label+" table="+p.leg.name+" col="+n+" index="+nindex);
             if( nindex==p.index[ n ] ) return;
             p.setIndex(n,nindex);
-            flagTime = p.isTime();
+            setTimePlot( p.isTime() );
             adjustPlot();
             viewSimple.repaint();
          }
@@ -620,9 +644,12 @@ public class Plot {
                ((Graphics2D)g).setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, aladin.view.opaciteGrid));
                ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING,  RenderingHints.VALUE_ANTIALIAS_ON);
                st = ((Graphics2D)g).getStroke();
-               float epaisseur = w<1500 ? 0.5f : w/1500f;
-               if( epaisseur<0.5f ) epaisseur=0.5f;
-               else if( epaisseur>2f ) epaisseur=2f;
+               float epaisseur = aladin.configuration.getGridThickness();
+               if( epaisseur<=0 ) {
+                  epaisseur = w<1500 ? 0.5f : w/1500f;
+                  if( epaisseur<0.5f ) epaisseur=0.5f;
+                  else if( epaisseur>2f ) epaisseur=2f;
+               }
                ((Graphics2D)g).setStroke(new BasicStroke(epaisseur));
            }
 
@@ -650,7 +677,8 @@ public class Plot {
                      if( plotProj.isLogXPlot() )  s1 = i<0 ? "1e"+i : i==0 ? "1" : "1e+"+i;
                      else s1 = c.al+"";
                   }
-                  g.drawString(Util.myRound(s1,nbRoundX),p1.x+2+dx, h-5+dy);
+                  String s2 = s1.indexOf(':')>=0 ? s1 : Util.myRound(s1,nbRoundX);
+                  g.drawString(s2,p1.x+2+dx, h-5+dy);
                }
                if( p1==null || p1.x+dx>w ) break;
                if( plotProj.isLogXPlot() ) incrX*=10;
@@ -689,10 +717,8 @@ public class Plot {
 
    // Retourne le PlotItem concernant le plan pass en paramtre sinon null
    protected PlotItem findPlotTable(Plan plan) {
-      if( ! hasTable() ) return null;
-      Enumeration<PlotItem> e = plotTable.elements();
-      while( e.hasMoreElements() ) {
-         PlotItem p = e.nextElement();
+      if( !hasTable() ) return null;
+      for( PlotItem p : plotTable ) {
          if( p.plan==plan ) return p;
       }
       return null;
@@ -724,6 +750,13 @@ public class Plot {
       int [] index = new int[2];
       boolean flagIsTime=false;
       
+      void set(Plan p, int indexX, int indexY, boolean isTime ) {
+         plan      = p;
+         index[0]  = indexX;
+         index[1]  = indexY;
+         flagIsTime= isTime;
+      }
+
       void setIndex(int n, int nindex) {
          
          // si c'est une ordonne, cas trivial 
@@ -731,19 +764,16 @@ public class Plot {
          
          // Sinon il faut vrifier que ce n'est pas le champ temporel, et si c'est le cas l'indiquer
          else {
+            index[0]=nindex;
             Legende leg = plan.getFirstLegende();
-            if( flagIsTime=(leg!=null && leg.getTime()==nindex) ) {
-               index[0] = -nindex;   // on l'enregistre alors en valeur ngative ( tout hasard si besoin)
-            } else index[0]=nindex;
+            flagIsTime=(leg!=null && leg.getTime()==nindex);
          }
-
       }
       
+      // Retourne true s'il s'agit d'un plot temporel
       boolean isTime() { return flagIsTime; }
       
-      public String toString() { return "Plot["+plan.label+"] index="+index[0]+","+index[1]+" flagisTime="+flagIsTime; }
-      
-      
+      public String toString() { return "Plot["+(plan==null?"null":plan.label)+"] index="+index[0]+","+index[1]+" flagisTime="+flagIsTime; }
    }
 
    
