@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -494,7 +494,8 @@ class Printer implements Runnable {
          Source o=null;
          while( it.hasNext() ) {
             Obj o1 = it.next();
-            if( !(o1 instanceof Source) ) continue;
+//            if( !(o1 instanceof Source) ) continue;
+            if( !o.asSource() ) continue;
             o = (Source)o1;
             break;
          }
