@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -36,6 +36,7 @@ import java.awt.Point;
 import java.awt.TextField;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.event.KeyAdapter;
 import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.awt.event.MouseEvent;
@@ -153,6 +154,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
    Couleur[] contoursCouleurs;   // Pour PlanContour, pour choisir la couleur de chaque contour
    Curseur curs = null;          // Pour PlanContour, permet de changer la valeur des niveaux
    JButton filterButtons[]=null;	 // Pour plan CATALOG, boutons pour filtres prdfinis
+   JTextArea adql=null;          // Champ de saisie de la requte ADQL
    JPanel panelCont;
    JPanel panelScroll;
    JScrollPane scroll;
@@ -436,7 +438,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
    // Transforme le plan SMOC en STMOC avec un intervalle de temps par dfaut
    private boolean moc2STmoc() {
       try {
-         plan=aladin.calque.moc2STMoc( (PlanMoc)plan ,24, timeField.getJdmin(), timeField.getJdmax());
+         plan=aladin.calque.moc2STMoc( (PlanMoc)plan ,54, timeField.getJdmin(), timeField.getJdmax());
          aladin.view.repaintAll();
          SwingUtilities.invokeLater( new Runnable() {
             public void run() {
@@ -506,7 +508,6 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          try { p.changeTimeRange( jdmin, jdmax ); } catch( Exception e ) { e.printStackTrace(); }
          start.setForeground( Color.black );
          end.setForeground( Color.black );
-         aladin.calque.resetTimeRange();
          aladin.view.repaintAll();
      }
                
@@ -685,10 +686,29 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          // Une ligne de separation
          if( plan.isCatalog() || plan.isImage() ) PropPanel.addFilet(p,g,c);
          
+         // La requte ADQL, et les mcanismes ncessaires pour la rejouer
          if (plan.isCatalog() && plan.query != null) {
-            JTextArea ta = new JTextArea(plan.query, 5, 40);
+            final JButton b1 = new JButton(" Re-submit ");
+            b1.setEnabled(false);
+            
+            final JTextArea ta = new JTextArea(plan.query, 5, 40);
+            adql = ta;
+            ta.addKeyListener( new KeyAdapter() {
+               public void keyReleased(KeyEvent e) {
+                  b1.setEnabled( !plan.query.equals(ta.getText()) );
+               }
+            });
             JScrollPane sc = new JScrollPane( ta );
-        	PropPanel.addCouple(p,"ADQL query ", sc, g,c);
+            
+            JPanel pa = new JPanel( new BorderLayout(5,5) );
+            JLabel la = new JLabel("ADQL query: ");
+            la.setFont(la.getFont().deriveFont(Font.ITALIC));
+            pa.add( la, BorderLayout.NORTH);
+            b1.addActionListener( new ActionListener() {
+               public void actionPerformed(ActionEvent e) {  redoAdql(); }
+            });
+            pa.add(b1, BorderLayout.SOUTH);
+        	PropPanel.addCouple(p,pa, sc, g,c);
 		}
       }
 
@@ -747,8 +767,8 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
       // Plan MultiExtension
       // Bouton de recuperation de visualisation du header FITS
-      if( plan instanceof PlanFolder  && ((PlanFolder)plan).headerFits!=null ||
-            plan instanceof PlanCatalog && ((PlanCatalog)plan).headerFits!=null ) {
+      if( plan.headerFits!=null 
+            && (plan instanceof PlanFolder|| plan instanceof PlanCatalog ) ) {
          PropPanel.addCouple(p,"Fits extension", b=new JButton(SEEFITS), g,c);
          b.addActionListener(this);
       }
@@ -766,9 +786,9 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          // Format d'image
          JLabel fmtl = new JLabel(
                plan instanceof PlanHealpix ? "HEALPix map" :
-               plan instanceof PlanSTMoc ?   "Space Time Multi-Order Coverage map (STMOC)" :
-               plan instanceof PlanTMoc ?    "Temporal Multi-Order Coverage map (TMOC)" :
-               plan instanceof PlanMoc ?     "Spatial Multi-Order Coverage map (MOC)" :
+               plan instanceof PlanSTMoc ?   "Space Time MOC (STMOC)" :
+               plan instanceof PlanTMoc ?    "Temporal MOC (TMOC)" :
+               plan instanceof PlanMoc ?     "Spatial MOC (MOC)" :
                plan instanceof PlanBG ?      "Hierarchical Progressive Survey (HiPS)" :
                         PlanImage.describeFmtRes(pimg.dis,pimg.res));
 
@@ -1049,37 +1069,36 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
       if( plan.isMoc() ) {
          final PlanMoc pmoc = (PlanMoc)plan;
-//         final Frame frameProp = this;
+//         final Frame frameProp = this;         
          
          if( plan.type==Plan.ALLSKYMOC ) {
             double cov = ((SMoc)pmoc.getMoc()).getCoverage();
             double degrad = Math.toDegrees(1.0);
             double skyArea = 4.*Math.PI*degrad*degrad;
-            PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of sky => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
+            String body = plan.getBody();
+            if( body==null ) body="?";
+            PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of "+body+" => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
             PropPanel.addCouple(p,"Best ang.res: ",new JLabel(Coord.getUnit(((SMoc)pmoc.getMoc()).getAngularRes())
-                  +" (order="+pmoc.getMoc().getMocOrder()+")"),g,c);
-            
-            if( Aladin.BETA ) {
-               if( timeField!=null ) {
-                  PropPanel.addCouple(p,"Time : ", timeField, g,c);
-               } else {
-                  JButton t = new JButton("Range?");
-                  t.addActionListener(new ActionListener() {
-                     public void actionPerformed(ActionEvent e) { createTimeField();  }
-                  });
-                  PropPanel.addCouple(p,"Time",t,g,c);
-               }
+                  +" (order="+((SMoc)pmoc.getMoc()).getMocOrder()+")"),g,c);
+
+            if( timeField!=null ) {
+               PropPanel.addCouple(p,"Time : ", timeField, g,c);
+            } else {
+               JButton t = new JButton("Range?");
+               t.addActionListener(new ActionListener() {
+                  public void actionPerformed(ActionEvent e) { createTimeField();  }
+               });
+               PropPanel.addCouple(p,"Time",t,g,c);
             }
          }
 
          else if( plan.type==Plan.ALLSKYTMOC ) {
             TMoc moc = (TMoc) ((PlanTMoc)plan).moc;
-            long nbSec = moc.getNbCells();
-            int order = moc.getMocOrder();
+            long nbSec = moc.getNbValues();
             PropPanel.addCouple(p,"Start", new JLabel( Astrodate.JDToDate( moc.getTimeMin()) ), g,c);
             PropPanel.addCouple(p,"End", new JLabel( Astrodate.JDToDate( moc.getTimeMax()) ), g,c);
             PropPanel.addCouple(p,"Accuracy", new JLabel(
-                  TMoc.getTemps(  TMoc.getDuration(moc.getMocOrder()))), g,c);
+                  Util.getTemps(  TMoc.getDuration(moc.getMocOrder()))), g,c);
 //            PropPanel.addCouple(p,"Sum", new JLabel( Util.getTemps(nbSec*1000000, true) ), g,c);
          }
 
@@ -1089,13 +1108,14 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                double cov = spaceMoc.getCoverage();
                double degrad = Math.toDegrees(1.0);
                double skyArea = 4.*Math.PI*degrad*degrad;
-               PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of sky => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
+               String body = plan.getBody();
+               if( body==null ) body="?";
+               PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of "+body+" => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
                PropPanel.addCouple(p,"Best ang.res: ",new JLabel(Coord.getUnit(spaceMoc.getAngularRes())
                      +" (order="+spaceMoc.getMocOrder()+")"),g,c);
 
                TMoc timeMoc =  ((STMoc) ((PlanMoc)plan).moc).getTimeMoc();
-               long nbSec = timeMoc.getNbCells();
-               int order = timeMoc.getMocOrder();
+               long nbSec = timeMoc.getNbValues();
                if( ((PlanSTMoc)plan).isOneTimeRange() ) {
                   PropPanel.addCouple(p,"Time : ", new TimeField((PlanSTMoc)plan), g,c);
 
@@ -1103,7 +1123,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                   PropPanel.addCouple(p,"Start", new JLabel( Astrodate.JDToDate( timeMoc.getTimeMin()) ), g,c);
                   PropPanel.addCouple(p,"End", new JLabel( Astrodate.JDToDate( timeMoc.getTimeMax()) ), g,c);
                   PropPanel.addCouple(p,"Time Accuracy", new JLabel( 
-                        TMoc.getTemps(  TMoc.getDuration(timeMoc.getMocOrder()))), g,c);
+                        Util.getTemps(  TMoc.getDuration(timeMoc.getMocOrder()))), g,c);
                }
                
             } catch( Exception e1 ) {
@@ -1246,8 +1266,10 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                if( sS==null && pbg.moc!=null ) sS = pbg.moc.getCoverage()+"";
                JPanel p1 = new JPanel();
                if( sS!=null ) {
+                  String body = pbg.getBody();
+                  if( body==null ) body="?";
                   try { sS = Util.myRound( Double.parseDouble(sS)*100); } catch( Exception e) {}
-                  p1.add( new JLabel(sS+" % of sky"));
+                  p1.add( new JLabel(sS+" % of "+body));
                }
                if( hasMoc ) {
                   JButton bt = new JButton(aladin.MOC);
@@ -1364,7 +1386,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                else  longitudeDescending.setSelected(true);
                plong.add( longitudeAscending );
                plong.add( longitudeDescending );
-               if( Aladin.BETA ) PropPanel.addCouple(p,".longitude", plong, g,c);
+               PropPanel.addCouple(p,".longitude", plong, g,c);
 
             }
          }
@@ -1405,7 +1427,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                      if( !(pi instanceof PlanImage) ) return;
                      Astrotime t = new Astrotime();
                      t.set( ((PlanImage)pi).getDateObs() );
-                     System.out.println("Epoch="+t+" => "+t.toString("J"));
+//                     System.out.println("Epoch="+t+" => "+t.toString("J"));
                      epField.setText(t.toString("J"));
                      apply();
                   } catch( ParseException e ) {
@@ -1710,6 +1732,19 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          planRefChoice.setSelectedIndex(j);
       }
    }
+   
+   private void redoAdql() {
+      try {
+         ((PlanCatalog)plan).redoAdql( adql.getText().replace('\n',' '), true );
+         SwingUtilities.invokeLater(new Runnable() {
+            public void run() {
+               dispose();
+            }
+         });
+      } catch( Exception e ) {
+         e.printStackTrace();
+      }
+   }
 
    protected void majProjInitCat() {
       if( !plan.isCatalog() ) return;
@@ -1928,6 +1963,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
    }
 
    private void apply() {
+      
       setFlagFullMaj(true);// Pour que toutes les fenetres soient maj par la suite
       if( label!=null && !label.getText().equals(plan.label) ) {
          plan.setLabel(label.getText());
@@ -2041,9 +2077,12 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          }
       }
 
-
       if( ! actionSourceType() ) aladin.calque.repaintAll();
-
+      
+      // Une requte ADQL qui aurait t modifie ?
+      if( adql!=null && !plan.query.equals(adql.getText()) ) {
+         redoAdql();
+      }
    }
 
    public void actionPerformed(ActionEvent e) {
