@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -196,12 +196,17 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
     static final protected String MSG_PING = "samp.app.ping";
 
     // liste des messages supports (i.e auxquels on rpond)
-    static final protected String[] SUPPORTED_MESSAGES = {MSG_LOAD_FITS_IMAGE, MSG_POINT_AT_COORDS, MSG_GET_COORDS,MSG_GET_SNAPSHOT,
+    static final protected String[] SUPPORTED_MESSAGES = {
+            MSG_LOAD_FITS_IMAGE, MSG_POINT_AT_COORDS, MSG_GET_COORDS,MSG_GET_SNAPSHOT,
             MSG_LOAD_VOT_FROM_URL,
             MSG_LOAD_FITS_TABLE_FROM_URL, MSG_HIGHLIGHT_OBJECT, MSG_SELECT_OBJECTS,
             MSG_PING, MSG_SEND_ALADIN_SCRIPT_CMD,
             HUB_MSG_SHUTDOWN, HUB_MSG_REGISTRATION, HUB_MSG_UNREGISTRATION,
-            HUB_MSG_SUBSCRIPTIONS, HUB_MSG_DISCONNECT};
+            HUB_MSG_DISCONNECT,
+            HUB_MSG_SUBSCRIPTIONS
+            };
+    
+
 
     // strings associated to reply to a message
     static final protected String MSG_REPLY_SAMP_STATUS = "samp.status";
@@ -288,7 +293,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
         EXCEPTION = a.chaine.getString("PMEXCEPTION").replaceAll("SAMP", name);
         CONFIRM_STOP_HUB = a.chaine.getString("PMCONFIRMSTOPHUB").replaceAll("SAMP", name);
     }
-
+    
     /**
      * Register with the SAMP hub
      *
@@ -299,53 +304,63 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
     public boolean register(boolean silent, boolean launchHubIfNeeded) {
         trace("Try to register Aladin with the SAMP hub");
 
-        if( isRegistered() ) {
-            trace("Aladin was already registered !");
-
-            return true;
-        }
-
-        if( !getHubListener(silent, launchHubIfNeeded) ) {
-            trace("Could not register to the SAMP hub");
-            return false;
-        }
-
-        boolean pbOccured = false;
-        // ###SAMP
-
         Vector params = new Vector();
-        params.add(sampSecret);
         Object result;
-        try {
-            // PF Mai 2017 - Tous les execute(...) ont t remplacs par callAndWait(...)
-            //           Map resultMap = (Map)hubClient.execute(HUB_MSG_REGISTER, params);
-            Map resultMap = (Map)hubClient.callAndWait(HUB_MSG_REGISTER, params);
-            // TODO : se prvenir contre les pbs de cast incorrect et de cl absentes !!
-            myPrivateKey = resultMap.get(KEY_PRIVATE_KEY).toString();
-            selfId = resultMap.get(KEY_SELF_ID).toString();
-            hubId = resultMap.get(KEY_HUB_ID).toString();
-            trace("Aladin has registered, self-id="+selfId);
-        }
-        catch(Exception e) {
-            pbOccured = true;
-        }
-        if( pbOccured ) isRegistered = false;
-        else {
-            isRegistered = true;
-            Aladin.trace(3, "Successful registration with the SAMP hub");
+        
+        // PF - Mar 2021 - J'ai ajout ce verrou exclusif sur la variable isRegistered pour viter
+        // qu'Aladin puisse s'inscrire deux fois. La consquence tait que le message table.highlight.row
+        // tait alors envoy  lui-mme lorsque l'on survolait les sources dans la vue, et donc automatiquement
+        // slectionnes
+        // Le problme n'arrivait que de temps en temps, probablement li  un dlai sur la rponse
+        // du getHubListener(...)
+        synchronized( lockRegistered ) {
+           
+           if( isRegistered() ) {
+              trace("Aladin was already registered !");
+              return true;
+           }
+
+           if( !getHubListener(silent, launchHubIfNeeded) ) {
+              trace("Could not register to the SAMP hub");
+              return false;
+           }
+
+           boolean pbOccured = false;
+           // ###SAMP
+
+           params.add(sampSecret);
+           try {
+              // PF Mai 2017 - Tous les execute(...) ont t remplacs par callAndWait(...)
+              //           Map resultMap = (Map)hubClient.execute(HUB_MSG_REGISTER, params);
+              Map resultMap = (Map)hubClient.callAndWait(HUB_MSG_REGISTER, params);
+              // TODO : se prvenir contre les pbs de cast incorrect et de cl absentes !!
+              myPrivateKey = resultMap.get(KEY_PRIVATE_KEY).toString();
+              selfId = resultMap.get(KEY_SELF_ID).toString();
+              hubId = resultMap.get(KEY_HUB_ID).toString();
+              trace("Aladin has registered, self-id="+selfId);
+              
+           }
+           catch(Exception e) {
+              pbOccured = true;
+           }
+           if( pbOccured ) setRegistered(false);
+           else {
+              setRegistered(true);
+              Aladin.trace(3, "Successful registration with the SAMP hub");
+           }
         }
 
         // declare metadata
         params = new Vector();
         Map map = new Hashtable();
         map.put("samp.name", "Aladin");
-        map.put("samp.description.text", "The Aladin sky atlas");
+        map.put("samp.description.text", "The Aladin Sky Atlas");
         map.put("samp.icon.url", ICON_URL);
-        map.put("samp.documentation.url", "http://aladin.u-strasbg.fr/java/FAQ.htx");
-        map.put("author.name", "Pierre Fernique, Thomas Boch");
-        map.put("author.email", "pierre.fernique@astro.unistra.fr");
+        map.put("samp.documentation.url", "https://"+Aladin.ALADINMAINSITE+"/java/AladinManual.pdf");
+        map.put("author.name", "Pierre Fernique, Thomas Boch, Anais Oberto, Francois Bonnarel, Chaitra");
+        map.put("author.email", "cds-question@astro.unistra.fr");
         map.put("author.affiliation", "CDS, Observatoire astronomique de Strasbourg");
-        map.put("home.page", "http://aladin.u-strasbg.fr/");
+        map.put("home.page", "https://"+Aladin.ALADINMAINSITE);
         map.put("aladin.version", a.VERSION);
         params.add(myPrivateKey);
         params.add(map);
@@ -447,14 +462,14 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
             if( mType.equals(HUB_MSG_SHUTDOWN) ) {
                 trace("hub is shutting down");
                 unregister(true);
-                isRegistered = false;
+                setRegistered(false);
                 updateState();
             }
 
             else if( mType.equals(HUB_MSG_DISCONNECT) ) {
                 trace("Hub wants us to disconnect ... proceed");
                 unregister(true);
-                isRegistered = false;
+                setRegistered(false);
                 a.dontReconnectAutomatically = true;
                 updateState();
             }
@@ -820,7 +835,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
 
             if( force ) {
                 // TODO :  factoriser
-                isRegistered = false;
+                setRegistered(false);
                 selfId = null;
 
                 // arrt du hub interne
@@ -850,7 +865,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
             if( !force ) return false;
         }
 
-        isRegistered = false;
+        setRegistered(false);
         selfId = null;
 
         // arrt du hub interne
@@ -869,9 +884,16 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
 
         return true;
     }
+    
+    private Object lockRegistered = new Object();
 
     public boolean isRegistered() {
-        return isRegistered;
+       synchronized( lockRegistered ) { return isRegistered; }
+    }
+    
+    // PF, pour viter les accs parallle => Aladin pouvait s'inscrire deux fois
+    public void setRegistered(boolean flag) {
+       synchronized( lockRegistered ) { isRegistered=flag; }
     }
 
     // rfrence au hub interne
@@ -1388,7 +1410,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
      */
     static private String readStream(InputStream in) throws IOException {
         try {
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
             for( int c; (c = in.read()) >= 0; ) {
                 sb.append((char)c);
             }
@@ -1404,8 +1426,13 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
     }
 
     private static final Object VOID = "";
-    private static final Object TRUE = Boolean.TRUE;
-    private static final Object FALSE = Boolean.FALSE;
+    
+    // PF 3/7/2022 - Les boolens ne sont pas support par SAMP, uniquement par PLASTIC
+    // Il faut utiliser une chaine "0" ou "1"  la place.
+//    private static final Object TRUE = Boolean.TRUE;
+//    private static final Object FALSE = Boolean.FALSE;
+    private static final Object TRUE = "1";
+    private static final Object FALSE = "0";
 
     /** Select in aladin objects designated by their oid
      *
@@ -1641,7 +1668,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
      */
     private void sendNotification(Map message, final String[] recipients) {
 
-        if( !isRegistered ) {
+        if( !isRegistered() ) {
             return;
         }
 
@@ -2374,7 +2401,7 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
         }
         catch(Exception e) {
             unregister(true);
-            isRegistered = false;
+            setRegistered(false);
             updateState();
             return false;
         }
@@ -2432,9 +2459,6 @@ public class SAMPManager implements AppMessagingInterface, SampXmlRpcHandler, Pl
         return this.widget;
     }
 
-
-
-
     private boolean sampTrace = false;
 
     // mode "trace" pour dbugging PLASTIC
