@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -50,6 +50,7 @@ import javax.swing.JComponent;
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
 import javax.swing.JPopupMenu;
+import javax.swing.Timer;
 
 import cds.tools.Util;
 
@@ -72,7 +73,7 @@ Runnable, SwingWidgetFinder, Widget {
 
    String HSTACK0,HSTACK,HEYE,WAITMIN,NOPROJ,MSELECT,MBROADCASTALL,MALLAPPS,MBROADCASTTABLE,MBROADCASTIMAGE,
    MDEL,MDELALL,MDELALLOTHERS,MDELEMPTY,MCREATFOLD,HSTACK2,
-   MINSFOLD,MCOL,MEXP,MPROP,SHOW,GOTO,BROWSE,HIDE,WARNING,WARNINGSLIDER;
+   MINSFOLD,MCOL,MEXP,MPROP,SHOW,REDO,GOTO,BROWSE,HIDE,WARNING,WARNINGSLIDER;
    String [] BEGIN;
 
    // Les references aux autres objets
@@ -94,7 +95,8 @@ Runnable, SwingWidgetFinder, Widget {
    //   boolean clinDoeil=false;       // Vrai si je dois faire un clin d'oeil
    //   Thread clin=null;              // Thread d'attente pour le clin d'oeil
    Plan newRef=null;              // !=null si on est entrain de changer de ref.
-   Plan planIn=null;		      // si !=null, dernier plan sous la souris
+   Plan planIn=null;              // si !=null, dernier plan sous la souris
+   Plan planBlink=null;           // si !=null, dernier plan qui blink
 
    Thread thread = null;   	  // thread pour pour le clignotement
 
@@ -163,6 +165,7 @@ Runnable, SwingWidgetFinder, Widget {
       GOTO = a.chaine.getString("GOTO");
       BROWSE = a.chaine.getString("BROWSE");
       SHOW = a.chaine.getString("SHOW");
+      REDO = a.chaine.getString("REDO");
       HIDE = a.chaine.getString("HIDE");
       WARNING = a.chaine.getString("SWARNING");
       WARNINGSLIDER = a.chaine.getString("SWARNINGSLIDER");
@@ -231,7 +234,8 @@ Runnable, SwingWidgetFinder, Widget {
    //   public Dimension getPreferredSize() { return new Dimension(ws+5,hs); }
       public Dimension getPreferredSize() { return new Dimension(100,100); }
 
-   JMenuItem menuBroadcast,menuDel,menuDelEmpty,menuDelAll,menuDelAllOthers,menuShow,menuGoto,menuBrowse,
+   JMenuItem menuBroadcast,menuDel,menuDelEmpty,menuDelAll,menuDelAllOthers,menuShow,menuRedo,
+   menuGoto,menuBrowse,
    menuColl,menuCreatFold,menuInsertFold,menuProp,menuSelect,menuUnselect,
    menuConcat1,menuConcat2,menuTableInfo,menuPlot,menuCreateMulti,menuCreateUniq;
 
@@ -243,6 +247,8 @@ Runnable, SwingWidgetFinder, Widget {
       JMenuItem j;
       popMenu.add( menuShow=j=new JMenuItem(SHOW));
       j.addActionListener(this);
+      popMenu.add( menuRedo=j=new JMenuItem(REDO));
+      j.addActionListener(this);
       popMenu.add( menuGoto=j=new JMenuItem(GOTO));
       j.addActionListener(this);
       popMenu.add( menuBrowse=j=new JMenuItem(BROWSE));
@@ -317,6 +323,7 @@ Runnable, SwingWidgetFinder, Widget {
       else if( src==menuCreateMulti )a.cloneObj(false);
       else if( src==menuCreateUniq ) a.cloneObj(true);
       else if( src==menuShow )       a.calque.setActivatedSet(((JMenuItem)src).getActionCommand().equals(SHOW));
+      else if( src==menuRedo )       redo();
       else if( src==menuGoto )       a.view.syncPlan(a.calque.getFirstSelectedPlan());
       else if( src==menuBrowse )     a.command.browse(a.calque.getFirstSelectedPlan().id+"");
       else if( src==menuDel )        a.calque.FreeSet(true);
@@ -401,6 +408,7 @@ Runnable, SwingWidgetFinder, Widget {
       int nbCatalog=0;
       int nbImg=0;
       int nbTool=0;
+      int nbConeSearch=0;
       menuPlan = new Vector(10);
 
       Plan [] plan = a.calque.getPlans();
@@ -412,6 +420,7 @@ Runnable, SwingWidgetFinder, Widget {
          if( pc.type==Plan.TOOL && pc.flagOk ) nbTool++;
          if( pc.type==Plan.IMAGE && pc.flagOk ) nbImg++;
          if( pc.type==Plan.IMAGEHUGE && pc.flagOk ) nbImg++;
+         if( pc.isRedoable() ) nbConeSearch++;
          menuPlan.addElement(pc);
          allShow = allShow && pc.active;
          canBeColl = canBeColl && pc.type==Plan.FOLDER;
@@ -425,6 +434,7 @@ Runnable, SwingWidgetFinder, Widget {
       else menuColl.setText(MEXP);
       if( allShow ) menuShow.setText(HIDE);
       else menuShow.setText(SHOW);
+      menuRedo.setEnabled(nbConeSearch>0);
       menuShow.setEnabled(!a.calque.isFree());
       menuColl.setEnabled(canBeColl && menuPlan.size()>0);
       menuInsertFold.setEnabled(menuPlan.size()>0);
@@ -505,6 +515,7 @@ Runnable, SwingWidgetFinder, Widget {
    }
 
    boolean memoBoutonDroit = false;
+   boolean flagMouseIgnore=false;
    
    /** Gestion de la souris */
    public void mousePressed(MouseEvent e) {
@@ -532,6 +543,15 @@ Runnable, SwingWidgetFinder, Widget {
       // Recherche du plan clique
       if( (currentPlan = getPlan(y))==null ) return;
       
+      // Un double clic sur le logo, sur ca marche
+      if( x>gapL && x<gapL+DX && e.getClickCount()>1 ) { //&& e.getWhen()-lastClick<250) {
+         planIn=currentPlan;
+         planIn.setActivated(true);
+         doBlinkPlan(currentPlan);
+         flagMouseIgnore=true;
+         return;
+      }
+      
       //set current plan in all tap guis by default of what is chosen on plan stack.
       TapManager.getInstance(a).setCurrentUploadPlane(currentPlan.label);
       
@@ -577,9 +597,6 @@ Runnable, SwingWidgetFinder, Widget {
       // Image sans astromtrie non encore pris comme rfrence
       if( !Projection.isOk(currentPlan.projd) && !currentPlan.hasXYorig && !currentPlan.ref ) return x<frMax;
 
-      // Un double clic sur le logo, sur ca marche
-      //      if( x>gapL && x<gapL+DX && e.getClickCount()>1 && e.getWhen()-lastClick<250) return true;
-
       // Un plan dj pris en rfrence visible ne pourra tre repris en rfrence
       if( currentPlan.isRefForVisibleView() ) return false;
 
@@ -848,6 +865,8 @@ Runnable, SwingWidgetFinder, Widget {
    public void mouseReleased(MouseEvent e) {
       Plan newPlan;
       this.x=e.getX(); this.y=e.getY();
+      
+      if( flagMouseIgnore ) { flagMouseIgnore=false; return; }
 
       if( a.inHelp ) { a.helpOff(); return; }
 
@@ -1036,6 +1055,16 @@ Runnable, SwingWidgetFinder, Widget {
       a.calque.repaintAll();
       if( boutonDroit ) showPopMenu(x, y);
    }
+   
+   
+   /** Ractualise le contenu du plan en fonction de la position courante
+    * (uniquement pour les plans concerns)
+    */
+   protected void redo() {
+      for( Plan p : a.calque.getPlanSelected() ) {
+         p.redoConeSearch();
+      }
+   }
 
    // Collapse ou uncollapse de tous les plans d'un folder
    protected void switchCollapseFolder(Plan f) {
@@ -1116,7 +1145,60 @@ Runnable, SwingWidgetFinder, Widget {
       if( canDrawFoVImg() ) lastPlanUnderMouse=p;
       else lastPlanUnderMouse=null;
    }
-
+   
+   private Timer timerBlinkPlan = null;
+   
+   /** Lancement du timer d'attente pour faire clignoter le plan sous la souris */
+   private void startBlinkPlan() {
+      
+      // Lancement du dcompte
+      if( timerBlinkPlan==null ) {
+         timerBlinkPlan = new Timer(2000, new ActionListener() {
+            public void actionPerformed(ActionEvent e) { doBlinkPlan(null) ; }
+         });
+         timerBlinkPlan.setRepeats( false );
+      }
+      
+      timerBlinkPlan.start();
+   }
+   
+   /** Relance d'une attente de plan Blink */
+   private void restartBlinkPlan() {
+      
+      // N'a jamais t lanc => on lance
+      if( timerBlinkPlan==null ) {
+         startBlinkPlan();
+         return;
+      }
+      
+      // Rdmarrage du dcompte
+      timerBlinkPlan.restart();
+   }
+   
+   // Je demande ou non le blink du plan mmoris sous la souris
+   private void doBlinkPlan( Plan p) {
+      
+      // Arrt du prcdent plan qui clignotait
+      if( planBlink!=null && planBlink.isPlanBlink() ) {
+         planBlink.setPlanBlink(false);
+//         System.out.println("J'ai arrt de faire clignoter le plan "+planBlink);
+      }
+      
+      // On sort (avec repaint)
+      if( p==null ) {
+         planBlink=null;
+         a.view.repaintAll();
+         return;
+      }
+      
+      // Lancement du clignotement du nouveau plan
+      planBlink=p;
+//      System.out.println("Je dois faire clignoter le plan sous la souris pdt 4 sec => "+planBlink);
+      planBlink.setPlanBlink(true);
+      restartBlinkPlan();
+      a.view.startTimer();
+      repaint();
+   }
 
    //   private Slide oSlide=null;
 
@@ -1139,7 +1221,7 @@ Runnable, SwingWidgetFinder, Widget {
 
       Slide s = getSlide(y);
       Plan p = s==null?null:s.getPlan();
-
+      
       // Specification du plan sous la souris
       underMouse(p);
 
@@ -1448,8 +1530,11 @@ Runnable, SwingWidgetFinder, Widget {
          String s = p.getMessageInfo();
          if( s==null ) hideMessage();
          else {
-            setMessageInfo( s );
-            messageType=MESSAGE_INFO_PLAN;
+            if( s.startsWith("!!") ) setMessageError(s.substring(2));
+            else {
+               setMessageInfo( s );
+               messageType=MESSAGE_INFO_PLAN;
+            }
          }
       }
    }
@@ -1891,9 +1976,6 @@ Runnable, SwingWidgetFinder, Widget {
       g.setColor( getBackground() );
       g.fillRect(0,0,ws,hs);
 
-      // Le pourtour
-      //      Util.drawEdge(g,ws,hs);
-
       // Le clip Rect pour ne pas depasser
       g.clipRect(2,2,ws-3,hs-3);
 
