@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -355,10 +355,10 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
       status.setForeground( Color.blue );
 
       // Les serveurs Images par programme
-      if( Aladin.NETWORK ) { 
-         aladinServer = new ServerAladin(aladin);
-         if( Aladin.OLD ) sv.addElement(aladinServer);
-      }
+//      if( Aladin.NETWORK ) { 
+//         aladinServer = new ServerAladin(aladin);
+//         if( Aladin.OLD ) sv.addElement(aladinServer);
+//      }
 
       // Les serveurs Images via GLU
       if( Aladin.NETWORK ) addGluServer(sv, Glu.vGluServer, Server.IMAGE);
@@ -1348,7 +1348,10 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
 
 
       //Dans le cas ou il faut effacer la sous-fenetre VizieR
-      if( server[i] instanceof ServerVizieR ) ((ServerVizieR) server[VIZIER]).hideSFrame();
+      //SANS DOUTE DESORMAIS INUTILE/ERRONE DEPUIS LA V10 - JE METS EN TRY/CATCH AU CAS OU
+      try {
+         if( server[i] instanceof ServerVizieR ) ((ServerVizieR) server[VIZIER]).hideSFrame();
+      } catch( Exception e ) { }
 
       // initialisation du focus (Thomas, 16/03/06,  montrer  Pierre)
       // TODO :  vrifier depuis passage  swing
@@ -1395,6 +1398,12 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
          return;
       }
    }
+   
+   public boolean showGenericTapServer() {
+      if (!tapManager.checkDummyInitForServerDialog(tapServer))  return false;
+       setCurrent("TAP");
+       return true;
+   }
 
    // Gestion des evenement
    public boolean action(Event evt, Object what) {
@@ -1403,22 +1412,12 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
 
       // Changement du formulaire
       server[current].memTarget(); // Memorisation du precedent target
-      if ("TAP".equals(what)) {
-    	  if (!tapManager.checkDummyInitForServerDialog(tapServer)) {
-			return false;
-    	  };
-    	 /* if (aladin.glu.lastTapGluServer == null && tapManager.checkDummyTapServer(tapServer)) {
-    		  try {
-				this.tapManager.showTapRegistryForm();
-			} catch (Exception e) {
-				// TODO Auto-generated catch block
-				Aladin.warning(this, Aladin.getChaine().getString("GENERICERROR"));
-				e.printStackTrace();
-			}
-    		  return false;
-		  }*/
-      }
-      setCurrent((String) what);
+      
+      // Cas spcifique pour TAP
+      if ("TAP".equals(what)) showGenericTapServer();
+      
+      // Cas gnral
+      else setCurrent((String) what);
 
       return false;
    }
@@ -1441,52 +1440,4 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
       if( name.equalsIgnoreCase("aladin") ) return cl.getLocation( buttons[ALADIN], this);
       return null;
    }
-
-
-//   public void show() {
-//      super.show();
-////      if( !aladin.PROTO ) startHipsUpdater();
-//   }
-//
-//   public void hide() {
-//      if( !aladin.BETA ) stopHipsUpdater();
-//      super.hide();
-//   }
-//
-//   private Thread threadUpdater=null;
-//   private boolean encore=true;
-//
-//   private void startHipsUpdater() {
-//      if( threadUpdater==null ) {
-//         threadUpdater = new Updater("HipsUpdater");
-//         threadUpdater.start();
-//      } else encore=true;
-//   }
-//
-//   private void stopHipsUpdater() { encore=false; }
-//
-//   class Updater extends Thread {
-//      public Updater(String s) { super(s); }
-//
-//      public void run() {
-//         encore=true;
-//         //         System.out.println("Hips updater running");
-//         while( encore ) {
-//            try {
-//               //               System.out.println("Hips updater checking...");
-//               if( isOpened() ) ((ServerHips)aladin.dialog.hipsServer).hipsUpdate();
-//               Thread.currentThread().sleep(1000);
-//            } catch( Exception e ) { }
-//         }
-//         //         System.out.println("Hips updater stopped");
-//         threadUpdater=null;
-//      }
-//   }
-//
-//   private boolean isOpened() {
-////      Window window = SwingUtilities.windowForComponent(this);
-//      return isVisible();
-//   }
-
-
 }
