@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -42,7 +42,6 @@ import javax.swing.JPanel;
 import javax.swing.JScrollPane;
 import javax.swing.JTextField;
 
-import cds.allsky.Constante;
 import cds.moc.SMoc;
 import cds.moc.STMoc;
 import cds.moc.TMoc;
@@ -247,7 +246,6 @@ public class ServerFile extends Server implements XMLConsumer {
 			Server server, String target, String radius, URL url, String query, int requestId) {
       String serverTaskId = aladin.synchroServer.start("ServerFile.creatLocalPlane/"+label);
       try {
-         //         setSync(false);
          int n=0;
          MyInputStream in=null;
          long type;
@@ -258,16 +256,6 @@ public class ServerFile extends Server implements XMLConsumer {
 
          waitCursor();
          try {
-//            if( label==null ) {
-//               int i = f.lastIndexOf(f.startsWith("http:")||f.startsWith("https:")||f.startsWith("ftp:") ? "/"
-//                     : Util.FS);
-//               label=(i>=0)?f.substring(i+1):f;
-//
-//               // Suppression d'une extension ventuelle
-//               i = label.lastIndexOf('.');
-//               if( i>0 && label.length()-i<=5 ) label = label.substring(0,i);
-//            }
-            
             label = getDefaultLabelIfRequired( label, getLabelFromFile(f) );
 
             // Analyse du contenu d'un rpertoire local
@@ -278,27 +266,17 @@ public class ServerFile extends Server implements XMLConsumer {
                      //                     setSync(true);
                      Aladin.trace(4,"ServerFile.creatLocalPlane("+f+"...) => detect: DIR");
 
-                     if( PlanBG.isPlanBG(f) ) {
+                     // recherche des proprits d'un ventuel HiPS
+                     TreeObjDir gSky=null;
+                     try { gSky = new TreeObjDir(aladin, f, true); } catch( Exception e ) { }
 
-                        // recherche des proprits
-                        TreeObjDir gSky=null;
-                        try { gSky = new TreeObjDir(aladin, f); } catch( Exception e ) { }
+                     // HiPS trouv => on le charge
+                     if( gSky!=null ) {
+                        n=aladin.calque.newPlanBG(gSky,label,target,radius);
 
-                        if( gSky==null ) {
+                     // Ce n'est pas un Hips => exploration du rpertoire
+                     } else {
 
-                           // Progen ?
-                           if( PlanBG.isPlanHpxFinder(f) ) gSky = new TreeObjDir(aladin, null, null, null, null, null,null, null, null, null, null, null, f, "15 progen",null);
-
-                           // Catalogue ?
-                           else if(  (new File(f+"/"+Constante.FILE_METADATAXML)).exists() || (new File(f+"/Norder3/Allsky.xml")).exists() ) {
-                              gSky = new TreeObjDir(aladin, null, null, null, null, null,null, null, null, null, null, null, f, "15 cat",null);
-                           }
-                        }
-
-                        if( gSky!=null ) n=aladin.calque.newPlanBG(gSky,label,target,radius);
-                        else n=aladin.calque.newPlanBG(f,label,target,radius);
-
-                     }  else {
                         final ServerFile th = this;
                         (new Thread(){
                            public void run() {
@@ -338,12 +316,41 @@ public class ServerFile extends Server implements XMLConsumer {
                f = f.replaceAll("\\\\", "/");
             }
 
-            //Obtention du stream (donnee locale ou distante)
+            TreeObjDir gSky=null;
+
+            //Pas de stream, juste une URL en HTTP => dcouverte du contenu et/ou cration du stream
             if( is==null && (f.startsWith("http:")||f.startsWith("https:")) ) {
-               u = aladin.glu.getURL("Http",getNameWithoutBrackets(f),true,true);
+
+               // S'agirait-il d'un HiPS distant ?
+               try {
+                  gSky = new TreeObjDir(aladin, f, true);
+//                  System.err.println("!!! gSky="+gSky.toString1());
+                  n=aladin.calque.newPlanBG(gSky,label,null,null);
+
+               } catch( Exception e ) {
+                  aladin.trace(3, "ServerFile.creatLocalPlane(...) HiPS properties file not found => autodiscovery");
+               }
                
-               try { in = Util.openStream(u); } catch( Exception e ) { if( aladin.levelTrace>=3 ) e.printStackTrace(); }
-               mode="http";
+               // Non ! Alors obtention du stream
+               if( gSky==null ) {
+                  u = aladin.glu.getURL("Http",getNameWithoutBrackets(f),true,true);
+
+                  try { in = Util.openStream(u); } catch( Exception e ) {
+                     String s = e.getMessage();
+                     if( s.startsWith(Util.HTTPERROR) ) {
+                        int code=0;
+                        try { 
+                           code=Integer.parseInt(s.substring(Util.HTTPERROR.length()));
+                        } catch(Exception e1) {}
+                        if( code==400 ) throw new Exception("Bad request ("+code+" HTTP error)"); 
+                        if( code>=401 && code<=403 ) throw new Exception("Data access forbidden ("+code+" HTTP error)"); 
+                        if( code==404 ) throw new Exception("Resource not found ("+code+" HTTP error)"); 
+                        if( code>=405 ||code>=500 ) throw new Exception("Server error ("+code+" HTTP error)"); 
+                     }
+                     if( aladin.levelTrace>=3 ) e.printStackTrace();
+                  }
+                  mode="http";
+               }
             }
 
             // support FTP --> a fonctionne avec par exemple une URL du type ftp://user:passwd@server/....
@@ -358,9 +365,8 @@ public class ServerFile extends Server implements XMLConsumer {
                u = new URL(getNameWithoutBrackets(f));
                in = Util.openStream(u);
                mode="file";
-            }
-
-            else {
+               
+            }  else {
                if( is==null ) {
                   localFile=true;
                   Aladin.trace(3,"Opening "+getNameWithoutBrackets(f));
@@ -374,112 +380,116 @@ public class ServerFile extends Server implements XMLConsumer {
                   else in = new MyInputStream(is);
                }
             }
-            if( in!=null ) {
-               in = in.startRead();
-               type = in.getType();
-            } else type=MyInputStream.UNKNOWN;
+            
+            if( gSky==null ) {
 
-            // Petit rajouti pour reconnaitre l'extension AJS pour les scripts Aladin
-            if( f!=null && f.endsWith(".ajs") ) type |= MyInputStream.AJS;
+               if( in!=null ) {
+                  in = in.startRead();
+                  type = in.getType();
+               } else type=MyInputStream.UNKNOWN;
 
-            // Petit rajouti pour reconnaitre l'extension REG pour les regions DS9
-            if( f!=null && f.endsWith(".reg") ) type |= MyInputStream.AJS;
+               // Petit rajouti pour reconnaitre l'extension AJS pour les scripts Aladin
+               if( f!=null && f.endsWith(".ajs") ) type |= MyInputStream.AJS;
 
-            String t = in.decodeType(type);
-            Aladin.trace(3,(f==null?"stream":f)+" => detect: "+t);
-            aladin.log("load",mode+t);
+               // Petit rajouti pour reconnaitre l'extension REG pour les regions DS9
+               if( f!=null && f.endsWith(".reg") ) type |= MyInputStream.AJS;
 
+               String t = in.decodeType(type);
+               Aladin.trace(3,(f==null?"stream":f)+" => detect: "+t);
+               aladin.log("load",mode+t);
 
-            // Dans le cas d'un chargement d'une rgion ou d'un ancien contour, on va forcer la cration d'un nouveau plan
-            if( (type & (MyInputStream.DS9REG /* |MyInputStream.AJTOOL */))!=0 ) {
-               //               aladin.command.resetPreviousDrawing();
-               aladin.calque.newPlanTool(null);
-            }
+               String ustr = u==null ? null : u.toString();
 
-            if( (type & (MyInputStream.AJS|MyInputStream.AJSx|MyInputStream.UNKNOWN))!=0) aladin.command.readFromStream(in);
-            else if( (type & MyInputStream.AJ)!=0) n=loadAJ(in)?1:0;
-            else if( (type & MyInputStream.AJTOOL)!=0 ) n=loadTool(in,label)?1:0;
-            else if( (type & MyInputStream.IDHA)!=0) n=updateMetaData(in,server,"",null)?1:0;
- 
-            else if( (type & MyInputStream.SMOC)!=0 ) n=aladin.calque.newPlanMOC(in,label);
-            else if( (type & MyInputStream.TMOC)!=0 ) n=aladin.calque.newPlanTMOC(in,label);
-            else if( (type & MyInputStream.STMOC)!=0) n=aladin.calque.newPlanSTMOC(in,label);
-
-            else if( (type & MyInputStream.FITS)!=0 && (type & MyInputStream.RGB)!=0 ) {
-               if( u!=null ) {
-                  n=aladin.calque.newPlanImageRGB(u,in,PlanImage.OTHER,
-                        label,null,f, origin,
-                        PlanImage.UNKNOWN,PlanImage.UNDEF,
-                        null,resNode);
-               } else n=aladin.calque.newPlanImageRGB(f,null,in,resNode);
-            }
-            else if( (type & MyInputStream.HEALPIX)!=0 ) {
-               n=aladin.calque.newPlanHealpix(f,in,label,PlanBG.DRAWPIXEL,0, false,target,radius);
-            }
-            else if( (type & MyInputStream.XFITS)!=0) {
-               aladin.calque.newFitsExt(f,in,label,o,target,radius);
-               n=1;
-            }
-            else if( (type & (MyInputStream.FITS|MyInputStream.PDS))!=0) {
-               if( u!=null ) {
-                  n=aladin.calque.newPlanImage(u,in,PlanImage.OTHER,
-                        label,null,f, origin,
-                        PlanImage.UNKNOWN,PlanImage.UNDEF,
-                        o,resNode);
-                  
-               } else n=aladin.calque.newPlanImage(f,in,label,origin,o,resNode);
-            }
-            else if( (type & MyInputStream.FOV_ONLY) != 0 ) {
-               // un nouveau plan sera cr sur la pile si la description contient les PARAM de position
-               boolean newPlane = (n=aladin.processFovVOTable(in,null,true))>=0;
-               // si on a juste ajout un FOV  la liste des FOV chargs,
-               // on se place sur cet onglet et on slectionne le FOV en question
-               if( !newPlane ) {
-                  n=-2;  // Pour viter une erreur via VOAPP (--> c'est du propre !)
-                  aladin.dialog.setCurrent(ServerDialog.FIELD);
-                  ((ServerFoV)aladin.dialog.server[ServerDialog.FIELD]).selectFOV(ServerFoV.idLastRegistered);
+               // Dans le cas d'un chargement d'une rgion ou d'un ancien contour, on va forcer la cration d'un nouveau plan
+               if( (type & (MyInputStream.DS9REG /* |MyInputStream.AJTOOL */))!=0 ) {
+                  //               aladin.command.resetPreviousDrawing();
+                  aladin.calque.newPlanTool(null);
                }
-            } else if( (type & MyInputStream.DATALINK) != 0 && aladin.mesure.activeDataLinkWord != null) {
-            	aladin.mesure.showAssociatedDatalinks(f, o);
-             }
-            else if( (type & (MyInputStream.ASTRORES|MyInputStream.VOTABLE|
-                  MyInputStream.CSV|MyInputStream.BSV|MyInputStream.IPAC))!=0 ) {
-               if( u!=null ) n=aladin.calque.newPlanCatalog(u,in,label,"",f,null,server);
-               else if( f!=null) n=aladin.calque.newPlanCatalog(f,in);
-               else n=aladin.calque.newPlanCatalog(in,label,origin, server, url, query, requestId);
 
-               // C'est peut tre une image native ?
-            } else if( (type & MyInputStream.NativeImage())!=0 ) {
-               if( u!=null ) {
-                  n=aladin.calque.newPlanImageColor(u,in,PlanImage.OTHER,
-                        label,null,f, origin,
-                        PlanImage.UNKNOWN,PlanImage.UNDEF,
-                        o,resNode);
-               } else n=aladin.calque.newPlanImageColor(f,null,in,resNode);
+               if( (type & (MyInputStream.AJS|MyInputStream.AJSx|MyInputStream.UNKNOWN))!=0) aladin.command.readFromStream(in);
+               else if( (type & MyInputStream.AJ)!=0) n=loadAJ(in)?1:0;
+               else if( (type & MyInputStream.AJTOOL)!=0 ) n=loadTool(in,label)?1:0;
+               else if( (type & MyInputStream.IDHA)!=0) n=updateMetaData(in,server,"",null)?1:0;
 
-               // C'est peut tre un dico GLU ?
-            } else if( (type & MyInputStream.GLU)!=0 ) {
-               if( aladin.glu.loadGluDic(new DataInputStream(in), false,localFile) ) {
-                  aladin.glu.reload(false,true);
+               else if( (type & MyInputStream.SMOC)!=0 ) n=aladin.calque.newPlanMOC(in,label,ustr);
+               else if( (type & MyInputStream.TMOC)!=0 ) n=aladin.calque.newPlanTMOC(in,label,ustr);
+               else if( (type & MyInputStream.STMOC)!=0) n=aladin.calque.newPlanSTMOC(in,label,ustr);
+
+               else if( (type & MyInputStream.FITS)!=0 && (type & MyInputStream.RGB)!=0 ) {
+                  if( u!=null ) {
+                     n=aladin.calque.newPlanImageRGB(u,in,PlanImage.OTHER,
+                           label,null,f, origin,
+                           PlanImage.UNKNOWN,PlanImage.UNDEF,
+                           null,resNode);
+                  } else n=aladin.calque.newPlanImageRGB(f,null,in,resNode);
+               }
+               else if( (type & MyInputStream.HEALPIX)!=0 ) {
+                  n=aladin.calque.newPlanHealpix(f,in,label,PlanBG.DRAWPIXEL,0, false,target,radius);
+               }
+               else if( (type & MyInputStream.XFITS)!=0) {
+                  aladin.calque.newFitsExt(f,in,label,o,target,radius);
                   n=1;
                }
+               else if( (type & (MyInputStream.FITS|MyInputStream.PDS))!=0) {
+                  if( u!=null ) {
+                     n=aladin.calque.newPlanImage(u,in,PlanImage.OTHER,
+                           label,null,f, origin,
+                           PlanImage.UNKNOWN,PlanImage.UNDEF,
+                           o,resNode);
 
-               // C'est peut tre un fichier de properties ?
-            } else if( (type & MyInputStream.PROP)!=0 ) {
-               
-               // Dans le cas d'un fichier properties, on passe le path ou l'url pour 
-               // fournir l'accs au HiPS associ dans le cas o il n'est pas dcrit
-               // par un hips_service_url
-               String path = null;
-               int i = (f==null /* || !mode.equals("file")*/ ) ? -1 : f.lastIndexOf("properties");
-               if( i>0 ) path = f.substring(0,i-1);
-               
-               if( aladin.directory.addHipsProp( new InputStreamReader(in), true, path) ) {
-                  n=1;
+                  } else n=aladin.calque.newPlanImage(f,in,label,origin,o,resNode);
                }
+               else if( (type & MyInputStream.FOV_ONLY) != 0 ) {
+                  // un nouveau plan sera cr sur la pile si la description contient les PARAM de position
+                  boolean newPlane = (n=aladin.processFovVOTable(in,null,true))>=0;
+                  // si on a juste ajout un FOV  la liste des FOV chargs,
+                  // on se place sur cet onglet et on slectionne le FOV en question
+                  if( !newPlane ) {
+                     n=-2;  // Pour viter une erreur via VOAPP (--> c'est du propre !)
+                     aladin.dialog.setCurrent(ServerDialog.FIELD);
+                     ((ServerFoV)aladin.dialog.server[ServerDialog.FIELD]).selectFOV(ServerFoV.idLastRegistered);
+                  }
+               } else if( (type & MyInputStream.DATALINK) != 0 && aladin.mesure.activeDataLinkWord != null) {
+                  aladin.mesure.showAssociatedDatalinks(f, o);
+               }
+               else if( (type & (MyInputStream.ASTRORES|MyInputStream.VOTABLE|
+                     MyInputStream.CSV|MyInputStream.BSV|MyInputStream.IPAC))!=0 ) {
+                  if( u!=null ) n=aladin.calque.newPlanCatalog(u,in,label,"",f,null,server);
+                  else if( f!=null) n=aladin.calque.newPlanCatalog(f,in);
+                  else n=aladin.calque.newPlanCatalog(in,label,origin, server, url, query, requestId);
 
-               // C'est peut tre un planBG via HTTP
-            } else if( mode.equals("http") && f!=null && f.indexOf('?')<0 ) {
+                  // C'est peut tre une image native ?
+               } else if( (type & MyInputStream.NativeImage())!=0 ) {
+                  if( u!=null ) {
+                     n=aladin.calque.newPlanImageColor(u,in,PlanImage.OTHER,
+                           label,null,f, origin,
+                           PlanImage.UNKNOWN,PlanImage.UNDEF,
+                           o,resNode);
+                  } else n=aladin.calque.newPlanImageColor(f,null,in,resNode);
+
+                  // C'est peut tre un dico GLU ?
+               } else if( (type & MyInputStream.GLU)!=0 ) {
+                  if( aladin.glu.loadGluDic(new DataInputStream(in), false,localFile) ) {
+                     aladin.glu.reload(false,true);
+                     n=1;
+                  }
+
+                  // C'est peut tre un fichier de properties ?
+               } else if( (type & MyInputStream.PROP)!=0 ) {
+
+                  // Dans le cas d'un fichier properties, on passe le path ou l'url pour 
+                  // fournir l'accs au HiPS associ dans le cas o il n'est pas dcrit
+                  // par un hips_service_url
+                  String path = null;
+                  int i = (f==null /* || !mode.equals("file")*/ ) ? -1 : f.lastIndexOf("properties");
+                  if( i>0 ) path = f.substring(0,i-1);
+
+                  if( aladin.directory.addHipsProp( new InputStreamReader(in), true, path) ) {
+                     n=1;
+                  }
+
+                  // C'est peut tre un planBG via HTTP
+               } else /* if( mode.equals("http") && f!=null && f.indexOf('?')<0 ) {
 
                // Cubes ?
                TreeObjDir gSky=null;
@@ -521,16 +531,17 @@ public class ServerFile extends Server implements XMLConsumer {
 
                else throw new Exception("Data format not recognized");
 
-            } else {
-               throw new Exception("Data format not recognized");
-            }
-            aladin.endMsg();
+            } else */ {
+                  throw new Exception("Data format not recognized");
+               }
+               aladin.endMsg();
 
-            // Dans le cas de Meta-donnee (SIA ou IDHA) on va automatiquement ouvrir
-            // et positionner la fenetre des formulaires toFront
-            if( (type & (/*MyInputStream.SIA_SSA|*/MyInputStream.IDHA))!=0 ) {
-               aladin.dialog.show();
-               aladin.dialog.setCurrent(aladinLabel);
+               // Dans le cas de Meta-donnee (SIA ou IDHA) on va automatiquement ouvrir
+               // et positionner la fenetre des formulaires toFront
+               if( (type & (/*MyInputStream.SIA_SSA|*/MyInputStream.IDHA))!=0 ) {
+                  aladin.dialog.show();
+                  aladin.dialog.setCurrent(aladinLabel);
+               }
             }
          } catch(Exception e) {
             e.printStackTrace();
@@ -708,6 +719,7 @@ public class ServerFile extends Server implements XMLConsumer {
    private Vector vField;	// Vecteur contenant les FIELDS de la table courante
    private Legende leg=null;	// Legende de la table courante
    private Plan plan=null;	// Plan courant
+   private int footprintIdx=-1; // Indice de colonne d'un ventuel footprint associ  chaque source
    private ViewMemoItem vmi;  // ViewMemoItem courant
    private int firstView;   //indice de la premire vue  afficher
    private double ra=0;		// RA du centre du plan courant
@@ -755,6 +767,8 @@ public class ServerFile extends Server implements XMLConsumer {
       String type = (String)atts.get("type");
       typePlan= Util.indexInArrayOf(type, Plan.Tp);
       inFilter = inFitsHeader = inFilterScript = false;
+      footprintIdx = -1;
+      
       switch(typePlan) {
          case Plan.FILTER:
             PlanCatalog p = null;
@@ -837,7 +851,7 @@ public class ServerFile extends Server implements XMLConsumer {
                if( (s=(String)atts.get("hipsinPNG"))!=null )     pbg.inPNG = (new Boolean(s)).booleanValue();
                if( (s=(String)atts.get("hipshasMoc"))!=null )    pbg.hasMoc = (new Boolean(s)).booleanValue();
                if( (s=(String)atts.get("hipshasHpxFinder"))!=null )   pbg.hasHpxFinder = (new Boolean(s)).booleanValue();
-               if( (s=(String)atts.get("hipsbody"))!=null )      pbg.body = s;
+               if( (s=(String)atts.get("hipsbody"))!=null )      pbg.body = s;  // Pour compatibilit -  virer quand on veut
                if( (s=(String)atts.get("hipsframeOrigin"))!=null )   pbg.frameOrigin = Integer.parseInt(s);
                if( (s=(String)atts.get("hipsframeDrawing"))!=null )  pbg.frameDrawing = Integer.parseInt(s);
                if( (s=(String)atts.get("hipslive"))!=null )      pbg.live = (new Boolean(s)).booleanValue();
@@ -909,6 +923,8 @@ public class ServerFile extends Server implements XMLConsumer {
       // Traitements gnriques
       if( plan!=null ) {
          if( (s=(String)atts.get("label"))!=null )      plan.label = s;
+         if( (s=(String)atts.get("body"))!=null )       plan.body = s;
+
          if( (s=(String)atts.get("depth"))!=null ) plan.folder = Integer.parseInt(s);
          if( (s=(String)atts.get("activated"))!=null ) plan.active=plan.askActive = (new Boolean(s)).booleanValue();
          if( (s=(String)atts.get("selectable"))!=null ) plan.setSelectable( (new Boolean(s)).booleanValue() );
@@ -927,7 +943,7 @@ public class ServerFile extends Server implements XMLConsumer {
             c.xydpoly=parsePoly((String)atts.get("xydpoly"));
             c.CD=parseMat((String)atts.get("CD"));
             c.ID=parseMat((String)atts.get("ID"));
-            plan.setNewProjD(new Projection(proj,c));
+            plan.setNewProjD(new Projection(proj,c,plan));
             if( plan instanceof PlanImage ) ((PlanImage)plan).setHasSpecificCalib();
          }
          if( (s=(String)atts.get("flagepoch"))!=null ) {
@@ -1069,7 +1085,12 @@ public class ServerFile extends Server implements XMLConsumer {
          Enumeration e = atts.keys();
          while( e.hasMoreElements() ) {
             String key = (String)e.nextElement();
-            f.addInfo(key,(String)atts.get(key));
+            String val = (String)atts.get(key);
+            if( key.equals("sregion") && val.equals("true") ) {
+               footprintIdx = vField.size();
+//               System.out.println("footprint in "+footprintIdx+" on "+f);
+            }
+            else f.addInfo(key,val);
          }
          vField.addElement(f);
       }
@@ -1088,9 +1109,11 @@ public class ServerFile extends Server implements XMLConsumer {
                if( pdf==null ) pdf = new Vector();
                pdf.addElement(plan);
             }
-
-            // Post-traitement sur le plan CATALOG
-            //            if( typePlan==Plan.CATALOG || typePlan==CATALOGTOOL ) plan.pcat.postJob(ra,de,rm,true);
+            
+            // Post-traitement sur les plans catalogues HiPS
+            if( typePlan==Plan.ALLSKYCAT ) {
+               ((PlanBGCat)plan).loadGenericLegende();
+            }
 
             // Post-traitement sur le plan IMAGE
             if( typePlan==Plan.IMAGE || typePlan==Plan.IMAGEMOSAIC
@@ -1116,7 +1139,9 @@ public class ServerFile extends Server implements XMLConsumer {
       } else if( name.equals("MODEVIEW") )  {
          aladin.view.scrollOn(firstView,0,1);
       } else if( name.equals("SCRIPT") )  { inFilterScript =false;
-      } else if( name.equals("TABLE") )   { leg=null; vField=null; if( flagCatalogSource ) typePlan=CATALOGTOOL;
+      } else if( name.equals("TABLE") )   { 
+         leg=null; vField=null; 
+         if( flagCatalogSource ) typePlan=CATALOGTOOL;
       } else if( name.equals("VALUE") )   {
          inValue=false;
          if( plan instanceof PlanMoc ) {
@@ -1204,7 +1229,7 @@ public class ServerFile extends Server implements XMLConsumer {
     */
    private int getSource(char [] ch, int cur, int end) {
       double ra,de;
-      String id;
+      String id,glu;
 
       // On skippe un eventuel \r
       while( cur<end && (ch[cur]=='\r' || ch[cur]=='\n') ) cur++;
@@ -1212,7 +1237,8 @@ public class ServerFile extends Server implements XMLConsumer {
       // Recuperation des positions et de l'ID
       cur=getSourceField(ch,cur,end);	ra=Double.valueOf(rec).doubleValue();
       cur=getSourceField(ch,cur,end);	de=Double.valueOf(rec).doubleValue();
-      cur=getSourceField(ch,cur,end);	id=rec;
+      cur=getSourceField(ch,cur,end);   id=rec;
+      cur=getSourceField(ch,cur,end);   glu=rec;
 
       // Recuperation des infos
       int start=cur;
@@ -1223,9 +1249,16 @@ public class ServerFile extends Server implements XMLConsumer {
 
       // Ajout de l'objet dans le plan courant
       Source o = (leg!=null)?new Source(plan,ra,de,id,rec,leg): new Source(plan,ra,de,id,rec);
-
-      // Cas particulier de sources dans un plan tool
-//      if( leg!=null && typePlan==CATALOGTOOL && ((PlanTool)plan).legPhot==null ) ((PlanTool)plan).legPhot=leg;
+      
+      // Par dfaut on active l'affichage du label des tags (du grand bricolage PF 5/9/2022)
+      if( typePlan==CATALOGTOOL && glu!=null && glu.endsWith("|Tags>")) o.setWithLabel(true);
+      
+      // Gnration d'un footprint associ  la source
+      if( footprintIdx!=-1 ) {
+         String fov = o.getCodedValue(footprintIdx);
+         o.setFootprint(fov);
+         o.setIdxFootprint(footprintIdx);
+      }
 
       plan.pcat.setObjetFast(o);
 
@@ -1288,7 +1321,7 @@ public class ServerFile extends Server implements XMLConsumer {
       if( o!=null ) {
          o.raj=ra; o.dej=de;
          o.x=x;    o.y=y;
-
+         
          // Suivi de ligne
          try {
             if( prevFlagSuite ) {
@@ -1340,7 +1373,7 @@ public class ServerFile extends Server implements XMLConsumer {
     * Cette procedure peut etre appelee plusieurs fois en sequence.
     * @param ch le tableau de caracteres a analyser
     * @param cur l'indice du caractere courant
-    * @param end le dernier indice valide
+    * @param hi le dernier indice valide
     */
    private void getImageRGB(char [] ch, int start, int length) {
       PlanImageRGB p = (PlanImageRGB)plan;
@@ -1372,7 +1405,7 @@ public class ServerFile extends Server implements XMLConsumer {
     * Cette procedure peut etre appelee plusieurs fois en sequence.
     * @param ch le tableau de caracteres a analyser
     * @param cur l'indice du caractere courant
-    * @param end le dernier indice valide
+    * @param hi le dernier indice valide
     */
    private void getImage(char [] ch, int start, int length) {
       PlanImage p = (PlanImage)plan;
