@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -299,7 +299,7 @@ public class ServerMocQuery extends Server  {
       File tmp = File.createTempFile("tmp", "fits");
       tmp.deleteOnExit();
       FileOutputStream fo = new FileOutputStream(tmp);
-      try { hpxMoc.writeFits(fo); }
+      try { hpxMoc.writeFITS(fo); }
       finally { try { fo.close(); } catch(Exception e) {} }
       out.writeFile("mocfile", null, tmp, false);
    }
