@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -51,7 +51,8 @@ public class ServerSWarp extends Server  {
 
     private JComboBox comboProj;
 
-    private String baseUrl = "http://alasky.u-strasbg.fr/cgi/SWarp/nph-SWarp.py";
+//    private String baseUrl = "http://alasky.u-strasbg.fr/cgi/SWarp/nph-SWarp.py";
+    private String baseUrl = "http://aladin.u-strasbg.fr/java/post.pl";
 
     // TODO : liste complte  valider avec Caro et Mark
     private String[] availableProjections = {"TAN", "SIN", "ARC", "AIT",
