@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -81,7 +81,7 @@ public class ServerSimbad extends Server  {
                   "}\n" +
                   "${OTYPE}=\"Radio*\" || ${OTYPE}=\"Maser\" || ${OTYPE}=\"HI\" { draw triangle }\n" +
                   "${OTYPE}=\"UV\" {draw cross;draw plus}\n" +
-                  "${OTYPE}=\"IR\" || ${OTYPE}=\"Red*\" {draw rhomb}\n" +
+                  "${OTYPE}=\"IR\" || ${OTYPE}=\"FIR*\" {draw rhomb}\n" +
                   "${OTYPE}=\"Neb\"  || ${OTYPE}=\"PN*\" || ${OTYPE}=\"SNR*\" {draw square}\n" +
                   "${OTYPE}=\"HII\" { draw dot }" +
                   "${OTYPE}=\"X\" { draw cross }\n" +
@@ -113,7 +113,7 @@ public class ServerSimbad extends Server  {
                   "${OTYPE}=\"UV\" {draw cross;draw plus}\n" +
                   "}",
                   "#IR\nfilter IR {\n" +
-                  "${OTYPE}=\"IR\" || ${OTYPE}=\"Red*\" {draw rhomb}\n" +
+                  "${OTYPE}=\"IR\" || ${OTYPE}=\"FIR*\" {draw rhomb}\n" +
                   "}",
                   "#Neb\nfilter Neb {\n" +
                   "${OTYPE}=\"Neb\"  || ${OTYPE}=\"PN*\" || ${OTYPE}=\"SNR*\" {draw square}\n" +
