@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -39,7 +39,7 @@ public final class Slide {
    // Les valeurs accociees aux differents elements graphiques
    static final int gapL =  Select.gapL; // Marge de gauche (reserve pour le triangle)
    static final int DX   =  Select.DX;   // Largeur d'un slide (hors tout)
-   static final int DY   =  19;	         // Hauteur d'un slide (hors tout)
+   static int DY;	                     // Hauteur d'un slide (hors tout)
    static final int DFOLDER=10;		     // Decalage niveau de folder
 
    static final int VIDE=0;
@@ -104,6 +104,10 @@ public final class Slide {
       slides=null;
       p=plan;
       a=aladin;
+      
+      // ajustement de la hauteur inter slide en fonction du facteur d'chelle pour l'UI
+      float x = aladin.getUIScale();
+      DY = Math.round( 19 * x* (x>1.20 ? 0.85f: 1f));
    }
 
    /** retourne le plan associe au slide */
@@ -272,8 +276,10 @@ public final class Slide {
    // Adaptation du logo pour un plan Filter
    static void drawLogoFilter(Graphics g,int dx,int dy,
                               boolean active,boolean button,Color c,Color bord) {
-      int L=button?Tool.W-7:DX;		// Largeur
-      int H=button?Tool.H-24:frY[1]-frY[0];	// Hauteur
+//      int L=button?Tool.W-7:DX;     // Largeur
+//      int H=button?Tool.H-24:frY[1]-frY[0]; // Hauteur
+      int L= DX;     // Largeur
+      int H= frY[1]-frY[0]; // Hauteur
       int i;
       int deb=-3,fin=3;
       int x1=L/2-5,x2=L/2-6;
@@ -428,6 +434,8 @@ public final class Slide {
    // Affichage d'un label en diminuant la luminosit des sparateurs "/" le cas chant
    static protected void drawLabel( Graphics g, String label, int x, int y, Color fg) {
       
+      if( fg==null ) fg = Aladin.COLOR_FOREGROUND;
+      
       // Pas de /, mthode rapide
       if( label.indexOf('/')<0 ) {
          g.setColor(fg);
@@ -541,7 +549,8 @@ public final class Slide {
          }
          
          // Calcul la taille des lettres
-         int ht = Aladin.SIZE;
+         g.setFont( Aladin.PLAIN );
+         int ht = Math.round( Aladin.PLAIN.getSize2D() );
          
          // Determination de la couleur du dessin du logo dans le cas o
          // il y a un plan couleur (choix des trois composantes RGB)
@@ -564,7 +573,7 @@ public final class Slide {
          for( int i=0; i<frX.length; i++ ) xc[i]=frX[i]+dx;
          
          // Paramtres pour tracer le cadre et la bordure
-         int y = dy-1;
+         int y = Math.round( dy-(1+(Aladin.getUIScale()-1)*10) );
          int x = dx+xLabel-1; // 1;
          int H = ht+7;
          int W = Select.ws-x;
@@ -780,8 +789,6 @@ public final class Slide {
          }
          
          // Le libelle
-//         g.setFont( Aladin.BOLD );
-         g.setFont( Aladin.PLAIN );
          if( p.label!=null ) {
             int py = dy+15-1;
             int px = Select.ws-12;
@@ -811,8 +818,8 @@ public final class Slide {
                   setBlink(true);
                   p.status|=Plan.STATUS_INPROGRESS;
                   if( p.pourcent>0 ) drawFolderExt(g,(int)p.pourcent,0,dy,Color.gray);
-               }
-               else if( p.isSimpleCatalog() || p.isImage()
+                  
+               } else if( p.isSimpleCatalog() || p.isImage()
                      || p instanceof PlanContour || p.type==Plan.FILTER
                      || p instanceof PlanBG ) {
                   if( p.error!=null ) {
@@ -822,7 +829,7 @@ public final class Slide {
                      else if( p.isSimpleCatalog() && p.error.indexOf("OVERFLOW")>=0 ) { drawBall(g,px,py-9,Aladin.ORANGE); p.status|=Plan.STATUS_OVERFLOW; }
                      else if( p.isSimpleCatalog() && p.error!=null && !p.error.startsWith("EMPTY") ) { drawBall(g,px1,py-9,Color.red); p.status|=Plan.STATUS_ERROR;  }
                      else if( p.isSimpleCatalog() && !hasObj ) { drawCross(g,px1,py-9); p.status|=Plan.STATUS_EMPTYCAT; }
-                     else if( p instanceof PlanMoc && ((PlanMoc)p).getMoc().getSize()==0 ) { drawCross(g,px1,py-9); p.status|=Plan.STATUS_EMPTYMOC; }
+//                     else if( p.isMoc() && ((PlanMoc)p).getMoc().isEmpty() && p.error==null ) { drawCross(g,px1,py-9); p.status|=Plan.STATUS_EMPTYMOC; }
                      else { drawBall(g,px1,py-9,Color.red); p.status|=Plan.STATUS_ERROR; }
                   } else {
                      boolean flag=false;
@@ -847,9 +854,14 @@ public final class Slide {
                         setBlink(true);
                         
                      } else {
-                        if( p instanceof PlanBG  && p.active ) {
-                           drawBall(g,px,py-9, green );
-                         }
+                        if( p instanceof PlanBG  ) {
+                           if( p.isMoc() && ((PlanMoc)p).getMoc().isEmpty() ) {   
+                              drawCross(g,px1,py-9); 
+                              p.status|=Plan.STATUS_EMPTYMOC;
+                           } else if( p.active ) {
+                              drawBall(g,px,py-9, green );
+                           }
+                        }
                      }
                      if( p.getCompletude()>=0 && p.active ) {
                         int w = Select.sizeLabel+10-60;
