@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -22,8 +22,10 @@
 package cds.aladin;
 
 import java.awt.Color;
+import java.awt.Composite;
 import java.awt.FontMetrics;
 import java.awt.Graphics;
+import java.awt.Graphics2D;
 import java.awt.Point;
 import java.awt.Polygon;
 import java.awt.Rectangle;
@@ -656,9 +658,7 @@ public class Source extends Position implements Comparator {
       PlanFilter pf;
       for( int i=0;i<PlanFilter.allFilters.length;i++) {
          pf = PlanFilter.allFilters[i];
-         if( !pf.isOn() || !pf.flagOk ) {
-            continue;
-         }
+         if( !pf.isOn() || !pf.flagOk ) continue;
          if(plan.influence[pf.numero]) nb++;
       }
 
@@ -711,8 +711,11 @@ public class Source extends Position implements Comparator {
    */
    protected boolean draw(Graphics g,ViewSimple v,int dx,int dy) {
       if( !inTime( v )  ) return false;
-
-      //System.out.println("On repaint");
+      
+      if( plan.aladin.TESTANAGLYPH ) {
+         if( drawAnaglyph(g, v) ) return true;
+      }
+      
       int L =getL();
       Point p = getViewCoord(v,L,L);
       if( p==null ) return false;
@@ -739,22 +742,20 @@ public class Source extends Position implements Comparator {
       	  drawAssociatedFootprint(g,v,dx,dy);
       	  // si aucun des plans dont la source subit l'influence n'est pret
       	  // on dessine la source comme d'habitude
-      	  if( nbFiltersOk == 0 ) {
-      	     doDraw(g, p, plan.c);
-      	  }
+      	  if( nbFiltersOk == 0 ) doDraw(g, p, plan.c);
+      	  
       	  // on applique les differentes actions associees aux differents filtres
       	  else {
       	     boolean success = drawWithFilter(g, v, p, dx, dy);
       	     if ( ! success ) return false;
       	  }
-      }
 
       // pour les sources qui ne sont sous l'influence d'aucun filtre
-      else if(noInfluence) {
+      } else if( noInfluence ) {
          doDraw(g, p, plan.c);
          drawAssociatedFootprint(g,v,dx,dy);
-      }
-      else if(!iAmSelected && nbFiltersOk==0) {
+         
+      } else if(!iAmSelected && nbFiltersOk==0) {
          doDraw(g, p, plan.c);
          drawAssociatedFootprint(g,v,dx,dy);
       }
@@ -788,6 +789,82 @@ public class Source extends Position implements Comparator {
         }
     }
 
+
+    static final int MAXR=200;
+    
+    
+    private double getAnaglyphRadius() {
+       int i = plan.getIndexAnaglyphMag();
+       if( i==-1 ) return 0;
+       String s = getValue(i);
+       if( s.length()==0 ) return 0;
+       double val = Double.parseDouble(s);
+       return 25-val;                     // Entre 22 et 0
+    }
+    
+    private double getAnaglyphDistance() {
+       int i = plan.getIndexAnaglyphParallax();
+       if( i==-1 ) return 0;
+       String s = getValue(i);
+       if( s.length()==0 ) return 0;
+       double val = Double.parseDouble(s);
+       return val+2;                    // entre 300 et 0
+    }
+    
+    private void drawStar(Graphics g, int col, int lig, int ik ) {
+       int cx=0,cy=0;
+       Graphics2D g2d = (Graphics2D)g;
+       Composite saveComposite = g2d.getComposite();
+       try {
+          double k[][] = plan.aladin.kernelList.getKernel(ik);
+          int m = k[0].length;
+          for( int lk = 0; lk < m; lk++ ) {
+             for( int ck = 0; ck < m; ck++ ) {
+                float opacityLevel = (float) (k[lk][ck]) * plan.getOpacityLevel();
+                Composite myComposite = Util.getImageComposite(opacityLevel);
+                g2d.setComposite(myComposite);
+                int x = cx + (col - m / 2 + ck);
+                int y = cy + (lig - m / 2 + lk);
+                g2d.drawLine(x,y,x,y);
+             }
+          }
+       } finally {
+          g2d.setComposite(saveComposite);
+       }
+
+    }
+
+    private boolean drawAnaglyph(Graphics g, ViewSimple v) {
+       try {
+          double sizeView = v.getTaille();   // largeur du champ en degres
+          
+          Point p = getViewCoord(v,MAXR,MAXR);
+          if( p==null || p.x<0 ) return false;
+          
+          double radius = getAnaglyphRadius();
+          if( radius==0 ) throw new Exception("null radius");
+          double distance = getAnaglyphDistance();
+                
+          int r= (int) radius/4; if( r<2 ) r=2;
+          r= (int)radius*4;
+          int k = r<10?1:r<20?2:r<30?3:r<40?4:r<50?5:6;
+          int offset = (int) ( (distance/2)*plan.getScalingFactor() );
+          
+          g.setColor( Color.cyan );
+//          g.fillOval(p.x-r-offset, p.y-r, r*2, r*2);
+          drawStar(g,p.x-offset, p.y, k); 
+          
+          g.setColor( Color.red );
+//          g.fillOval(p.x-r+offset, p.y-r, r*2, r*2);
+          drawStar(g,p.x+offset, p.y, k); 
+         
+          return true;
+      } catch( Exception e ) {
+//         e.printStackTrace();
+      }
+       return true;
+    }
+
    /** Dessine la source en inversant sa couleur (ne concerne que les surcharges dues aux filtres)
     * @param g        le contexte graphique
     * @param v        rfrence  la vue sur laquelle on doit dessiner
@@ -839,10 +916,8 @@ public class Source extends Position implements Comparator {
    /** method that actually draws the source */
    protected void doDraw(Graphics g, Point p, Color c) {
 
-
     	if( c==null ) g.setColor(plan.c);
 		else g.setColor( c );
-//      	show=false;                // Le reaffichage supprime le caractere show
       	switch(sourceType) {
            case SOLIDOVAL:    drawOval(g,p,true);    break;
            case OVAL:         drawOval(g,p);         break;
@@ -1502,5 +1577,9 @@ public class Source extends Position implements Comparator {
 
    }
 
+   /** Retourne vrai si l'objet doit tre considr comme une Source, et par consquent repris
+    * dans la tables des mesures, dans les VOTables exports, etc...
+    */
+   protected boolean asSource() { return true; }
 
 }
