@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -118,15 +118,20 @@ public class SourceTag extends Source  {
    protected void suite() {
       setLeg(legende);
       setShape(Obj.PLUS);
-//      setTag(true);
       setWithLabel(true);
       resumeMesures();
    }
-
+   
+   /** Positionne l'id par defaut */
+   void setId() {
+      id="Phot "+ nextIndice();
+   }
+   
    /** (Re)genration des mesures et raffichage */
    protected void resume() {
-      resumeMesures();
+//      planBase = plan.aladin.calque.getPlanBase();
       plan.aladin.view.newView(1);
+      resumeMesures();
       plan.aladin.mesure.redisplay();
    }
    
@@ -138,13 +143,13 @@ public class SourceTag extends Source  {
       String x = " ";
       String y = " ";
       
-      if( planBase instanceof PlanImage ) {
+      if( planBase instanceof PlanImage && !(planBase instanceof PlanBG) ) {
          planBase.projd.getXY(c);
          x = Util.myRound(""+(c.x+0.5),4);
          y = Util.myRound(""+( ((PlanImage)planBase).naxis2-c.y+0.5));
       }
       
-      info = "<&_A Tags>\t"+raj+"\t"+dej+"\t"+id+"\t"+planBase.label+"\t"+c.getRA()+"\t"+c.getDE()+"\t"+x+"\t"+y;
+      info = "<&_A|Tags>\t"+raj+"\t"+dej+"\t"+id+"\t"+planBase.label+"\t"+c.getRA()+"\t"+c.getDE()+"\t"+x+"\t"+y;
    }
    
    /** Cet objet a des proprits spcifiques */
@@ -237,18 +242,5 @@ public class SourceTag extends Source  {
       return propList;
    }
    
-//   /** Trac effectif dans la vue indique */
-//   protected boolean draw(Graphics g,ViewSimple v,int dx, int dy) {
-//      if( !isVisible() ) return false;
-//      int L = getL();
-//      Point p = getViewCoord(v,L,L);
-//      g.setColor( getColor() );
-//      super.drawPlus(g, p);
-//      if( isSelected() ) {
-//         g.setColor( isTagged() ? Color.magenta : Color.green);
-//         g.drawRect(p.x-L-1,p.y-L-1, L*2+2, L*2+2);
-//      }
-//      return true;
-//   }
 }
 
