@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -58,7 +58,7 @@ public final class Synchro {
    public Synchro() { this(0); }
    public Synchro(long defaultDelay) {
       this.defaultDelay=defaultDelay;
-      taskList = new Vector<Task>();
+      taskList = new Vector<>();
    }
    
    /** Indique que plus aucune tche n'est en attente */
