@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -31,7 +31,6 @@ import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.InputStreamReader;
 import java.net.URL;
 import java.util.HashMap;
@@ -52,6 +51,7 @@ import cds.aladin.prop.Propable;
 import cds.allsky.Constante;
 import cds.allsky.Context;
 import cds.moc.Healpix;
+import cds.moc.Moc;
 import cds.moc.SMoc;
 import cds.mocmulti.MocItem2;
 import cds.mocmulti.MultiMoc;
@@ -64,7 +64,7 @@ import cds.tools.pixtools.CDSHealpix;
  * @version 2.0 Janvier 2017 - dsormais utilis pour le HipsStore
  */
 public class TreeObjDir extends TreeObj implements Propable {
-
+   
    public String internalId;    // Alternative  l'ID de l'identificateur GLU
    private String url;          // L'url ou le path du survey
    public String description;   // Courte description (une ligne max)
@@ -165,22 +165,56 @@ public class TreeObjDir extends TreeObj implements Propable {
 
    }
    
+   public void checkHiPSProp( MyProperties prop ) throws Exception {
+      String id = MultiMoc.getID(prop);
+      if( id==null ) throw new Exception("Not an HiPS (no ID)");
+   }
+   
    /** Construction d'un TreeObjHips  partir des infos qu'il est possible de glaner
-    *  l'endroit indiqu, soit par exploration du rpertoire, soit par le fichier Properties */
-   public TreeObjDir(Aladin aladin,String pathOrUrl) throws Exception {
+    *  l'endroit indiqu, soit par exploration du rpertoire, soit par le fichier Properties
+    * @param flagCheck true s'il y a un test que le HiPS point existe bien => renverra une exception au cas o
+    */
+   public TreeObjDir(Aladin aladin,String pathOrUrl) throws Exception { this(aladin,pathOrUrl,false); }
+   public TreeObjDir(Aladin aladin,String pathOrUrl,boolean flagCheck) throws Exception {
       String s;
       this.aladin = aladin;
+      
       local=!(pathOrUrl.startsWith("http:") || pathOrUrl.startsWith("https:") ||pathOrUrl.startsWith("ftp:"));
+      progen = pathOrUrl.endsWith("HpxFinder") || pathOrUrl.endsWith("HpxFinder/");
+      if( progen ) maxOrder=15;  // Par dfaut pour un Hips progenitor
+
       MyProperties prop = new MyProperties();
 
-      // Par http ou ftp ?
+      // Rcupration des properties
+      InputStreamReader in=null;
       try {
-         InputStreamReader in=null;
-         if( !local ) in = new InputStreamReader( (new URL(pathOrUrl+"/"+Constante.FILE_PROPERTIES)).openStream(), "UTF-8" );
-         else in = new InputStreamReader( new FileInputStream(new File(pathOrUrl+Util.FS+Constante.FILE_PROPERTIES)), "UTF-8" );
-         try { prop.load(in); } finally { in.close(); }
-      } catch( Exception e ) { aladin.trace(3,"No properties file found => auto discovery..."); }
-
+         if( !local ) {
+            if( pathOrUrl.indexOf('?')>0 ) throw new Exception("Probably not an HiPS URL"); 
+             in = new InputStreamReader( Util.openStream(pathOrUrl+"/"+Constante.FILE_PROPERTIES), "UTF-8" );
+         }
+         else in = new InputStreamReader( Util.openAnyStream( Util.concatDir(pathOrUrl,Constante.FILE_PROPERTIES)), "UTF-8" );
+         prop.load(in);
+         checkHiPSProp(prop);
+      } catch( Exception e ) {
+         if( flagCheck && !progen) throw e;
+         aladin.trace(3,"No properties file found => auto discovery...");
+      } finally {
+         if( in!=null ) try { in.close(); } catch( Exception e) {}
+      }
+      
+      // Pour les Hips progenitor, on teste la prsence du metada.xml
+      if( progen && flagCheck ) {
+         InputStreamReader meta=null;
+         try {
+            if( !local ) meta = new InputStreamReader( Util.openStream(pathOrUrl+"/"+Constante.FILE_METADATAXML), "UTF-8" );
+            else meta = new InputStreamReader( Util.openAnyStream( Util.concatDir(pathOrUrl,Constante.FILE_METADATAXML)), "UTF-8" );
+         } catch( Exception e ) {
+            if( flagCheck ) throw e;
+            aladin.trace(3,"No metadata.xml file found => auto discovery...");
+         } finally {
+            if( meta!=null ) try { meta.close(); } catch( Exception e) {}
+         }
+      }
 
       // recherche du frame Healpix (ancienne & nouvelle syntaxe)
       String strFrame = prop.getProperty(Constante.KEY_HIPS_FRAME);
@@ -297,8 +331,6 @@ public class TreeObjDir extends TreeObj implements Propable {
          }
       }
 
-      progen = pathOrUrl.endsWith("HpxFinder") || pathOrUrl.endsWith("HpxFinder/");
-
       s = prop.getProperty(Constante.KEY_DATAPRODUCT_TYPE);
       if( s!=null) {
          cat = s.indexOf("catalog")>=0;
@@ -575,7 +607,7 @@ public class TreeObjDir extends TreeObj implements Propable {
 
       // dans le cas d'un rpertoire local => pas d'utilisateur du cache
       if( url!=null && !url.startsWith("http") && !url.startsWith("ftp") ) useCache=false;
-
+      
       if( color && !inJPEG && !inPNG ) inJPEG=true;
       
 //      if( !Aladin.BETA ) {
@@ -613,27 +645,27 @@ public class TreeObjDir extends TreeObj implements Propable {
 //      return u;
 //   }
 //   
-   /** Retourne l'URL d'accs au MOC, null sinon */
-   protected String getTMocUrl() {
-      String u=null;
-      
-      // On dispose du MOC directement dans le MOC server qui a fourni les proprits ?
-      if( hasTMocByMocServer() ) {
-         String params = internalId+"&get=tmoc";
-         u = aladin.glu.getURL("MocServer",params,true).toString();
-         
-      // Serait-il explicitement mentionn ?
-      } else  {
-         
-         if( prop!=null ) u = prop.getProperty("tmoc_access_url");
-
-         // Pour un HiPS on peut y accder directement
-         if( u==null && isHiPS() ) return getUrl()+"/TMoc.fits";
-      }
-      return u;
-   }
+//   /** Retourne l'URL d'accs au MOC, null sinon */
+//   protected String getTMocUrl() {
+//      String u=null;
+//      
+//      // On dispose du MOC directement dans le MOC server qui a fourni les proprits ?
+//      if( hasTMocByMocServer() ) {
+//         String params = internalId+"&get=tmoc";
+//         u = aladin.glu.getURL("MocServer",params,true).toString();
+//         
+//      // Serait-il explicitement mentionn ?
+//      } else  {
+//         
+//         if( prop!=null ) u = prop.getProperty("tmoc_access_url");
+//
+//         // Pour un HiPS on peut y accder directement
+//         if( u==null && isHiPS() ) return getUrl()+"/TMoc.fits";
+//      }
+//      return u;
+//   }
    
-   protected String getCoverage() {
+   protected String getSpaceCoverage() {
       if( prop==null ) return null;
       return Plan.getCoverageSpace( prop.getFirst("moc_sky_fraction") );
    }
@@ -743,9 +775,27 @@ public class TreeObjDir extends TreeObj implements Propable {
    /** Retourne true s'il s'agit d'un catalogue */
    protected boolean isCatalog() { return cat; }
    
+   /** retourne true s'il s'agit d'un catalogue VizieR avec des donnes images associes */
+   protected boolean hasVizieRAssocData() {
+      if( !isCDSCatalog() || prop==null ) return false;
+      String s = prop.get("associated_dataproduct_type");
+      return s!=null; // && s.indexOf("image")>=0;
+   }
+   
+   /** true s'il s'agit d'une table VizieR sans position => on se base sur la prsence d'une TMOC */
+   protected boolean isVizierTimeOnly() {
+      if( !isCDSCatalog() || prop==null ) return false;
+      if( prop==null ) return false;
+      String t = prop.get("moc_type");
+      return t!=null && t.equals("tmoc");
+   }
+   
    /** Retourne true s'il s'agit de Simbad et quu'on doit montrer un accs un mode live */
    protected boolean isSimbadLive() { return Aladin.CDS && internalId.equals("CDS/Simbad"); }
 
+   /** Retourne true s'il s'agit de Simbad */
+   protected boolean isSimbad() { return internalId.equals("CDS/Simbad"); }
+
    /** Retourne true s'il s'agit d'un catalogue hirarchique */
    protected boolean isCDSCatalog() { return cat && internalId.startsWith("CDS/"); }
    
@@ -771,6 +821,9 @@ public class TreeObjDir extends TreeObj implements Propable {
    protected boolean hasHips() { return prop!=null && 
          (prop.get("hips_service_url")!=null || prop.get("hips_service_path")!=null); }
    
+   /** Retourne true si la collection dispose d'un ou plusieurs liens vers d'autres collections */
+   protected boolean hasLinks() { return prop!=null && (prop.get("client_link")!=null); }
+   
    /** Retourne true si la collection is est un HiPS catalogue */
    protected boolean isHipsCat() {
       if( !hasHips() ) return false;
@@ -815,14 +868,14 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /** Retourne true si la collection dispose d'une URL donnant accs  un preview */
    protected boolean hasPreview() {
-      return (isCDSCatalog() || isHiPS());
+      return (isCDSCatalog() && !isVizierTimeOnly() || isHiPS());
    }
    
    /** Retourne l'URL du preview
     * ex: http://alasky.u-strasbg.fr/footprints/tables/vizier/B_denis_denis/densityMap?format=png&size=small
     */
    protected String getPreviewUrl() {
-      if( isCDSCatalog() ) {
+      if( isCDSCatalog() && !isVizierTimeOnly() ) {
          String s;
          if( internalId.startsWith("CDS/Simbad") ) s="simbad";
          else {
@@ -850,6 +903,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /** Retourne le type de donnes auquel on va accder */
    protected String getDataType() {
+      if( isVizierTimeOnly() ) return "Time serie";
       if( hasCS() || hasTAP() || isHipsCat() ) return "Catalog";
       if( hasHips() || hasSIA() ) return "Image";
       if( hasSSA() ) return "Spectrum";
@@ -880,38 +934,51 @@ public class TreeObjDir extends TreeObj implements Propable {
       return false;
    }
 
+   /** retourne de type de MOC disponible: STMOC, TMOC, SMOC, ou MOC si indiffrenci,
+    * ou "" si aucun */
+   protected String getMocType() {
+      if( !hasMoc() ) return "";
+      String s;
+      if( prop==null || (s=prop.get("moc_type"))==null ) return "MOC";
+      return s.toUpperCase();
+   }
    
    /** Retourne true si la collection dispose d'un MOC */
    protected  boolean hasMoc() {
       return hasMocByMocServer() || prop!=null && prop.getProperty("moc_access_url")!=null || isHiPS();
    }
    
-   /** Retourne true si la collection dispose d'un TMOC */
-   protected  boolean hasTMoc() {
-      return hasTMocByMocServer() || prop!=null && prop.getProperty("tmoc_access_url")!=null
-            ;
-            // || hasHiPSTMocFile();   // CA PREND TROP DE TEMPS
-   }
-   
-   /** En attendant que ce soit dans le MocServer, je regarde l'existence du fichier HpxFinder/TMoc.fits */
-   private boolean hasHiPSTMocFile() {
-      if( !isHiPS() ) return false;
-      String url = getTMocUrl();
-      return Util.isUrlResponding( url );
-   }
+//   /** Retourne true si la collection dispose d'un TMOC */
+//   protected  boolean hasTMoc() {
+//      return hasTMocByMocServer() || prop!=null && prop.getProperty("tmoc_access_url")!=null
+//            ;
+//            // || hasHiPSTMocFile();   // CA PREND TROP DE TEMPS
+//   }
+//   
+//   /** En attendant que ce soit dans le MocServer, je regarde l'existence du fichier HpxFinder/TMoc.fits */
+//   private boolean hasHiPSTMocFile() {
+//      if( !isHiPS() ) return false;
+//      String url = getTMocUrl();
+//      return Util.isUrlResponding( url );
+//   }
    
    /** Retourne true si la collection dispose d'un MOC via le MocServer
     * (=> celui-ci ayant ajout le mot cl moc_sky_fraction) */
-   private boolean hasMocByMocServer() { return prop!=null && prop.get("moc_sky_fraction")!=null; }
+   private boolean hasMocByMocServer() { return prop!=null && 
+         (prop.get("moc_type")!=null
+         || prop.get("moc_sky_fraction")!=null || prop.getProperty("moc_access_url")!=null ); }
    
-   /** Retourne true si la collection dispose d'un TMOC via le MocServer
-    * (=> celui-ci ayant ajout le mot cl tmoc_total_time) */
-   private boolean hasTMocByMocServer() { return prop!=null && prop.get("tmoc_total_time")!=null; }
+//   /** Retourne true si la collection dispose d'un TMOC via le MocServer
+//    * (=> celui-ci ayant ajout le mot cl tmoc_total_time) */
+//   private boolean hasTMocByMocServer() { return prop!=null && prop.get("tmoc_total_time")!=null; }
    
    /** Retourne l'URL d'un Cone search ou null si aucun */
    protected String getCSUrl() {
       if( prop==null ) return null;
       
+      // En attendant que Thomas B. enlve le cds_service_url de ces tables
+      if( isVizierTimeOnly() ) return null;
+      
       // J'ai un mode custom qui remplace le simple CS
       if( prop.get("cs_glutag")!=null ) return null;
       
@@ -962,7 +1029,7 @@ public class TreeObjDir extends TreeObj implements Propable {
 
    protected int getLosangeOrder() {
       if( progen || cat || nside==-1 /*|| maxOrder==-1 */) return -1;
-      return (int)Healpix.log2(nside) /*- maxOrder*/;
+      return (int)CDSHealpix.log2(nside) /*- maxOrder*/;
    }
    
    /** true si dj charg dans la pile */
@@ -1133,9 +1200,6 @@ public class TreeObjDir extends TreeObj implements Propable {
    }
    
    private void exec(String cmd) {
-//      if( cmd.indexOf("browse")>=0  ) aladin.command.execScriptAsStream( cmd );
-//      else aladin.execAsyncCommand( cmd );
-      
       aladin.console.addLot( cmd );
    }
  
@@ -1158,6 +1222,17 @@ public class TreeObjDir extends TreeObj implements Propable {
       return "get "+gluTag;
    }
    
+   /** Gnration et excution de la requte script correspondant  la requte TAp AssocData */
+   protected void loadAssoc() { loadAssoc( getDefaultTarget()+" "+getDefaultRadius(1)); }
+   protected void loadAssoc( String cone ) { 
+      if( cone==null ) { loadAssoc(); return; }
+      String ident = "\"AD "+internalId+"\"";
+     exec( ident+" = "+getAssocCmd()+" "+cone+"; select "+ident );
+   }
+   protected String getAssocCmd() {
+      String cat = Directory.getCatParent( internalId.substring(internalId.indexOf('/')+1) );
+      return "get VizieRAssocData("+cat+")";
+   }
    
    /** Gnration et excution de la requte script correspondant au protocole SIA ou SIA2 */
    protected void loadSIA() { loadSIA( getDefaultTarget()+" "+getDefaultRadius(1)); }
@@ -1248,7 +1323,8 @@ public class TreeObjDir extends TreeObj implements Propable {
          String cat = internalId.substring(i+1);
          if( internalId.startsWith("CDS/Simbad") ) cmd = "get Simbad";
          else {
-            String s = allcolumns.equals("all") ? ",allcolumns":"";
+            boolean hasTime = getMocType().indexOf('T')>=0;  // toutes les colonnes pour ne pas louper l'poque
+            String s = allcolumns.equals("all") || hasTime ? ",allcolumns":"";
             cmd = "get VizieR("+cat+s+")";
          }
 
@@ -1280,25 +1356,40 @@ public class TreeObjDir extends TreeObj implements Propable {
       return "get "+glutag+"(....) $TARGET $RADIUS";
    }
    
-   /** Gnration et excution de la requte script correspondant au protocole TMOC */
-   protected void loadTMoc( ) {exec( getTMocCmd() ); }
-   protected String getTMocBkm() { return getTMocCmd(); }
-//   private String getTMocCmd() { return "get TMOC("+Tok.quote(internalId)+")"; }
-   private String getTMocCmd() {
-      String id = Tok.quote(internalId!=null?internalId:label);
-      return "TMoc_"+id+"=load "+getTMocUrl();
-   }
+//   /** Gnration et excution de la requte script correspondant au protocole TMOC */
+//   protected void loadTMoc( ) {exec( getTMocCmd() ); }
+//   protected String getTMocBkm() { return getTMocCmd(); }
+////   private String getTMocCmd() { return "get TMOC("+Tok.quote(internalId)+")"; }
+//   private String getTMocCmd() {
+//      String id = Tok.quote(internalId!=null?internalId:label);
+//      return "TMoc_"+id+"=load "+getTMocUrl();
+//   }
 
    /** Gnration et excution de la requte script correspondant au protocole MOC */
-   protected void loadMoc( ) {exec( getMocCmd() ); }
+   protected void loadMoc( ) {
+      exec( getMocCmd() ); 
+   }
    protected String getMocBkm() { return getMocCmd(); }
-   private String getMocCmd() { return "get MOC("+Tok.quote(internalId)+")"; }
+   private String getMocCmd() {
+      String param = aladin.directory.mocServerVersion>=5 ? ",anymoc":"";
+      return "get MOC("+Tok.quote(internalId)+param+")";
+   }
 
    /** Gnration et excution de la requte script correspondant au protocole HiPS */
    protected void loadHips() {
-      String trg = getDefaultTarget();
+      String trg=null;
+      
+      // Si cette ressource n'est pas hors champ, on prend le centre de la vue courante
+      if( getIsIn()!=0 ) {
+         trg = getDefaultTarget();
+         
+      // Sinon on laisse vide
+      } else {
+         Aladin.trace(4,"TreeObjDir.loadHips(): default target out of HiPS field => no target");
+      }
+      
       String s = trg==null ? "" : " "+trg+" "+getDefaultRadius();
-     exec( getHipsCmd()+s );
+      exec( getHipsCmd()+s );
    }
    protected String getHipsBkm() { return getHipsCmd()+" $TARGET $RADIUS"; }
    protected String getHipsCmd() {
@@ -1412,11 +1503,11 @@ public class TreeObjDir extends TreeObj implements Propable {
 //      System.out.println("ULR scan = "+url);
       
       // Mmorisation du rsultat
-      SMoc moc;
+      Moc moc;
       try { 
          moc = scan(url); 
          if( mo.moc==null ) mo.moc=moc;
-         else mo.moc.add(moc);
+         else mo.moc = mo.moc.union(moc);
       } catch( Exception e ) { if( aladin.levelTrace>=3 )  e.printStackTrace();  }
       
       // Mmorisation de la surface couverte
@@ -1438,7 +1529,7 @@ public class TreeObjDir extends TreeObj implements Propable {
 //         System.out.println();
          
          if( mo.mocRef==null ) mo.mocRef=moc;
-         else mo.mocRef.add(moc);
+         else mo.mocRef = mo.mocRef.union(moc);
         
          
       } catch( Exception e ) { if( aladin.levelTrace>=3 )  e.printStackTrace(); }
@@ -1454,7 +1545,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /**  Gnration d'un Moc  partir du catalogue retourne par l'URL
     * passe en paramtre */
-   private SMoc scan( String url) {
+   private Moc scan( String url) {
       Pcat pcat = new Pcat(aladin);
       pcat.plan = new PlanCatalog(aladin);
       pcat.plan.label="test";
@@ -1463,6 +1554,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       
       try {
          moc = new SMoc(order);
+         moc.bufferOn();
          inScan=new MyInputStream( Util.openStream(url,false,true,30000) );
          pcat.tableParsing(inScan,null);
          
@@ -1479,14 +1571,14 @@ public class TreeObjDir extends TreeObj implements Propable {
                double [] c = CDSHealpix.normalizeRaDec( ((Position)o).raj, ((Position)o).dej);
                long pix = hpx.ang2pix(order, c[0], c[1] );
 //               long pix = hpx.ang2pix(order, ((Position)o).raj, ((Position)o).dej);
+               
                moc.add(order,pix);
-               n++;
-               if( n>10000 ) { moc.checkAndFix(); n=0; }
+               
             } catch( Exception e ) {
                if( aladin.levelTrace>=3 ) e.printStackTrace();
             }
          }
-         moc.setCheckConsistencyFlag(true);
+         moc.bufferOff();
           
       } catch( Exception e ) { if( aladin.levelTrace>=3 )  e.printStackTrace();
       } finally { if( inScan!=null ) try { inScan.close(); inScan=null; } catch( Exception e) {} }
