@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -21,14 +21,32 @@
 
 package cds.aladin;
 
-import java.awt.*;
-import java.awt.event.*;
-import java.io.File;
-import java.io.FileInputStream;
-import java.security.Key;
+import java.awt.BorderLayout;
+import java.awt.Color;
+import java.awt.Cursor;
+import java.awt.Dimension;
+import java.awt.FileDialog;
+import java.awt.Font;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
+import java.awt.Insets;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.KeyEvent;
+import java.awt.event.KeyListener;
+import java.awt.event.MouseAdapter;
+import java.awt.event.MouseEvent;
 import java.util.EventListener;
 
-import javax.swing.*;
+import javax.swing.BorderFactory;
+import javax.swing.JButton;
+import javax.swing.JCheckBox;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JScrollPane;
+import javax.swing.JTextArea;
+import javax.swing.JTextField;
+import javax.swing.Timer;
 import javax.swing.border.Border;
 
 import cds.tools.Util;
@@ -67,7 +85,8 @@ public class VOToolDescription extends JPanel {
       SYSTEM = "Command line";
       DIR = "Running directory";
       INSTALL = "Install. method";
-      MODEINSTALL = new String[]{ "Local","Java jar package","Web download page","Java Webstart (no installation)","Java Applet (no installation)" };
+      MODEINSTALL = new String[]{ "Local","Java jar package","Web download page","Java Webstart (no installation)",
+            "Java Applet (no installation)","Aladin Plugin" };
       createPanel();
       timer=new Timer(500,new ActionListener() {
          public void actionPerformed(ActionEvent e) { downloading(); }
@@ -132,8 +151,7 @@ public class VOToolDescription extends JPanel {
       m.addMouseListener(new MouseAdapter() {
          public void mouseReleased(MouseEvent e) {
             if( doc.getToolTipText() == null ) return;
-            aladin.glu.showDocument("Http", ((JLabel) e.getSource())
-                  .getToolTipText(), true);
+            aladin.glu.showDocument("Http", ((MyLabel) e.getSource()).getToolTipText(), true);
          }
 
          public void mouseEntered(MouseEvent e) {
@@ -244,8 +262,6 @@ public class VOToolDescription extends JPanel {
       this.dir.setText(t);
       aladin.frameVOTool.apply.setEnabled(hasBeenChanged());
    }
-
-
    
    /** Retourne true si le formulaire a chang par rapport  son contenu initial */
    protected boolean hasBeenChanged() {
@@ -349,11 +365,16 @@ public class VOToolDescription extends JPanel {
          system.setMemo(" -- ");
          system.setEditable(false); 
          browse.setEnabled(false);
+         
+      } else if( mode==GluApp.PLUGIN ) {
+         dir.setMemo(vo.pluginUrl!=null ? aladin.getPluginDir() : "", vo==null ? null : vo.dir);
+         system.setMemo(" -- ");
+         system.setEditable(false); 
+         browse.setEnabled(true);
+         
       } else {
-         dir.setMemo(vo.jarUrl!=null ? aladin.getVOPath() : "",
-               vo==null ? null : vo.dir);
-         system.setMemo(vo.javaParam!=null ? vo.getJavaCommand() : "",
-               vo == null ? "" : vo.getCommand());
+         dir.setMemo(vo.jarUrl!=null ? aladin.getVOPath() : "", vo==null ? null : vo.dir);
+         system.setMemo(vo.javaParam!=null ? vo.getJavaCommand() : "", vo == null ? "" : vo.getCommand());
          browse.setEnabled(true);
       }
 
