@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -72,6 +72,11 @@ public abstract class Builder {
          case CLEANPNG:  return new BuilderCleanPng(context);
          case CLEANDATE: return new BuilderCleanDate(context);
          case CLEANWEIGHT:return new BuilderCleanWeight(context);
+         case UPDATE:    return new BuilderUpdate(context);
+         case CHECKCODE: return new BuilderCheckCode(context);
+         case UPDATEDATASUM: return new BuilderUpdateDataSum(context);
+         case CHECK:     return new BuilderCheck(context);
+         case CHECKDATASUM: return new BuilderCheckFits(context);
          case LINT:      return new BuilderLint(context);
 //         case GZIP:      return new BuilderGzip(context);
          case GUNZIP:    return new BuilderGunzip(context);
@@ -82,10 +87,10 @@ public abstract class Builder {
          case CUBE:      return new BuilderCube(context);
          case DETAILS:   return new BuilderDetails(context);
          case MAPTILES:  return new BuilderMapTiles(context);
-         case UPDATE:    return new BuilderUpdate(context);
          case PROP:      return new BuilderProp(context);
          case MIRROR:    return new BuilderMirror(context);
          case MAP:       return new BuilderMap(context);
+         case SMOC:      return new BuilderSMoc(context);
          case TMOC:      return new BuilderTMoc(context);
          case STMOC:     return new BuilderSTMoc(context);
 //         case ZIP:       return new BuilderZip(context);
@@ -115,7 +120,7 @@ public abstract class Builder {
    /** Indique le mode Just-print - not run -> retourne true si c'est le cas avec un message d'info */
    public boolean isFake() {
       if( !context.fake ) return false;
-      context.info("Action "+getAction()+" not run due to the -n option");
+      context.warning("Action "+getAction()+" not run due to the -n option");
       return true;
    }
 
@@ -152,7 +157,7 @@ public abstract class Builder {
       String name = null;
       String path = null;
       int i = output==null ? -1 : output.lastIndexOf('/');
-      if( i==-1 && output!=null ) i=output.lastIndexOf('\\');
+      if( i==-1 && output!=null ) i=output.lastIndexOf('\\');      
 
       // Path indiqu spcifiquement ?
       if( i>=0 ) path = output.substring(0,i);
@@ -203,7 +208,7 @@ public abstract class Builder {
 
       File f = new File(output);
       if( f.exists() && (!f.isDirectory()  || !f.canRead())) throw new Exception("Ouput directory not available ["+output+"]");
-      context.info("the output directory will be "+output);
+      context.info("the output directory is "+output);
       
       
       context.setValidateOutput(true);
@@ -548,6 +553,10 @@ public abstract class Builder {
    
    // Retourne la taille moyenne d'une tuile (tous les formats additionns) - en bytes
    private long getTileSize( int bitpix, int width, int depth, String fmt ) {
+      
+      // Cas d'un catalogue HiPS (on estime  20Ko chaque tuile)
+      if( fmt.toLowerCase().indexOf("tsv")>=0 ) return 20*1024;  
+      
       long nbpix = width*width;
       
       // Cas couleur
@@ -573,7 +582,7 @@ public abstract class Builder {
       
       // Dtermination de la taille totale requise (en Ko)
       moc.setMocOrder(order);
-      long numberOfTiles = moc.getNbCells();
+      long numberOfTiles = moc.getNbValues();
       long tileSize = getTileSize( bitpix, tileWidth, depth, fmt);
       if( tileSize==0 ) throw new Exception("No remote tile found");
       long fullSize = (long)( ( 1.3 * tileSize * numberOfTiles + 8 ) / 1024. );
@@ -597,13 +606,14 @@ public abstract class Builder {
          part[n].max = i<0 ? -1 : getMem( s1.substring(i+1) );
          part[n].dir = i<0 ? s1 : s1.substring(0,i);
          
-         if( !( new File(part[n].dir).isAbsolute() ) ) throw new Exception("alternative target partitions must use an absolute path");
+         char a = part[n].dir.charAt(0);
+         if( a!='\\' && a!='/' ) throw new Exception("alternative target partitions must use an absolute path");
 //         System.out.println(part[n].dir+" => "+Util.getUnitDisk( (long)( part[n].max)*1024L ));
       }
       
       // Dtermination de la taille de chaque Dirnn de niveau le plus profond (en Ko)
       HashMap<Long, Double> hashDir = new HashMap<>();
-      Iterator<Long> it = moc.pixelIterator();
+      Iterator<Long> it = moc.valIterator();
       while( it.hasNext() ) {
          long ndirLink = it.next()/10000;
          Double mem = hashDir.get(ndirLink);
