@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -61,6 +61,8 @@ public class BuilderConcat extends BuilderTiles {
 
    public void run() throws Exception {
       build();
+      
+      context.resetCheckCode();
 
       // Regeneration de l'arborescence pour la zone concerne
       (new BuilderTree(context)).run();
@@ -75,21 +77,24 @@ public class BuilderConcat extends BuilderTiles {
 
          // Regeneration des tuiles png et de l'arborescence pour la zone concerne si ncessaire
          inPng = (new File(context.getOutputPath()+Util.FS+"Norder3"+Util.FS+"Allsky.png")).exists();
-         if( inPng ) { (new BuilderPng(context)).run(); context.info("PNG tiles updated"); }
+         if( inPng ) {
+            context.info("Updating PNG tiles...");
+            context.setMode(Mode.REPLACETILE);
+            (new BuilderPng(context)).run(); 
+         }
       }
 
-      // INUTILE CAR DEJA FAIT
-//      // Regeneration des allsky si non encore fait
-//      if( !inJpg && !inPng ) (new BuilderAllsky(context)).run();
-//      context.info("Allsky updated");
-
+      // Dans le cas d'un traitement algorythmique, les mises  jour des mtadonnes
+      // sont spcifiques (voir ci-dessous)
+      boolean coaddOp = mode==Mode.SUM || mode==mode.DIV || mode==Mode.MUL;
+      
       // Mise  jour ou generation du MOC final
       String outputPath = context.getOutputPath();
       outputMoc = new SMoc();
-      File f = new File(outputPath+Util.FS+Constante.FILE_MOC);
+      File f = coaddOp ? null :new File(outputPath+Util.FS+Constante.FILE_MOC);
       if( f.exists() ) {
          outputMoc.read( f.getCanonicalPath() );
-         outputMoc = (SMoc)outputMoc.union(inputMoc);
+         outputMoc = outputMoc.union(inputMoc);
          outputMoc.write( context.getOutputPath()+Util.FS+Constante.FILE_MOC);
          context.info("MOC updated");
       } else {
@@ -98,24 +103,26 @@ public class BuilderConcat extends BuilderTiles {
       }
 
       // Post traitement sur le HpxFinder si ncessaire
-      if( !doHpxFinder ) {
-         f = new File(outputPathIndex);
-         if( f.isDirectory() ) {
-            f.renameTo( new File(outputPathIndex+"-partial"));
-            context.warning("Previous HpxFinder has been removed as "+Constante.FILE_HPXFINDER+"-partial");
-         }
-      } else {
-         // Il faut refaire le MOC de l'index cumul
-         (new BuilderMocIndex(context)).run();
-         context.info("Index MOC updated");
+      if( !coaddOp ) {
+         if( !doHpxFinder ) {
+            f = new File(outputPathIndex);
+            if( f.isDirectory() ) {
+               f.renameTo( new File(outputPathIndex+"-partial"));
+               context.warning("Previous HpxFinder has been removed as "+Constante.FILE_HPXFINDER+"-partial");
+            }
+         } else {
+            // Il faut refaire le MOC de l'index cumul
+            (new BuilderMocIndex(context)).run();
+            context.info("Index MOC updated");
 
-         // Faut-il lancer galement une commande PROGEN
-         f = new File(outputPathIndex+Util.FS+"metadata.xml");
-         if( f.exists() ) {
-            BuilderDetails  b = new BuilderDetails(context);
-            b.validateContext();
-            b.run();
-            context.info("PROGEN tiles updated");
+            // Faut-il lancer galement une commande PROGEN
+            f = new File(outputPathIndex+Util.FS+"metadata.xml");
+            if( f.exists() ) {
+               BuilderDetails  b = new BuilderDetails(context);
+               b.validateContext();
+               b.run();
+               context.info("PROGEN tiles updated");
+            }
          }
       }
    }
@@ -134,6 +141,9 @@ public class BuilderConcat extends BuilderTiles {
       File f = new File(inputPath);
       if( f.exists() && (!f.isDirectory() || !f.canRead() )) throw new Exception("\"inputPath\" directory not available ["+inputPath+"]");
 
+      // Mise  jour des proprits pour le suivi des addendum
+      addAddendum( context.getInputPath(), context.getOutputPath() );
+
       int order = Util.getMaxOrderByPath( outputPath );
       if( order==-1 )  throw new Exception("No HiPS found in ouput dir");
       context.setOrder(order);
@@ -142,11 +152,6 @@ public class BuilderConcat extends BuilderTiles {
       if( order!=inputOrder ) throw new Exception("Uncompatible HiPS: out.order="+order+" input.order="+inputOrder);
       context.info("Order retrieved from ["+inputPath+"] => "+order);
       
-      // Mise  jour des proprits pour le suivi des addendum
-     try {
-        addAddendum( context.getInputPath(), context.getOutputPath() );
-     } catch( Exception e ) { e.printStackTrace(); }
-
       String allsky = context.getOutputPath()+Util.FS+"Norder3"+Util.FS+"Allsky.fits";
       if( (new File(allsky)).exists() ) {
          setContextParamFromPreviousAllskyFile(allsky);
@@ -167,10 +172,6 @@ public class BuilderConcat extends BuilderTiles {
          }
       }
 
-      // faudra-t-il traiter les index
-      doHpxFinder = (new File(inputPathIndex)).isDirectory() && (new File(outputPathIndex)).isDirectory();
-      if( doHpxFinder ) context.info("HpxFinder will be also concatenated (mode="+mode+")");
-
       inputMoc = new SMoc();
       f = new File(inputPath+Util.FS+Constante.FILE_MOC);
       if( f.exists() ) inputMoc.read( f.getCanonicalPath() );
@@ -183,7 +184,7 @@ public class BuilderConcat extends BuilderTiles {
          inputMoc.read( f.getCanonicalPath() );
       }
 
-      if( context.mocArea!=null ) inputMoc = (SMoc)inputMoc.intersection(context.mocArea);
+      if( context.mocArea!=null ) inputMoc = inputMoc.intersection(context.mocArea);
       context.moc = inputMoc;
 
       // Dans le cas de rngration des allsky.png il faut connatre les cuts
@@ -196,13 +197,25 @@ public class BuilderConcat extends BuilderTiles {
       liveOut = checkLiveByProperties(context.getOutputPath());
       liveIn = checkLiveByProperties(context.getInputPath());
       live = liveIn && liveOut;
+      
+      // faudra-t-il traiter les index
+      doHpxFinder = (new File(inputPathIndex)).isDirectory() && (new File(outputPathIndex)).isDirectory();
+      if( mode==Mode.DIV || mode==Mode.MUL || mode==Mode.SUM ) {
+         doHpxFinder=false;
+         if( liveIn ) context.warning("Source HiPS does provide weight tiles => ignored");
+      }
+      if( doHpxFinder ) context.info("HpxFinder will be also concatenated (mode="+mode+")");
+
+
+      
       if( mode==Mode.AVERAGE ) {
          if( !live ) context.warning("Both HiPS to merge do not provide weight tiles => assuming basic average");
          else if( !liveOut ) context.warning("Target HiPS do not provide weight tiles => assuming weigth 1 for each output pixel");
          else if( !liveIn ) context.warning("Source HiPS do not provide weight tiles => assuming weigth 1 for each input pixel");
-         
       }
       
+      context.info("Coadd mode: "+Mode.getExplanation(mode));
+      
    }
    
    /** Vrifie que les 2 hips disposent des cartes de poids */
@@ -224,8 +237,7 @@ public class BuilderConcat extends BuilderTiles {
 
       String sourceHipsId = getHipsIdFromProperty(sourcePath);
       context.addAddendum(sourceHipsId);
-      context.info("Merging "+sourceHipsId+" into "+targetHipsId+"...");
-
+      context.info("Concat "+sourceHipsId+" into "+targetHipsId+"...");
    }
    
    
@@ -363,15 +375,24 @@ public class BuilderConcat extends BuilderTiles {
             break;
          case AVERAGE:
             if( out!=null ) {
-               if( !live ) input.coadd(out,true);
+               if( !live ) input.coadd(out,Fits.AVG);
                else input.coadd(out, weightIn, weightOut);
             }
             out=input;
             weightOut=weightIn;
             break;
+         case DIV:
+            if( out!=null ) out.coadd(input,Fits.DIV);
+            break;
+         case MUL:
+            if( out!=null ) out.coadd(input,Fits.MUL);
+            break;
+         case SUM:
+            if( out!=null ) out.coadd(input,Fits.ADD);
+            break;
          case ADD:
             if( out!=null ) {
-               input.coadd(out,false);
+               input.coadd(out,Fits.ADD);
                if( live ) for( int i=0; i<weightOut.length; i++ ) weightIn[i] += weightOut[i];
             }
             out=input;
@@ -394,6 +415,9 @@ public class BuilderConcat extends BuilderTiles {
             out=input;
             weightOut=weightIn;
             break;
+         default:
+            throw new Exception("Coadd mode ["+mode+"] not supported for CONCAT action");
+
       }
 
       if( out==null ) throw new Exception("Y a un blme ! out==null");
