@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -53,6 +53,8 @@ public class BuilderCube extends Builder {
 
    public void run() throws Exception {
       build();
+      
+      context.resetCheckCode();
 
       if( !context.isTaskAborting() ) {
          BuilderMoc bm = new BuilderMoc(context);
@@ -124,7 +126,7 @@ public class BuilderCube extends Builder {
             SMoc m = new SMoc();
             m.read( path+Util.FS+Constante.FILE_MOC);
             if( context.moc==null ) context.moc=m;
-            else context.moc = (SMoc)context.moc.union(m);
+            else context.moc = context.moc.union(m);
          } catch( Exception e ) {
             context.warning("Missing original MOC in "+path+" => running time estimation will be wrong");
          }
@@ -139,7 +141,12 @@ public class BuilderCube extends Builder {
       context.depth=inputPath.length;
 
       // Mode de travail (link ou copy)
-      if( context.getMode().equals(Mode.LINK) ) mode = context.getMode();
+      Mode m = context.getMode();
+      if( m==Mode.getDefault() ) mode=Mode.COPY;   // Le dfaut de TILES est remplac par le dfaut de CUBE
+      else mode=m;
+      if( mode!=Mode.COPY && mode!=Mode.LINK ) {
+         throw new Exception("Coadd mode ["+mode+"] not supported for CUBE action");
+      }
       context.info(mode.getExplanation(mode));
    }
 
