@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -80,6 +80,8 @@ public class BuilderGzip extends Builder {
       int maxOrder = Util.getMaxOrderByPath(path);
       int order;
       
+      context.resetCheckCode("fits");
+      
       // Parcours de tous les rpertoire Norder?? trouvs
       for( File nOrder : (new File(path)).listFiles() ) {
          if( context.isTaskAborting() ) throw new Exception("Task abort !");
