@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -75,23 +75,13 @@ public class BuilderMapTiles extends Builder {
       build();
 
       if( !context.isTaskAborting() ) {
+         context.resetCheckCode();
 
-//         if( context.getOrder() <= 3 ) {
-//            (new BuilderAllsky(context)).run();
-//            context.info("ALLSKY file done");
-//         } else {
-//            (new BuilderTree(context)).run();
-//            context.info("HiPS TREE done");
-//         }
-         
          (new BuilderTree(context)).run();
          context.info("HiPS TREE done");
 
-
          context.setProgressLastNorder3(1);
-         
          (b = new BuilderMoc(context)).run();
-
       }
    }
 
