@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -25,13 +25,10 @@ import static cds.tools.Util.FS;
 
 import java.io.File;
 import java.io.FileInputStream;
-import java.util.Iterator;
 
-import cds.aladin.Coord;
-import cds.aladin.Localisation;
 import cds.aladin.MyInputStream;
 import cds.fits.Fits;
-import cds.moc.Healpix;
+import cds.moc.Moc;
 import cds.moc.SMoc;
 import cds.tools.pixtools.CDSHealpix;
 import cds.tools.pixtools.Util;
@@ -43,6 +40,7 @@ public class BuilderMoc extends Builder {
 
    protected SMoc moc;
    protected int mocOrder;
+   protected long maxSize=-1;
    protected int fileOrder;
    protected int tileOrder;
    protected boolean isMocHight;
@@ -83,7 +81,7 @@ public class BuilderMoc extends Builder {
 //      } catch( Exception e ) { }
       
       
-      // mocOrder explicitement fourni par l'utilisateur
+      // mocOrder explicitement fourni par l'utilisateur ?
       if( context.getMocOrder()!=-1 ) mocOrder = context.getMocOrder();
       
       // Sinon on prend classiquement le niveau des tuiles
@@ -91,6 +89,10 @@ public class BuilderMoc extends Builder {
          mocOrder = fileOrder;
          if( mocOrder< Constante.DEFAULTMOCORDER ) mocOrder = Constante.DEFAULTMOCORDER;
       }
+      
+      // Taille limite explicitement fournie par l'utilisateur ?
+      maxSize = context.getMocMaxSize();
+      
     
 // POSAIT TROP DE SOUCI DE MOC TROP LONG A CALCULER => LA DERTERMINATION AUTOMATIQUE EST SUPPRIME (PF 14/1/2019)
 //      // mocOrder dtermin par la nature du survey
@@ -126,13 +128,14 @@ public class BuilderMoc extends Builder {
       String outputFile = path + FS + Constante.FILE_MOC;
       
       long t = System.currentTimeMillis();
-      context.info("MOC generation ("+(isMocHight?"deep resolution":"regular resolution")+" mocOrder="+moc.getMocOrder()+")...");
+      context.info("MOC generation ("+(isMocHight?"deep resolution":"regular resolution")+" mocOrder="+moc.getMocOrder()
+            +(maxSize!=-1?" <"+cds.tools.Util.getUnitDisk(maxSize):"")
+            +")...");
       
       String  frame = getFrame();
-      moc.setSys(frame);
-      moc.setCheckConsistencyFlag(false);
+      moc.setSpaceSys(frame);
       generateMoc(moc,fileOrder, path);
-      moc.setCheckConsistencyFlag(true);
+      adjustSize(moc,true);
       moc.write(outputFile);
       
       // Faut-il changer le rfrentiel du MOC ?
@@ -144,45 +147,11 @@ public class BuilderMoc extends Builder {
 //      }
       
       long time = System.currentTimeMillis() - t;
-      context.info("MOC done in "+cds.tools.Util.getTemps(time,true)
+      context.info("MOC done in "+cds.tools.Util.getTemps(time*1000L)
                         +": mocOrder="+moc.getMocOrder()
-                        +" size="+cds.tools.Util.getUnitDisk( moc.getSize()));
+                        +" size="+cds.tools.Util.getUnitDisk( moc.getMem()));
    }
    
-   /** Changement de rfrentiel si ncessaire */
-   // IL FAUDRA REMETTRE TOUT CA DANS SpaceMOC
-   static public SMoc convertTo(SMoc moc, String coordSys) throws Exception {
-      if( coordSys.equals( moc.getSys()) ) return moc;
-
-      char a = moc.getSys().charAt(0);
-      char b = coordSys.charAt(0);
-      int frameSrc = a=='G' ? Localisation.GAL : a=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
-      int frameDst = b=='G' ? Localisation.GAL : b=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
-
-      Healpix hpx = new Healpix();
-      int order = moc.getMaxUsedOrder();
-      SMoc moc1 = new SMoc(coordSys,moc.getMinOrder(),moc.getMocOrder());
-      moc1.setCheckConsistencyFlag(false);
-      long onpix1=-1;
-      Iterator<Long> it = moc.pixelIterator();
-      while( it.hasNext() ) {
-         long npix = it.next();
-         for( int i=0; i<4; i++ ) {
-            double [] coo = hpx.pix2ang(order+1, npix*4+i);
-            Coord c = new Coord(coo[0],coo[1]);
-            c = Localisation.frameToFrame(c, frameSrc, frameDst);
-            long npix1 = hpx.ang2pix(order+1, c.al, c.del);
-            if( npix1==onpix1 ) continue;
-            onpix1=npix1;
-            moc1.add(order,npix1/4);
-         }
-
-      }
-      moc1.setCheckConsistencyFlag(true);
-      return moc1;
-   }
-
-   
    private String getDefaultExt(String path) {
       if( (new File(path+FS+"Norder3"+FS+"Allsky.fits")).exists() ) return "fits";
       if( (new File(path+FS+"Norder3"+FS+"Allsky.jpg")).exists() ) return "jpg";
@@ -209,18 +178,37 @@ public class BuilderMoc extends Builder {
       long cTime = now-startTime;
       if( cTime<2000 ) return;
       
-      context.stat(nbTiles+" tile"+(nbTiles>1?"s":"")+" scanned in "+cds.tools.Util.getTemps(cTime) );
+      context.stat(nbTiles+" tile"+(nbTiles>1?"s":"")+" scanned in "+cds.tools.Util.getTemps(cTime*1000L) );
    }
    
    /** Retourne la surface du Moc (en nombre de cellules de plus bas niveau */
-   public long getUsedArea() { return moc.getNbCells(); }
+   public long getUsedArea() { return moc.getNbValues(); }
 
    /** Retourne le nombre de cellule de plus bas niveau pour la sphre complte */
-   public long getArea() { return moc.getNbCellsFull(); }
+//   public long getArea() { return moc.getNbCellsFull(); }
+   public long getArea() { return moc.maxVal(); }
+   
+   // retourne true s'il est temps de tester un ajustement de taille
+   protected boolean mustAdjustSize(Moc m, boolean force) {
+      if( force ) m.flush();
+      return m.bufferSize()==0;
+   }
+   
+   protected void reduction(Moc m) throws Exception { m.reduction(maxSize); }
+   
+   protected void adjustSize(Moc m, boolean force) throws Exception {
+      if( maxSize==-1 ) return;
+      if( !mustAdjustSize(m,force) ) return;
+      if( m.getMem() <= maxSize ) return;
+      
+      reduction(m);
+      
+   }
 
    protected void generateMoc(SMoc moc, int fileOrder,String path) throws Exception {
       
       initStat();
+      moc.bufferOn();
       
       File f = new File(path + Util.FS + "Norder" + fileOrder );
       
@@ -249,14 +237,15 @@ public class BuilderMoc extends Builder {
 
             generateTileMoc(moc,sf1[j], fileOrder, npix);
          }
-         moc.checkAndFix();
       }
+      moc.bufferOff();
    }
    
    protected void generateTileMoc(SMoc moc,File f,int fileOrder, long npix) throws Exception {
       updateStat();
       if( isMocHight ) generateHighTileMoc(moc,fileOrder,f,npix);
       else moc.add(fileOrder,npix);
+      adjustSize(moc,false);
    }
    
    private void generateHighTileMoc(SMoc moc,int fileOrder, File f, long npix) throws Exception {
@@ -305,7 +294,6 @@ public class BuilderMoc extends Builder {
             }
          }
       }
-      moc.checkAndFix();
    }
 
    
