@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -80,6 +80,7 @@ public class BuilderRgb extends BuilderTiles {
    public void run() throws Exception {
       Fits.setToolKit();
       build();
+      context.resetCheckCode();
       context.setPropriete(Constante.KEY_HIPS_PROCESS_HIERARCHY, context.getJpegMethod().toString().toLowerCase());
 
       if( !context.isTaskAborting() ) (new BuilderMoc(context)).createMoc(output);
@@ -215,7 +216,7 @@ public class BuilderRgb extends BuilderTiles {
             // Ajustement de la rgion qu'il faudra calculer
             SMoc m = moc[c] = loadMoc( inputs[c] );
             if( context.moc==null ) context.moc = m;
-            else context.moc = (SMoc)context.moc.union(m);
+            else context.moc = context.moc.union(m);
 
             // Vrification de la cohrence des systmes de coordonnes
             String f = getFrameFromProp( prop[c] );
@@ -272,7 +273,7 @@ public class BuilderRgb extends BuilderTiles {
       }
 
       // dtermination de la zone  calculer
-      if( context.mocArea!=null ) context.moc = (SMoc)context.moc.intersection( context.mocArea );
+      if( context.mocArea!=null ) context.moc = context.moc.intersection( context.mocArea );
       
       // Faut-il un filtre gaussien
       if( context.gaussFilter ) context.info("Gauss filter activated...");
