@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -87,6 +87,8 @@ public class BuilderTiles extends Builder {
    public void run() throws Exception {
       context.info("Creating "+context.getTileExt()+" tiles and allsky (max depth="+context.getOrder()+")...");
       context.info("sky area to process: "+context.getNbLowCells()+" low level HEALPix cells");
+      
+      context.resetCheckCode( context.getTileExt());
 
       // Un peu de baratin
       if( !context.isColor() ) {
@@ -105,8 +107,9 @@ public class BuilderTiles extends Builder {
          if( bs!=1 || bz!=0 ) { context.info("BSCALE="+bs+" BZERO="+bz); }
          double bl0 = context.getBlankOrig();
          double bl1 = context.getBlank();
+         String bkey = context.getBlankKey();
          if( context.hasAlternateBlank() ) context.info("BLANK conversion from "+(Double.isNaN(bl0)?"NaN":bl0)+" to "+(Double.isNaN(bl1)?"NaN":bl1));
-         else context.info("BLANK="+ (Double.isNaN(bl1)?"NaN":bl1));
+         else context.info("BLANK="+ (bkey!=null? bkey : Double.isNaN(bl1)?"NaN":bl1));
          if( context.good!=null ) context.info("Good pixel values ["+ip(context.good[0],bz,bs)+" .. "+ip(context.good[1],bz,bs)+"] => other values are ignored");
          context.info("Tile aggregation method="+Context.JpegMethod.MEAN);
          if( context.live ) context.info("Live HiPS => Weight tiles saved for potential future additions"); 
@@ -132,8 +135,11 @@ public class BuilderTiles extends Builder {
          }
       }
       context.setPropriete(Constante.KEY_HIPS_PROCESS_OVERLAY,
-            context.isMap() ? "none" : context.mode==Mode.ADD ? "add" :
-               context.fading ? "border_fading" : context.mixing ? "mean" : "first");
+               context.isMap() ? "none" : 
+               context.mode==Mode.ADD ? "add" :
+               context.mode==Mode.SUM ? "cumul" :
+               context.fading ? "border_fading" : 
+               context.mixing ? "mean" : "first");
       context.setPropriete(Constante.KEY_HIPS_PROCESS_HIERARCHY, context.getJpegMethod().toString().toLowerCase());
       
       if( !context.isTaskAborting() ) { (new BuilderAllsky(context)).run(); context.done("ALLSKY file done"); }
@@ -163,7 +169,7 @@ public class BuilderTiles extends Builder {
          // retry order validation with tiles directory
          validateOrder(context.getOutputPath());
       }
-
+      
       //      if(  !context.isColor() ) {
 
       String img = context.getImgEtalon();
@@ -231,8 +237,11 @@ public class BuilderTiles extends Builder {
 
 
       // Info sur la mthode
-      Mode m = context.getMode();
-      if( !context.isColor() || m==Mode.KEEPTILE || m==Mode.REPLACETILE ) context.info("mode="+Mode.getExplanation(m));
+      Mode mode = context.getMode();
+      if( mode==Mode.MUL || mode==Mode.DIV || mode==Mode.COPY || mode==Mode.LINK ) {
+         throw new Exception("Coadd mode ["+mode+"] not supported for TILES action");
+      }
+      context.info("Coadd mode: "+Mode.getExplanation(mode));
       
       // Info sur le coordinate frame
       context.info("HiPS coordinate frame => "+context.getFrameName());
@@ -255,7 +264,7 @@ public class BuilderTiles extends Builder {
       order        = context.getOrder();
       depth        = context.getDepth();
       format       = context.isColor() ? "jpeg" : "fits png";  // on suppose qu'on va crer des tuiles fits et preview (png)
-      moc          = (SMoc)( context.mocIndex.clone() );
+      moc          = ( context.mocIndex.clone() );
       if( moc==null ) throw new Exception("No MOC available => splitting action not possible");
       if( !outputIsFree( outputPath, order ) ) {
          context.warning("HiPS output dir not empty => split function ignored");
@@ -514,10 +523,11 @@ public class BuilderTiles extends Builder {
 
       SMoc moc = new SMoc();
       SMoc m = context.getRegion();
-      if( m==null ) m = new SMoc("0/1-11");
+      if( m==null ) m = new SMoc("0/0-11");
       moc.add( m );
-      int minorder=0; //3;
-      context.setMinOrder( minorder );
+//      int minorder=0; //3;
+//      context.setMinOrder( minorder );
+      int minorder = context.getMinOrder();
       
       moc.setMocOrder(minorder);
       int depth = context.getDepth();
@@ -532,7 +542,7 @@ public class BuilderTiles extends Builder {
 //      }
       
       for( int z=0; z<depth; z++ ) {
-         Iterator<Long> it = moc.pixelIterator();
+         Iterator<Long> it = moc.valIterator();
          while( it.hasNext() ) {
             long npix=it.next();
             fifo.add( new Item(minorder, npix,z,null,true ) );
@@ -594,7 +604,7 @@ public class BuilderTiles extends Builder {
                   ThreadBuilderTile.statOnePass+" in one step, "+
                   ThreadBuilderTile.statMultiPass+" in multi steps");
          if( context.cacheFits!=null ) Aladin.trace(3,"Cache FITS status: "+ context.cacheFits);
-         Aladin.trace(3,"Healpix survey build in "+cds.tools.Util.getTemps(System.currentTimeMillis()-t));
+         Aladin.trace(3,"Healpix survey build in "+cds.tools.Util.getTemps((System.currentTimeMillis()-t)*1000L));
       }
 
       if( context.cacheFits!=null ) context.cacheFits.reset();
@@ -644,7 +654,13 @@ public class BuilderTiles extends Builder {
       context.setCache(new CacheFits(sizeCache));
       context.info("Available RAM: "+cds.tools.Util.getUnitDisk(size)+" => Cache size: "+cds.tools.Util.getUnitDisk(sizeCache));
    }
-
+   
+   /** Par dfaut, il s'agit d'un simple accs  la tuile dj calcule,
+    * sauf pour BuilderMirror */
+   protected Fits findLeaf(ThreadBuilderTile hpx, String file, String path,int order,long npix, int z) throws Exception { 
+      return findLeaf(file);
+   }
+   
    /** Cration d'un losange et de toute sa descendance si ncessaire.
     * Mthode rcursive qui
     * 1) Vrifie si le travail n'a pas dj t fait en se basant sur
@@ -665,18 +681,23 @@ public class BuilderTiles extends Builder {
 
       // si le process a t arrt on essaie de ressortir au plus vite
 //      if (stopped) return null;
+      
 
       // si on n'est pas dans le Moc, il faut retourner le fichier
       // pour la construction de l'arborescence...
       if( !context.isInMocTree(order,npix) ) return findLeaf(file);
       
-      // si le losange a dj t calcul on le renvoie
+      // si le losange a dj t calcul on le renvoie directement
+      // ou que l'on n'a pas besoin de descendre plus loin dans c"ette branche
       if( coaddMode==Mode.KEEPTILE ) {
-         Fits oldOut = findLeaf(file);
+         Fits oldOut = findLeaf(hpx,file,path,order,npix, z);
          if( oldOut!=null ) {
             SMoc moc = context.getRegion();
-            moc = (SMoc)moc.intersection(new SMoc(order+"/"+npix));
-            int nbTiles = (int)moc.getNbCells();
+            SMoc a = new SMoc(order+"/"+npix);
+            a.setSys( context.getFrameCode() );
+            moc = moc.intersection(a);
+            moc.setMocOrder(ordermax);
+            int nbTiles = (int)moc.getNbValues();
             updateStat(0,0,nbTiles,0,nbTiles/4,0);
             return oldOut;
          }
@@ -1196,19 +1217,6 @@ public class BuilderTiles extends Builder {
                   }
                }
 
-               //               int pix=0;
-               //               if( in!=null ) {
-               //                  for( int i=0;i<4; i++ ) {
-               //                     int gx = i==1 || i==3 ? 1 : 0;
-               //                     int gy = i>1 ? 1 : 0;
-               //                     int p = in.getPixelRGBJPG(x+gx,y+gy);
-               //                     pix=p;
-               //                     break;
-               //                     //	                        pix+=p/4;
-               //                  }
-               //               }
-
-
                out.setPixelRGBJPG(offX+(x>>>1), offY+(y>>>1), pix);
 
                // Normal
@@ -1269,7 +1277,10 @@ public class BuilderTiles extends Builder {
    protected void write(String file, Fits out) throws Exception {
       String filename = file+context.getTileExt();
       if( isColor ) out.writeCompressed(filename,0,0,null, Constante.TILE_MODE[ context.targetColorMode ]);
-      else out.writeFITS(filename);
+      else {
+         out.addDataSum();
+         out.writeFITS(filename);
+      }
    }
 
    /** Cration d'un losange terminal
@@ -1310,8 +1321,9 @@ public class BuilderTiles extends Builder {
                // celui renseign par l'utilisateur, et sinon celui par dfaut
                if( oldOut.bitpix>0 && Double.isNaN(oldOut.blank)) oldOut.setBlank(blank);
 
-               if( coaddMode==Mode.AVERAGE ) out.coadd(oldOut,true);
-               else if( coaddMode==Mode.ADD ) out.coadd(oldOut,false);
+               if( coaddMode==Mode.AVERAGE ) out.coadd(oldOut,Fits.AVG);
+               else if( coaddMode==Mode.ADD 
+                     || coaddMode==Mode.SUM ) out.coadd(oldOut,Fits.ADD);
                else if( coaddMode==Mode.OVERWRITE ) out.mergeOnNaN(oldOut);
                else if( coaddMode==Mode.KEEP ) {
                   oldOut.mergeOnNaN(out);
