@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -29,6 +29,7 @@ import cds.aladin.Aladin;
 import cds.aladin.Calib;
 import cds.aladin.Coord;
 import cds.fits.Fits;
+import cds.fits.HeaderFits;
 import cds.moc.Healpix;
 import cds.moc.SMoc;
 import cds.tools.Util;
@@ -38,6 +39,7 @@ import cds.tools.pixtools.CDSHealpix;
  * 
  * @author Pierre Fernique [CDS]
  * @version 1.0 - may 2013
+ * @version 1.1 - may 2020 - ajout paramtres pour mots cls FITS additionnels
  *
  */
 public class MocGen {
@@ -54,6 +56,7 @@ public class MocGen {
    private boolean recursive=false;
    private boolean multWrite=false; // true for updating output MOC continously
    private int hdu[] = null;
+   private ArrayList <Prop> prop;         // Property list
    
    private SMoc moc;      // Le MOC en cour de calcul
    private boolean ready;       // true si le MOC est prt
@@ -64,9 +67,22 @@ public class MocGen {
    private int nbImg;           // Le nombre d'images sources
    private long tStart=0;       // Date de lancement
    
+   private class Prop {
+      String key,value,comment;
+      Prop(String s) throws Exception {
+         HeaderFits h = new HeaderFits();
+         h.readFreeHeader(s);
+         key     = h.getKeys().nextElement();
+         if( key==null ) throw new Exception("addprop error => ignored ["+s+"]");
+         value   = h.getStringFromHeader(key);
+         comment = h.getDescriptionFromHeader(key);
+      }
+   }
+   
    /** Gnration d'un MOC  partir d'une ligne de commande */
    public MocGen(String [] args) {
       super();
+      prop = new ArrayList<>();
       execute(args);
    }
    
@@ -78,6 +94,7 @@ public class MocGen {
       this.recursive=recursive;
       this.order=order;
       this.hdu=hdu;
+      prop = new ArrayList<>();
    }
    
    /** Retourne true si le MOC est prt - il faut tester isError() pour vrifier
@@ -116,9 +133,9 @@ public class MocGen {
                nbImg=0;
                moc = new SMoc();
                moc.setMocOrder(order);
-               moc.setCheckConsistencyFlag(false);
+               moc.bufferOn();
                scanAndDo(moc,new File(in),order);
-               moc.checkAndFix();
+               moc.bufferOff();
                ready=true;
             } catch( Exception e ) {
 //               System.out.println("Moc Exception => "+e.getMessage());
@@ -181,7 +198,6 @@ public class MocGen {
                      f.ext = ext;
                      String currentCell = f.getCellSuffix();
                      rep |= addInMocPixel1(f, moc, currentfile, currentCell, order);
-                     moc.checkAndFix();
                   }
 //               }
             } catch (Exception e) {
@@ -248,7 +264,6 @@ public class MocGen {
                n++;
                if( n>100000 ) {
                   n=0;
-                  moc.checkAndFix();
                   updateMoc(moc,out,fmt);
                }
                rep=true;
@@ -368,10 +383,7 @@ public class MocGen {
          if( abort ) throw new Exception("MOC aborted");
          if( f.isFile() ) {
             if( addInMoc(moc,f,order,strict) ) nbImg++;
-            if( nbImg>0 && nbImg%100==0 ) {
-               moc.checkAndFix();
-               updateMoc(moc,out,fmt);
-            }
+            if( nbImg>0 && nbImg%100==0 ) updateMoc(moc,out,fmt);
          }
       }
       if( recursive ) {
@@ -401,13 +413,14 @@ public class MocGen {
       if( t2>0 && t-t2>60000 ) {
          if( needNL ) System.out.println();
          String s = nbImg+1>1 ? "s":"";
-         System.out.println((nbImg+1)+" image"+s+" in progress (MOC size="+Util.getUnitDisk(moc.getMem())+" in "+Util.getTemps(t-tStart)+")...");
+         System.out.println((nbImg+1)+" image"+s+" in progress (MOC size="+Util.getUnitDisk(moc.getMem())+" in "+Util.getTemps((t-tStart)*1000L)+")...");
          needNL=false;
          t2=t;
       }
 
-      if( !multWrite || moc.getSize()==0 ) return false;
+      if( !multWrite || moc.isEmpty() ) return false;
       if( verbose ) System.out.println("Updating output MOC ["+out+"]...");
+      moc.flush();
       moc.write(out, fmt);
       return true;
    }
@@ -466,6 +479,11 @@ public class MocGen {
                System.out.println("Unavailable HDU numbers ["+s.substring(x)+"]");
                return false;
             }
+         
+         } else if( s.startsWith("addprop=") ) {
+            try { prop.add( new Prop(s.substring(x) ) ); } catch( Exception e ) { 
+               System.out.println("Unavailable addprop syntax => ignored ["+s.substring(x)+"]");
+            }
             
          } else if( s.equals("-v") || s.equals("-verbose") ) {
             verbose=true;
@@ -551,6 +569,7 @@ public class MocGen {
             "[mocfmt=fits|json] : MOC output format (default Fits)\n" +
       		"[previous=moc.fits]: Previous MOC (if additions)\n" +
       		"[hdu=n1,n2-n3|all] : List of concerned Fits extensions\n" +
+      		"[addprop=\"cle=value[ / comment]\" : Additionnal property stored in the target MOC file header fits file\n" +
             "[-r]               : Recursive directory scanning\n" +
             "[-o]               : Output MOC updated continuously rather than generated at the end\n" +
             "in=fileOrDir       : Directory of images/headers collection\n" +
@@ -564,7 +583,7 @@ public class MocGen {
       		"(WCS header in the comment segment), .hhh file (FITS header files without pixels)\n" +
       		"and .txt simple ASCII file (FITS header as keyword = value basic ASCII lines).\n" +
       		"\n" +
-      		"Version: 1.5 - based on Aladin "+Aladin.VERSION+" - Oct 2015 - P.Fernique [CDS]");
+      		"Version: 1.6 - based on Aladin "+Aladin.VERSION+" - May 2020 - P.Fernique [CDS]");
    }
    
    // Generation d'un MOC pour toute une hirarchie de fichiers FITS (ou JPEG/PNG avec calibration)
@@ -583,21 +602,27 @@ public class MocGen {
          nbImg=0;
          moc = new SMoc();
          if( previous!=null ) moc.read(previous);
+         
+         // Ajout des proprits additionnelles
+         for( Prop p : prop ) {
+            moc.setProperty(p.key,p.value,p.comment);
+         }
+         
          moc.setMocOrder(order);
-         moc.setCheckConsistencyFlag(false);
          tStart = System.currentTimeMillis();
+         moc.bufferOn();
 //         if( in!=null ) scanAndDo(moc,new File(in),order);
 //         else scanStdin(moc,order);
          scanAndDo(moc,new File(in),order);
-         moc.checkAndFix();
+         moc.bufferOff();
          long ms = System.currentTimeMillis()-tStart;
          if( needNL ) System.out.println();
          if( verbose ) {
             String s = nbImg>1?"s":"";
-            System.out.println(nbImg+" image"+s+" added in the MOC in "+cds.tools.Util.getTemps(ms));
+            System.out.println(nbImg+" image"+s+" added in the MOC in "+cds.tools.Util.getTemps(ms*1000L));
          }
          moc.write(out,fmt);
-         System.out.println("MOC achieved in "+Util.getTemps(ms)+" => "+out);
+         System.out.println("MOC achieved in "+Util.getTemps(ms*1000L)+" => "+out);
          ready=true;
       } catch( Exception e ) {
          moc=null;
