@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -28,6 +28,9 @@ public enum Mode {
    OVERWRITE,     // si la nouvelle valeur du pixel est != BLANK, alors remplace la valeur existante
    AVERAGE,       // Effectue la moyenne de la valeur existante avec la nouvelle valeur
    ADD,           // Effectue la somme de la valeur existante avec la nouvelle valeur
+   SUM,           // Effectue une somme (CONCAT action et TILES actions)
+   MUL,           // Effectue une multiplication (CONCAT action)
+   DIV,           // Effectue une division (CONCAT action)
 
    // La mthode utilis pour coadder agit au niveau des tuiles HEALPix
    REPLACETILE,    // Recalcule toutes les tuiles (de niveau le plus bas)
@@ -46,6 +49,9 @@ public enum Mode {
       if( m==OVERWRITE )   return m+": "+"Replace existing pixel values if the new value is not BLANK";
       if( m==AVERAGE )     return m+": "+"Compute the weighted average value based on the new pixel value and the existing one";
       if( m==ADD )         return m+": "+"Compute the sum value of new new pixel value and the existing one";
+      if( m==SUM )         return m+": "+"Compute the sum value (any superposition)";
+      if( m==MUL )         return m+": "+"Multiply the values (any superposition)";
+      if( m==DIV )         return m+": "+"Divide the values (any superposition)";
       if( m==REPLACETILE ) return m+": "+"Add new tiles, and if necessary, replace existing tiles (low level tiles)";
       if( m==KEEPTILE )    return m+": "+"Add new tiles but only for those not yet computed (low level tiles)";
       if( m==LINK )        return m+": "+"Composite cube tiles based on symbolic links on original tiles";
