@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -472,7 +472,9 @@ public class TabDesc extends JPanel implements ActionListener {
    public String getInputField() { return inputField.getText(); }
 
    public String getOutputField() { return outputField.getText(); }
-   public void setOutputField(String output) { outputField.setText(output); }
+   public void setOutputField(String output) { 
+      outputField.setText(output); 
+   }
 
    public Mode getCoaddModeField() {
       return resetTiles.isSelected() ? Mode.REPLACETILE : 
@@ -584,7 +586,7 @@ public class TabDesc extends JPanel implements ActionListener {
       // enlve les multiples FS  la fin
       while (path.endsWith(Util.FS)) path = path.substring(0, path.lastIndexOf(Util.FS));
 
-      outputField.setText(path + Constante.HIPS + Util.FS);
+      outputField.setText(path + Constante.HIPS); //+ Util.FS);
    }
 
    public boolean isResetTiles() {
