@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -629,7 +629,7 @@ public class TabRgb extends JPanel implements ActionListener {
          for( Object p : x ) {
             if( ((PlanBG)p).isColored() ) continue;
             else if( !((PlanBG)p).canbeTruePixels() ) continue;
-            else if( !Aladin.PROTO && !((PlanBG)p).isLocalAllSky() ) continue;
+            else if( !Aladin.BETA && !((PlanBG)p).isLocalAllSky() ) continue;
             pi[i++]=(PlanBG)p;
          }
          
