@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -103,6 +103,11 @@ final public class ThreadBuilderTile {
       }
       fading = context.getFading();
       borderSize = context.getBorderSize();
+      
+      // Ajustement pour viter le problme des pixels inconnues sur les bords (algo bilinaire)
+      if( borderSize[0]<1 ) borderSize[0]=1;
+      if( borderSize[1]<1 ) borderSize[1]=1;
+      
 //      circle = context.circle;
       globalShape=context.globalShape;
       hpxFinderPath = context.getHpxFinderPath();
@@ -240,7 +245,8 @@ final public class ThreadBuilderTile {
 
             // poids dj calculs
             double [] weight = null;
-            double [] fWeight = coaddMode==Mode.ADD ? null : new double[tileSide*tileSide];
+            double [] fWeight = coaddMode==Mode.ADD || coaddMode==Mode.SUM  ? null 
+                  : new double[tileSide*tileSide];
 
             for( int deb=0; deb<n; deb+=Constante.MAXOVERLAY ) {
                int fin = deb+Constante.MAXOVERLAY;
@@ -255,9 +261,10 @@ final public class ThreadBuilderTile {
                   if( out==null ) {
                      out=f;
                      weight=fWeight;
-                     fWeight = coaddMode==Mode.ADD ? null : new double[tileSide*tileSide];
+                     fWeight = coaddMode==Mode.ADD || coaddMode==Mode.SUM  ? null 
+                           : new double[tileSide*tileSide];
                   } else {
-                     if( coaddMode==Mode.ADD ) out.coadd(f,false);
+                     if( coaddMode==Mode.ADD || coaddMode==Mode.SUM ) out.coadd(f,Fits.ADD);
                      else out.coadd(f,weight,fWeight);
                   }
                }
@@ -332,6 +339,7 @@ final public class ThreadBuilderTile {
             fits.setPixelDouble(x,y,weight[i++]);
          }
       }
+      fits.addDataSum();
       fits.writeFITS(file+"_w.fits");
    }
 
@@ -551,7 +559,7 @@ final public class ThreadBuilderTile {
                      if( totalCoef==0 )  pixelFinal = 0xFF000000 | (((int)pixval[0] & 0xFF)<<16) | (((int)pixvalG[0] & 0xFF)<<8) | ((int)pixvalB[0] & 0xFF);
 
                      // Addition simple
-                     else if( coaddMode==Mode.ADD ) {
+                     else if( coaddMode==Mode.ADD || coaddMode==Mode.SUM ) {
                         double r=0,g=0,b=0;
                         for( int i=0; i<nbPix; i++ ) {
                            r += pixval[i];
@@ -591,7 +599,14 @@ final public class ThreadBuilderTile {
                   if( nbPix==0 ) pixelFinal = Double.NaN;
 
                   // Mode ADD simple
+                  else if( coaddMode==Mode.SUM ) {
+                     empty=false;
+                     for( int i=0; i<nbPix; i++ ) {
+                        pixelFinal += pixval[i];
+                     }
+                  }
                   else if( coaddMode==Mode.ADD ) {
+                  
                      empty=false;
                      for( int i=0; i<nbPix; i++ ) {
                         if( pixelFinal/2. + pixval[i]/2 > max/2. ) { pixelFinal=max; break; }
