@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -167,7 +167,7 @@ public class CircularRegion extends Region {
 
     	double[] v = new double[3];
     	Qbox.ucorners(abox.qbox, u4);
-	if (DEBUG) System.out.println(Coo.toString("#corners ", u4));
+	if (DEBUG) for(i=0; i<4; i++) AstroMath.printMatrix("#corners#" + i + " ", u4[i]);
     	u4[4][0] = u4[0][0];
     	u4[4][1] = u4[0][1];
     	u4[4][2] = u4[0][2];
