@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.fits;
@@ -64,8 +64,8 @@ public class CacheFits {
    protected HashMap<String, FitsFile> map;             // Table des fichiers
    //   private TreeMap<String,FitsFile> sortedMap;        // Table tri par ordre de dernier accs
    Context context;
-   private Hashtable<String, double[]> cutCache = new Hashtable<String, double[]>();
-   private Hashtable<String, double[]> shapeCache = new Hashtable<String, double[]>();
+   private Hashtable<String, double[]> cutCache = new Hashtable<>();
+   private Hashtable<String, double[]> shapeCache = new Hashtable<>();
 
 
    //private boolean skyvalSub = false;// condition d'application d'une soustraction du skyval au moment
@@ -89,7 +89,7 @@ public class CacheFits {
       cacheOutOfMem = maxMem==0;
       nextId = 0;
       statNbFree = statNbOpen = statNbFind = 0;
-      map = new HashMap<String, FitsFile>(MAXFILE+MAXFILE/100);
+      map = new HashMap<>(MAXFILE+MAXFILE/100);
       //      sortedMap = new TreeMap<String, FitsFile>( new ValueComparator(map) );
    }
 
@@ -156,7 +156,7 @@ public class CacheFits {
    public Fits getFits(String fileName,int mode,boolean flagLoad,boolean keepHeader) throws Exception,MyInputStreamCachedException {
       if( cacheOutOfMem )  return open(fileName,mode,flagLoad,keepHeader).fits;
 
-//      synchronized( lockObj  ) {
+      synchronized( lockObj  ) {
          FitsFile f = find(fileName);
 
          // Trouv, je le mets  jour
@@ -171,14 +171,14 @@ public class CacheFits {
             try {
                f=add(fileName,mode,flagLoad,keepHeader);
             } catch( OutOfMemoryError e ) {
-               System.err.println("CacheFits.getFits("+fileName+") out of memory... clean and try again...");
+               System.out.println("CacheFits.getFits("+fileName+") out of memory... clean and try again...");
                if( maxMem<0 ) maxMem*=2;
                else maxMem /= 2;
                try {
                   clean();
                   f=add(fileName,mode,flagLoad,keepHeader);
                } catch( OutOfMemoryError e1 ) {
-                  System.err.println("CacheFits.getFits("+fileName+") out of memory... double error... removing the cache...");
+                  System.out.println("CacheFits.getFits("+fileName+") out of memory... double error... removing the cache...");
                   e1.printStackTrace();
                   reset();
                   cacheOutOfMem=true;
@@ -189,7 +189,7 @@ public class CacheFits {
          }
 
          return f.fits;
-//      }
+      }
    }
 
    // Retrouve l'objet Fits dans le cache, null si inconnu
@@ -228,6 +228,7 @@ public class CacheFits {
       FitsFile f = new FitsFile();
       f.fits = new Fits();
       if( context!=null && context.skyvalName!=null ) { flagLoad=true; f.fits.setReleasable(false); }
+      if( context!=null && context.getBlankKey()!=null ) f.fits.setBlank(context.getBlankKey());
 
       // Il faut lire deux fichiers, le HHH, puis le JPEG ou PNG, voire FITS suivant le cas
       if( (mode&HHH)!=0 ) {
@@ -285,16 +286,20 @@ public class CacheFits {
                   } catch( Exception e) {
                      context.warning("All original data sets do no used the same CRPIX3,CRVAL3 & CDELT3 factors => factors ignored");
                      context.crpix3=context.crval3=context.cdelt3=0;
-                     context.bunit3=null;
+                     context.cunit3=null;
                   }
                }
             }
          }
       }
-
+      
       // applique un filtre spcial
-      if ( context!=null && (context.skyvalName!=null || context.expTimeName!=null
-            || context.pixelGood!=null || flagChangeOrig || context.dataArea!=Constante.SHAPE_UNKNOWN)) delSkyval(f.fits,flagChangeOrig);
+      if ( context!=null && (context.skyvalName!=null 
+            || context.expTimeName!=null
+            || context.pixelGood!=null || flagChangeOrig 
+            || context.dataArea!=Constante.SHAPE_UNKNOWN )) {
+         applyPostFilter(f.fits,flagChangeOrig);
+      }
 
       // On ne conserve pas le HeaderFits
       if( !keepHeader ) f.fits.freeHeader();
@@ -364,8 +369,8 @@ public class CacheFits {
          try {
             cacheOutOfMem=true;
 
-            HashMap<String,String> libere = new  HashMap<String,String>(map.size());
-            HashMap<String,FitsFile> map1 = new  HashMap<String,FitsFile>(map.size());
+            HashMap<String,String> libere = new  HashMap<>(map.size());
+            HashMap<String,FitsFile> map1 = new  HashMap<>(map.size());
 
             // en premier tour, on supprime les fits non utiliss
             // depuis plus de 5s, et si pas assez de mmoire, on ne regarde plus la date
@@ -424,7 +429,7 @@ public class CacheFits {
          String s1 = i>1 ? "s":"";
          long freeRam = getFreeMem();
          if( context!=null ) {
-            context.stat("Cache: freeRAM="+Util.getUnitDisk(freeMem)+" => "+nb+" files released ("+Util.getUnitDisk(totMem)+") in "+i+" step"+s1+" in "+Util.getTemps(duree)
+            context.stat("Cache: freeRAM="+Util.getUnitDisk(freeMem)+" => "+nb+" files released ("+Util.getUnitDisk(totMem)+") in "+i+" step"+s1+" in "+Util.getTemps(duree*1000L)
             +" => freeRAM="+Util.getUnitDisk(freeRam));
          }
       }
@@ -435,7 +440,7 @@ public class CacheFits {
       statNbFree+=map.size();
       
       if( this instanceof CacheFitsWriter ) {
-         ArrayList<String> a = new ArrayList<String>(map.size());
+         ArrayList<String> a = new ArrayList<>(map.size());
          for( String key: map.keySet() ) a.add(key);
          for( String key: a ) {
             try { remove(key); } catch( Exception e1 ) { }
@@ -574,13 +579,14 @@ public class CacheFits {
 
 
    private boolean first=true;
-
+   
+   
    /**
-    * Applique un filtre (soustraction du skyval, division par le expTime)
+    * Applique un filtre (soustraction du skyval, division par le expTime, masque, Lupton, ...)
     * sur les pixels avant de les mettre dans le cache
     * @param f fitsfile
     */
-   private void delSkyval(Fits f,boolean flagChangeOrig) {
+   private void applyPostFilter(Fits f,boolean flagChangeOrig) {
       double skyval = 0;
       double expTime = 1;
       //      double newval = f.blank;
@@ -591,7 +597,7 @@ public class CacheFits {
       boolean flagAuto=true;
       double pourcentMin = context.pourcentMin;
       double pourcentMax = context.pourcentMax;
-
+      
       // Faut-il retrancher le fond du ciel, et par quelle mthode ?
       if( context.skyvalName!=null ) {
 
@@ -604,15 +610,16 @@ public class CacheFits {
             } else {
                try {
                   skyval = f.headerFits.getDoubleFromHeader(context.skyvalName);
-                  skyval = (skyval - context.bZeroOrig)/context.bScaleOrig;
                   double cutOrig [] = context.getCutOrig();
-                  skyval = skyval - cutOrig[0];
+                  
+                  double refSkyVal = cutOrig[0]*context.bScaleOrig+context.bZeroOrig;
+                  skyval -= refSkyVal;
+                  skyval = (skyval - f.bzero)/f.bscale;
                   flagAuto=false;
                } catch( Exception e ) {
                   double cut [] = findAutocutRange(f,pourcentMin,pourcentMax);
                   double cutOrig [] = context.getCutOrig();
                   skyval = cut[0] - cutOrig[0];
-                  //                skyval = cut[0];
                   if( first ) {
                      context.warning("\nSKYVAL="+context.skyvalName+" not found is some images => use an estimation for these images");
                      first=false;
@@ -636,10 +643,11 @@ public class CacheFits {
          } catch (NullPointerException e) { }
       }
 
-      if( !skyValTag && !expTimeTag && !flagChangeOrig && context.pixelGood==null && shape==null ) return;
-
-      Aladin.trace(4,"SkyVal="+skyval+(flagAuto?"( estimation)":"( from header)")+" => "+f.getFileNameExtended());
+      // Pas besoin d'aller plus loin
+      if( !skyValTag && !expTimeTag && !flagChangeOrig 
+            && context.pixelGood==null && shape==null ) return;
 
+      if( skyValTag ) Aladin.trace(4,"SkyVal="+skyval+(flagAuto?"( estimation)":"( from header)")+" => "+f.getFileNameExtended());
 
       double blank = context.hasAlternateBlank() ? context.getBlankOrig() : f.blank;
       double a2=0,b2=0;
@@ -688,18 +696,19 @@ public class CacheFits {
 
                if( skyValTag  ) pixelFull -= skyval;
                if( expTimeTag ) pixelFull /= expTime;
-
+               
                if( flagChangeOrig ) {
                   pixelFull = pixelFull*f.bscale + f.bzero;
                   pixelFull = (pixelFull - context.bZeroOrig) / context.bScaleOrig;
                }
-
+               
                if( f.bitpix<0 ) f.setPixelDouble(x+f.xCell, y+f.yCell, z+f.zCell, pixelFull);
                else f.setPixelInt(x+f.xCell, y+f.yCell, z+f.zCell, (int)(pixelFull+0.5));
             }
          }
       }
    }
+   
 
    static double obscale=-1;
 
