@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -364,7 +364,7 @@ public final class HeaderFits {
     * @param c caractre  rechercher
     * @return position de c, ou de \n ou de finLigne
     */
-   private int getPos(char buf[],int from,int finLigne,char c) {
+   static private int getPos(char buf[],int from,int finLigne,char c) {
       int max = buf.length;
       boolean inQuote= from+2<buf.length && buf[from+2]=='\'' && c=='/';
       int deb=from;
@@ -375,7 +375,7 @@ public final class HeaderFits {
       }
       return from;
    }
-
+   
    /** Lecture d'une entte Fits quelque soit sa structure. Soit des lignes ASCII spares
     * par des \n, soit des lignes de 80 caractres sans \n. Le = n'est pas obligatoirement
     * en 8me position.
@@ -530,6 +530,7 @@ public final class HeaderFits {
    /** Surcharge ou ajout d'un mot cl */
    public void setKeyword(String key,String value,String description) {
       header.put(key,value);
+      if( !keysOrder.contains(key) ) keysOrder.add(key);
       headDescr.put(key,description);
    }
 
@@ -626,6 +627,17 @@ public final class HeaderFits {
       if( !hasKey(key) ) keysOrder.addElement(key);
       header.put(key, value);
    }
+   
+   /** Retourne le DATASUM de l'entte si prsent */
+   public String getDataSum() { return (String)header.get("DATASUM"); }
+   
+   /** Mmorise le DATASUM et la date associe
+    * @param dataSum  Le DATASUM sous la forme d'une chaine quivalente  UINT32
+    * @param dateISO le commentaire correspondant  la date ISO de gnration de ce DATASUM
+    */
+   public void addDataSum(String dataSum, String dateISO) {
+      setKeyword("DATASUM",dataSum,dateISO);
+   }
 
    /** Ecriture de l'entte FITS des mots cls mmoriss. L'ordre est conserv
     * comme  l'origine - les commentaires sont restitus 
@@ -649,21 +661,6 @@ public final class HeaderFits {
       n+=b.length;
       os.write(buf);
       return n;
-      
-//      int n=0;
-//      Enumeration e = keysOrder.elements();
-//      while( e.hasMoreElements() ) {
-//         String key = (String)e.nextElement();
-//         String value = (String) header.get(key);
-//         if( value==null ) continue;
-//         os.write( getFitsLine(key,value) );
-//         n+=80;
-//      }
-//      byte [] b= getEndBourrage(n);
-//      n+=b.length;
-//      os.write(b);
-//      return n;
-
    }
 
    /** Gnration de la fin de l'entte FITS, cd le END et le byte de bourrage
@@ -706,7 +703,7 @@ public final class HeaderFits {
         a = value.toCharArray();
 
         // Valeur numrique => alignement  droite
-        if( !isFitsString(value) ) {
+        if( !isFitsString(value) && !key.equals("DATASUM") ) {
            for( j=0; j<20-a.length; j++)  b[i++]=(byte)' ';
            for( j=0; i<80 && j<a.length; j++,i++) b[i]=(byte)a[j];
 
@@ -741,7 +738,7 @@ public final class HeaderFits {
      if( s.length()==0 ) return true;
      char c = s.charAt(0);
      if( s.length()==1 && (c=='T' || c=='F') ) return false;   // boolean
-     if( !Character.isDigit(c) && c!='.' && c!='-' && c!='+' ) return true;
+     if( !Character.isDigit(c) && c!='.' && c!='-' && c!='+' && c!='E' && c!='e' ) return true;
      try {
         Double.valueOf(s);
         return false;
