@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -21,6 +21,8 @@
 
 package cds.tools;
 
+import java.util.Locale;
+
 import org.jastronomy.jsofa.JSOFA;
 
 import cds.aladin.Tok;
@@ -42,27 +44,121 @@ public class Astrodate {
    static public double MJDToJD(double MJD) { return MJD+2400000.5; }
    
    
-   /** Conversion d'une date ISOTIME (UTC) en jour julien */
-   static public double dateToJD(String date) {
-      
-      // Vrifie que a a tout de mme un peu la tte d'une ISOTIME AAA-MM-DD[THH:MM:SS]
-      // On renvoit une exception NumberFormat pour viter d'avoir  patcher le code partout
+   /** Conversion d'une date ISOTIME (UTC) en jour julien
+    * Rq: Reconnaissance stricte de la syntaxe suivante uniquement
+    *  ISOTIME1 => YYYY-MM-DD[THH:MM[:SS.SS]]...
+    */
+   static public double ISOToJD(String date) throws Exception {
       int i = date.indexOf('-');
-      if( i<0 ) throw new NumberFormatException();
-      i = date.indexOf('-',i+1);
-      if( i<0 ) throw new NumberFormatException();
+      if( i<0 ) throw new Exception("Not ISO date");
       
-      Tok tok = new Tok(date,"-T:");
+      Tok tok = new Tok(date,"-T");
       double A = Double.parseDouble( tok.nextToken() );
       double M = Double.parseDouble( tok.nextToken() );
       double J = Double.parseDouble( tok.nextToken() );
+      
       double HH=0,MM=0,SS=0;
       if( tok.hasMoreTokens() ) HH = Double.parseDouble( tok.nextToken() );
       if( tok.hasMoreTokens() ) MM = Double.parseDouble( tok.nextToken() );
       if( tok.hasMoreTokens() ) SS = Double.parseDouble( tok.nextToken() );
-      return dateToJD(A,M,J,HH,MM,SS);
+      
+      double res = dateToJD(A,M,J,HH,MM,SS);
+      return res;
    }
    
+   /** Conversion d'une date ISOTIME (UTC) en jour julien
+    * Tolre quelques autres syntaxes
+    *  ISOTIME1 => YYYY-MM-DD[THH:MM[:SS.SS]]...
+    *  ISOTIME2 => YYYYMMDD[THHMM[SS.SS]]...
+    *  DATE => DD/MM/YYYY... (ventuellement)
+    *  L'anne peut tre sur 2 digits (=> +1900) sauf pour ISOTIME2
+    */
+   static public double dateToJD(String date) {
+      boolean slash=false;
+      
+      int i = date.indexOf('-');
+      if( i<0 ) { i = date.indexOf('/'); slash=true; }
+      if( i<0 ) i = date.indexOf(' ');
+      
+      // aucun sparateur, donc YYYYMMDD[THHMMSS]... => conversion en YYYY-MM-DD...
+      if( i<0 ) {
+         String yyyy,mm,dd,hh,m,ss;
+         yyyy=mm=dd=hh=m=ss=null;
+         yyyy = date.substring(0,4);
+         mm = date.substring(4,6);
+         dd = date.substring(6,8);
+         int offsetT = date.indexOf('T');
+         if( offsetT>0 ) {
+            hh = date.substring(9,11);
+            m = date.substring(11,13);
+            if( date.length()>13 ) ss=date.substring(13);
+         }
+         date=yyyy+"-"+mm+"-"+dd;
+         if( hh!=null ) {
+            date+="T"+hh+":"+m;
+            if( ss!=null ) date+=":"+ss;
+         }
+      }
+      
+      // On supprime un ventuel suffixe Z, (TT) ...
+      int suffix = date.length();
+      for( suffix--; suffix>0; suffix--) {
+         if( Character.isDigit( date.charAt(suffix) ) ) { suffix++; break; }
+      }
+      if( suffix>0 ) date = date.substring(0,suffix);
+      
+      Tok tok = new Tok(date,"-/T: ");
+      double A = Double.parseDouble( tok.nextToken() );
+      double M = Double.parseDouble( tok.nextToken() );
+      double J = Double.parseDouble( tok.nextToken() );
+      
+      // HORREUR POUR SDSS, ON SUPPoSE QU'ACEC DES SLASHS LE FORMAT EST DD/MM/AA
+      if( slash ) { double x = A; A=J; J=x; }
+      
+      double HH=0,MM=0,SS=0;
+      if( tok.hasMoreTokens() ) HH = Double.parseDouble( tok.nextToken() );
+      if( tok.hasMoreTokens() ) MM = Double.parseDouble( tok.nextToken() );
+      if( tok.hasMoreTokens() ) SS = Double.parseDouble( tok.nextToken() );
+      
+      // Cas d'une anne sur deux digits
+      if( A<100 ) A+=1900;
+      
+      double res = dateToJD(A,M,J,HH,MM,SS);
+      return res;
+   }
+   
+//   static public void main(String [] a) {
+//      String s;
+//      s="1967-05-09T10:39:10.1Z";
+//      double b = dateToJD(s);
+//      System.out.println(s+" => "+b +" => "+JDToDate(b));
+//      s="67-05-09T10:39:10.1";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="19670509T103910.1";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="1967/05/09 10:39:10.1";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="67/5/9";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="1967 05 09";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="1967-05-09";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="1967/05/09";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//      s="1967-05-09T10:39";
+//      b = dateToJD(s);
+//      System.out.println(s+" => "+b+" => "+JDToDate(b));
+//     
+//   }
+   
    /** Conversion d'une date en jour julien
     * @param A anne
     * @param M mois (janvier = 1)
@@ -87,12 +183,13 @@ public class Astrodate {
    }
    
    /**
-    * Conversion d'une date en jour julien en AAAA-MM-JJTHH:MM:SS (UTC)
+    * Conversion d'une date en jour julien en AAAA-MM-JJTHH:MM:SS[.SSS] (UTC)
     * @param JD Jour Julien
     * @return
     */
    static public String JDToDate(double JD) { return JDToDate(JD,true); }
-   static public String JDToDate(double JD, boolean withTime) {
+   static public String JDToDate(double JD, boolean withTime) { return JDToDate(JD,withTime,true); }
+   static public String JDToDate(double JD, boolean withTime, boolean flagDecimalSecond) {
       long Z = (long)(JD+0.5);
       double F = JD+0.5 - Z;
       long G;
@@ -111,10 +208,15 @@ public class Astrodate {
       long M = E<13.5?E-1:E-13;
       long A = (M<2.5)?C-4715:C-4716;
       long s = (long)( (j-J)*86400 );
+      double ss = (j-J)*86400.;
       long HH = s/3600;
       long MM = (s  - HH*3600 )/60;
       long SS = (s - HH*3600 - MM*60);
-      return A+"-"+dd(M)+"-"+dd(J)+ (withTime ? "T"+dd(HH)+":"+dd(MM)+":"+dd(SS) : "");      
+      double SSf = (ss - HH*3600 - MM*60);
+      if( !withTime || !flagDecimalSecond || SSf-(int)SSf < 0.001 ) 
+         return A+"-"+dd(M)+"-"+dd(J)+ (withTime ? "T"+dd(HH)+":"+dd(MM)+":"+dd(SS) : "");
+      String s1 = String.format(Locale.ROOT,"%02.3f",SSf);
+      return A+"-"+dd(M)+"-"+dd(J)+ "T"+dd(HH)+":"+dd(MM)+":"+s1;
    }
    
    // Ajout d'un zro en primer digit si ncessaire
@@ -153,8 +255,23 @@ public class Astrodate {
    static public final int YEARS   = 16; 
    static public final int DATE    = 17; 
    static public final int UNIX    = 18; 
+   static public final int BES     = 19; 
    
-   /** Conversion d'une chaine en JD */
+   /** Conversion JD en String */
+   static public String editTime( double jd, int timeMode ) {
+      try {
+         if( timeMode==JD )      return jd+"";
+         if( timeMode==MJD )     return JDToMJD(jd)+"";
+         if( timeMode==ISOTIME ) return JDToDate(jd);
+         if( timeMode==YEARS )   return JDToYd(jd)+"";
+         if( timeMode==UNIX )    return JDToUnix(jd)+"";
+         if( timeMode==BES )     return JSOFA.jauEpb(jd,0.)+"";
+         
+      } catch( Exception e ) { }
+      return null;
+   }
+   
+   /** Conversion d'une String en JD */
    static public double parseTime( String date, int timeMode ) {
       try {
          if( timeMode==JD )      return Double.parseDouble(date);
@@ -162,6 +279,7 @@ public class Astrodate {
          if( timeMode==ISOTIME ) return dateToJD( date );
          if( timeMode==YEARS )   return YdToJD( Double.parseDouble(date) );
          if( timeMode==UNIX )    return UnixToJD( Long.parseLong(date) );
+         if( timeMode==BES )     return getSofaJD( JSOFA.jauEpb2jd( Double.parseDouble(date) ) );
          
          // Date format inconnue
          return MJDToJD( Util.ISOToMJD( Util.parseDate( date ) ) );
@@ -216,7 +334,8 @@ public class Astrodate {
       }
 
       if( timeScale.equals("TT") ) {
-         if( deltaT==-1 ) throw new Exception("Astrodate convertion not possible - missing Solar ephemerids");
+         if( dtr==-1 ) throw new Exception("Astrodate convertion not possible - missing Solar ephemerids");
+         
          jdTime = getSofaJD( JSOFA.jauTttdb(jdTime,0,dtr) );
          timeScale="TDB"; 
       }
