@@ -1,5 +1,5 @@
-// Copyright 1999-2020 - Universit de Strasbourg/CNRS
-// The Aladin Desktop program is developped by the Centre de Donnes
+// Copyright 1999-2022 - Universite de Strasbourg/CNRS
+// The Aladin Desktop program is developped by the Centre de Donnees
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
@@ -186,7 +186,7 @@ public final class CDSHealpix {
          else System.err.println("  Executing cp.overlappingCenters("+Math.toRadians(coo[0])+", "+Math.toRadians(coo[1])+")");
          e.printStackTrace();
       }
-      return bmoc2moc( bmoc);
+      return bmoc2moc( bmoc, order);
    }
    
    static SMoc getMocByPolygon(int order,ArrayList<double[]>cooList, boolean inclusive) throws Exception {
@@ -198,8 +198,9 @@ public final class CDSHealpix {
          vertices[i][0] = Math.toRadians(vertices[i][0]);
          vertices[i][1] = Math.toRadians(vertices[i][1]);
       }
+      
       final HealpixNestedBMOC bmoc = inclusive ? pc.overlappingCells(vertices) : pc.overlappingCenters(vertices);
-      return bmoc2moc( bmoc);
+      return bmoc2moc( bmoc, order);
    }
    
    static public SMoc getMocByEllipse(int order, double ra, double dec, double a, double b, double pa) throws Exception  {
@@ -212,12 +213,12 @@ public final class CDSHealpix {
       final NestedEllipticalConeComputerApprox cp 
       = new NestedEllipticalConeComputerApprox( aRad, bRad, paRad, Healpix.getNested(order));
       final HealpixNestedBMOC bmoc = cp.overlapping(lonRad, latRad, Mode.OVERLAPPING_CELLS);
-      return bmoc2moc( bmoc);
+      return bmoc2moc( bmoc, order);
    }
    
    /** Conversion d'un BMOC a la FX en Moc  la Fernique via les Range */
-   static public SMoc bmoc2moc(HealpixNestedBMOC bmoc) throws Exception {
-      int depthMax = SMoc.MAXORDER;
+   static public SMoc bmoc2moc(HealpixNestedBMOC bmoc, int order) throws Exception {
+      int depthMax = SMoc.MAXORD_S;
       Range range = new Range();
       
       Iterator<CurrentValueAccessor> it = bmoc.iterator();
@@ -229,13 +230,11 @@ public final class CDSHealpix {
          long hash = c.getHash();
          long a = hash << twiceDepthDiff;
          long b = a | ((1L << twiceDepthDiff) - 1L);
-         range.add(a, b);
+         range.add(a, b+1);
       }
       
-      int mocOrder = bmoc.getDepthMax();
-      SMoc moc = new SMoc(mocOrder);
-      moc.range = range;
-      moc.toMocSet();
+      SMoc moc = new SMoc(order);
+      moc.setRangeList( range );
       return moc;
 
    }
@@ -444,8 +443,9 @@ public final class CDSHealpix {
     * @return
     * @throws Exception
     */
-   static public SMoc createSMoc(ArrayList<double[]> radecList, int order ) throws Exception {
-      return getMocByPolygon( order, radecList, true);
+   static public SMoc createSMoc(ArrayList<double[]> radecList, int order ) throws Exception { return createSMoc(radecList,order,true); }
+   static public SMoc createSMoc(ArrayList<double[]> radecList, int order, boolean inclusive ) throws Exception {
+      return getMocByPolygon( order, radecList, inclusive);
    }
 
 //   static public SMoc createSMoc(ArrayList<double[]> radecList, int order ) throws Exception {
